@@ -149,21 +149,21 @@ class _AngleParser(object):
 
         def p_colon(p):
             '''
-            colon : sign UINT COLON UINT
+            colon : sign UINT COLON ufloat
                   | sign UINT COLON UINT COLON ufloat
             '''
             if len(p) == 5:
-                p[0] = (p[1] * p[2], p[4], 0.0)
+                p[0] = (p[1] * p[2], p[4])
             elif len(p) == 7:
                 p[0] = (p[1] * p[2], p[4], p[6])
 
         def p_spaced(p):
             '''
-            spaced : sign UINT UINT
+            spaced : sign UINT ufloat
                    | sign UINT UINT ufloat
             '''
             if len(p) == 4:
-                p[0] = (p[1] * p[2], p[3], 0.0)
+                p[0] = (p[1] * p[2], p[3])
             elif len(p) == 5:
                 p[0] = (p[1] * p[2], p[3], p[4])
 
@@ -182,8 +182,9 @@ class _AngleParser(object):
         def p_hms(p):
             '''
             hms : sign UINT HOUR
-                | sign UINT HOUR UINT
+                | sign UINT HOUR ufloat
                 | sign UINT HOUR UINT MINUTE
+                | sign UINT HOUR UFLOAT MINUTE
                 | sign UINT HOUR UINT MINUTE ufloat
                 | sign UINT HOUR UINT MINUTE ufloat SECOND
                 | generic HOUR
@@ -193,15 +194,16 @@ class _AngleParser(object):
             elif len(p) == 4:
                 p[0] = (p[1] * p[2], u.hourangle)
             elif len(p) in (5, 6):
-                p[0] = ((p[1] * p[2], p[4], 0.0), u.hourangle)
+                p[0] = ((p[1] * p[2], p[4]), u.hourangle)
             elif len(p) in (7, 8):
                 p[0] = ((p[1] * p[2], p[4], p[6]), u.hourangle)
 
         def p_dms(p):
             '''
             dms : sign UINT DEGREE
-                | sign UINT DEGREE UINT
+                | sign UINT DEGREE ufloat
                 | sign UINT DEGREE UINT MINUTE
+                | sign UINT DEGREE UFLOAT MINUTE
                 | sign UINT DEGREE UINT MINUTE ufloat
                 | sign UINT DEGREE UINT MINUTE ufloat SECOND
                 | generic DEGREE
@@ -211,7 +213,7 @@ class _AngleParser(object):
             elif len(p) == 4:
                 p[0] = (p[1] * p[2], u.degree)
             elif len(p) in (5, 6):
-                p[0] = ((p[1] * p[2], p[4], 0.0), u.degree)
+                p[0] = ((p[1] * p[2], p[4]), u.degree)
             elif len(p) in (7, 8):
                 p[0] = ((p[1] * p[2], p[4], p[6]), u.degree)
 
@@ -297,6 +299,8 @@ def _check_second_range(sec):
     """
     if np.any(sec == 60.):
         warn(IllegalSecondWarning(sec, 'Treating as 0 sec, +1 min'))
+    elif sec is None:
+        pass
     elif np.any(sec < -60.) or np.any(sec > 60.):
         # "Error: seconds not in range [-60,60) ({0}).".format(sec))
         raise IllegalSecondError(sec)
@@ -361,7 +365,7 @@ def degrees_to_dms(d):
     return np.floor(sign * d), sign * np.floor(m), sign * s
 
 
-def dms_to_degrees(d, m, s):
+def dms_to_degrees(d, m, s=None):
     """
     Convert degrees, arcminute, arcsecond to a float degrees value.
     """
@@ -372,13 +376,14 @@ def dms_to_degrees(d, m, s):
     # determine sign
     sign = np.copysign(1.0, d)
 
-    # TODO: This will fail if d or m have values after the decimal
-    # place
-
     try:
-        d = np.floor(np.abs(np.asarray(d)))
-        m = np.floor(np.abs(np.asarray(m)))
-        s = np.abs(s)
+        d = np.floor(np.abs(d))
+        if s is None:
+            m = np.abs(m)
+            s = 0
+        else:
+            m = np.floor(np.abs(m))
+            s = np.abs(s)
     except ValueError:
         raise ValueError(format_exception(
             "{func}: dms values ({1[0]},{2[1]},{3[2]}) could not be "
@@ -387,7 +392,7 @@ def dms_to_degrees(d, m, s):
     return sign * (d + m / 60. + s / 3600.)
 
 
-def hms_to_hours(h, m, s):
+def hms_to_hours(h, m, s=None):
     """
     Convert hour, minute, second to a float hour value.
     """
@@ -397,13 +402,14 @@ def hms_to_hours(h, m, s):
     # determine sign
     sign = np.copysign(1.0, h)
 
-    # TODO: This will fail if d or m have values after the decimal
-    # place
-
     try:
         h = np.floor(np.abs(h))
-        m = np.floor(np.abs(m))
-        s = np.abs(s)
+        if s is None:
+            m = np.abs(m)
+            s = 0
+        else:
+            m = np.floor(np.abs(m))
+            s = np.abs(s)
     except ValueError:
         raise ValueError(format_exception(
             "{func}: HMS values ({1[0]},{2[1]},{3[2]}) could not be "
@@ -634,7 +640,7 @@ def angular_separation(lon1, lat1, lon2, lat2):
     Notes
     -----
     The angular separation is calculated using the Vincenty formula [1]_,
-    which is slighly more complex and computationally expensive than
+    which is slightly more complex and computationally expensive than
     some alternatives, but is stable at at all distances, including the
     poles and antipodes.
 
