@@ -1,6 +1,8 @@
 from __future__ import (absolute_import, division, print_function, unicode_literals)
 
+import re
 import collections
+import warnings
 
 import numpy as np
 
@@ -9,6 +11,8 @@ from ..extern import six
 from ..extern.six.moves import zip
 from ..units import Unit, IrreducibleUnit
 from .. import units as u
+from ..wcs.utils import skycoord_to_pixel, pixel_to_skycoord
+from ..utils.exceptions import AstropyDeprecationWarning
 
 from .distances import Distance
 from .baseframe import BaseCoordinateFrame, frame_transform_graph, GenericFrame, _get_repr_cls
@@ -18,6 +22,13 @@ from .representation import (BaseRepresentation, SphericalRepresentation,
 
 __all__ = ['SkyCoord']
 
+PLUS_MINUS_RE = re.compile(r'(\+|\-)')
+J_PREFIXED_RA_DEC_RE = re.compile(
+    r"""J                              # J prefix
+    ([0-9]{6,7}\.?[0-9]{0,2})          # RA as HHMMSS.ss or DDDMMSS.ss, optional decimal digits
+    ([\+\-][0-9]{6}\.?[0-9]{0,2})\s*$  # Dec as DDMMSS.ss, optional decimal digits
+    """, re.VERBOSE)
+
 
 # Define a convenience mapping.  This is used like a module constants
 # but is actually dynamically evaluated.
@@ -34,11 +45,12 @@ class SkyCoord(object):
     """High-level object providing a flexible interface for celestial coordinate
     representation, manipulation, and transformation between systems.
 
-    The `SkyCoord` class accepts a wide variety of inputs for initialization.
-    At a minimum these must provide one or more celestial coordinate values
-    with unambiguous units.  Typically one also specifies the coordinate
-    frame, though this is not required.  The general pattern is for spherical
-    representations is::
+    The `SkyCoord` class accepts a wide variety of inputs for initialization. At
+    a minimum these must provide one or more celestial coordinate values with
+    unambiguous units.  Inputs may be scalars or lists/tuples/arrays, yielding
+    scalar or array coordinates (can be checked via ``SkyCoord.isscalar``).
+    Typically one also specifies the coordinate frame, though this is not
+    required. The general pattern for spherical representations is::
 
       SkyCoord(COORD, [FRAME], keyword_args ...)
       SkyCoord(LON, LAT, [FRAME], keyword_args ...)
@@ -83,6 +95,8 @@ class SkyCoord(object):
 
       >>> c = SkyCoord(w=0, u=1, v=2, unit='kpc', frame='galactic', representation='cartesian')
 
+      >>> c = SkyCoord([ICRS(ra=1*u.deg, dec=2*u.deg), ICRS(ra=3*u.deg, dec=4*u.deg)])
+
     As shown, the frame can be a `~astropy.coordinates.BaseCoordinateFrame`
     class or the corresponding string alias.  The frame classes that are built in
     to astropy are `ICRS`, `FK5`, `FK4`, `FK4NoETerms`, and `Galactic`.
@@ -125,6 +139,11 @@ class SkyCoord(object):
             Cartesian coordinates values for the Galactic frame.
     """
 
+
+    # Declare that SkyCoord can be used as a Table column by defining the
+    # attribute where column attributes will be stored.
+    _astropy_column_attrs = None
+
     def __init__(self, *args, **kwargs):
 
         # Parse the args and kwargs to assemble a sanitized and validated
@@ -223,7 +242,7 @@ class SkyCoord(object):
                 # coord_kwargs will contain keys like 'ra', 'dec', 'distance'
                 # along with any frame attributes like equinox or obstime which
                 # were explicitly specified in the coordinate object (i.e. non-default).
-                coord_kwargs = _parse_coordinate_arg(args[0], frame, units)
+                coord_kwargs = _parse_coordinate_arg(args[0], frame, units, kwargs)
 
             elif len(args) <= 3:
                 frame_attr_names = frame.representation_component_names.keys()
@@ -244,7 +263,7 @@ class SkyCoord(object):
             for attr, coord_value in coord_kwargs.items():
                 if (attr in valid_kwargs
                         and valid_kwargs[attr] is not None
-                        and valid_kwargs[attr] != coord_value):
+                        and np.any(valid_kwargs[attr] != coord_value)):
                     raise ValueError("Coordinate attribute '{0}'={1!r} conflicts with "
                                      "keyword argument '{0}'={2!r}"
                                      .format(attr, coord_value, valid_kwargs[attr]))
@@ -256,6 +275,14 @@ class SkyCoord(object):
         """
         Transform this coordinate to a new frame.
 
+        The frame attributes (e.g. equinox or obstime) for the returned object
+        depend on the corresponding attributes of SkyCoord object and the
+        supplied ``frame``, with the following precedence:
+
+        1. Non-default value in the supplied frame
+        2. Non-default value in the SkyCoord instance
+        3. Default value in the supplied frame
+
         Parameters
         ----------
         frame : str or `BaseCoordinateFrame` class / instance or `SkyCoord` instance
@@ -288,17 +315,20 @@ class SkyCoord(object):
             new_frame_cls = frame.__class__
 
             # Set the keyword args for making a new frame instance for the
-            # transform.  If the supplied frame instance has a non-default
-            # value set then use that, otherwise use the self attribute value
-            # if it is not None.
+            # transform.  Frame attributes track whether they were explicitly
+            # set by user or are just reflecting default values.  Precedence:
+            # 1. Non-default value in the supplied frame instance
+            # 2. Non-default value in the self instance
+            # 3. Default value in the supplied frame instance
             for attr in FRAME_ATTR_NAMES_SET():
                 self_val = getattr(self, attr, None)
                 frame_val = getattr(frame, attr, None)
-                if (frame_val is not None and
-                        attr not in frame._attr_names_with_defaults):
+                if frame_val is not None and not frame.is_frame_attr_default(attr):
                     frame_kwargs[attr] = frame_val
-                elif self_val is not None:
+                elif self_val is not None and not self.is_frame_attr_default(attr):
                     frame_kwargs[attr] = self_val
+                elif frame_val is not None:
+                    frame_kwargs[attr] = frame_val
         else:
             raise ValueError('Transform `frame` must be a frame name, class, or instance')
 
@@ -328,33 +358,51 @@ class SkyCoord(object):
         Overrides getattr to return coordinates that this can be transformed
         to, based on the alias attr in the master transform graph.
         """
+        if '_sky_coord_frame' in self.__dict__:
+            if self.frame.name == attr:
+                return self  # Should this be a deepcopy of self?
 
-        if self.frame.name == attr:
-            return self  # Should this be a deepcopy of self?
+            # Anything in the set of all possible frame_attr_names is handled
+            # here. If the attr is relevant for the current frame then delegate
+            # to self.frame otherwise get it from self._<attr>.
+            if attr in FRAME_ATTR_NAMES_SET():
+                if attr in self.frame.get_frame_attr_names():
+                    return getattr(self.frame, attr)
+                else:
+                    return getattr(self, '_' + attr)
 
-        # Anything in the set of all possible frame_attr_names is handled
-        # here. If the attr is relevant for the current frame then delegate
-        # to self.frame otherwise get it from self._<attr>.
-        if attr in FRAME_ATTR_NAMES_SET():
-            if attr in self.frame.get_frame_attr_names():
-                return getattr(self.frame, attr)
-            else:
-                return getattr(self, '_' + attr)
+            # Some attributes might not fall in the above category but still
+            # are available through self._sky_coord_frame.
+            if not attr.startswith('_') and hasattr(self._sky_coord_frame, attr):
+                return getattr(self._sky_coord_frame, attr)
 
-        # Some attributes might not fall in the above category but still
-        # are available through self._sky_coord_frame.
-        if not attr.startswith('_') and hasattr(self._sky_coord_frame, attr):
-            return getattr(self._sky_coord_frame, attr)
-
-        # Try to interpret as a new frame for transforming.
-        frame_cls = frame_transform_graph.lookup_name(attr)
-        if frame_cls is not None and self.frame.is_transformable_to(frame_cls):
-            return self.transform_to(attr)
+            # Try to interpret as a new frame for transforming.
+            frame_cls = frame_transform_graph.lookup_name(attr)
+            if frame_cls is not None and self.frame.is_transformable_to(frame_cls):
+                return self.transform_to(attr)
 
         # Fail
         raise AttributeError("'{0}' object has no attribute '{1}'"
                              .format(self.__class__.__name__, attr))
 
+    def __setattr__(self, attr, val):
+        # This is to make anything available through __getattr__ immutable
+        if '_sky_coord_frame' in self.__dict__:
+            if self.frame.name == attr:
+                raise AttributeError("'{0}' is immutable".format(attr))
+
+            if (attr in FRAME_ATTR_NAMES_SET() or
+                (not attr.startswith('_') and
+                 hasattr(self._sky_coord_frame, attr))):
+                setattr(self._sky_coord_frame, attr, val)
+
+            frame_cls = frame_transform_graph.lookup_name(attr)
+            if frame_cls is not None and self.frame.is_transformable_to(frame_cls):
+                raise AttributeError("'{0}' is immutable".format(attr))
+
+        # Otherwise, do the standard Python attribute setting
+        super(SkyCoord, self).__setattr__(attr, val)
+
     @override__dir__
     def __dir__(self):
         """
@@ -382,8 +430,12 @@ class SkyCoord(object):
         clsnm = self.__class__.__name__
         coonm = self.frame.__class__.__name__
 
-        s = '<{clsnm} ({coonm})'.format(**locals())
         crepr = repr(self.frame)
+        frameattrs = ''
+        if crepr.find('):') != -1:
+            frameattrs = ': '+crepr[crepr.index('(')+1:crepr.index('):')]
+
+        s = '<{clsnm} ({coonm}{frameattrs})'.format(**locals())
         return s + crepr[crepr.index(':'):]
 
     def to_string(self, style='decimal', **kwargs):
@@ -442,18 +494,63 @@ class SkyCoord(object):
                             sph_coord.lat.to_string(**latargs))
         else:
             coord_string = []
-            for lonangle, latangle in zip(sph_coord.lon, sph_coord.lat):
+            for lonangle, latangle in zip(sph_coord.lon.ravel(), sph_coord.lat.ravel()):
                 coord_string += [(lonangle.to_string(**lonargs)
                                  + " " +
                                  latangle.to_string(**latargs))]
+            if len(sph_coord.shape) > 1:
+                coord_string = np.array(coord_string).reshape(sph_coord.shape)
 
         return coord_string
 
-    # High-level convinience methods
+    def is_equivalent_frame(self, other):
+        """
+        Checks if this object's frame as the same as that of the ``other``
+        object.
+
+        To be the same frame, two objects must be the same frame class and have
+        the same frame attributes. For two `SkyCoord` objects, *all* of the
+        frame attributes have to match, not just those relevant for the object's
+        frame.
+
+        Parameters
+        ----------
+        other : SkyCoord or BaseCoordinateFrame
+            The other object to check.
+
+        Returns
+        -------
+        isequiv : bool
+            True if the frames are the same, False if not.
+
+        Raises
+        ------
+        TypeError
+            If ``other`` isn't a `SkyCoord` or a `BaseCoordinateFrame` or subclass.
+        """
+        if isinstance(other, BaseCoordinateFrame):
+            return self.frame.is_equivalent_frame(other)
+        elif isinstance(other, SkyCoord):
+            if other.frame.name != self.frame.name:
+                return False
+
+            for fattrnm in FRAME_ATTR_NAMES_SET():
+                if getattr(self, fattrnm) != getattr(other, fattrnm):
+                    return False
+            return True
+        else:
+            #not a BaseCoordinateFrame nor a SkyCoord object
+            raise TypeError("Tried to do is_equivalent_frame on something that "
+                            "isn't frame-like")
+
+    # High-level convenience methods
     def separation(self, other):
         """
         Computes on-sky separation between this coordinate and another.
 
+        For more on how to use this (and related) functionality, see the
+        examples in :doc:`/coordinates/matchsep`.
+
         Parameters
         ----------
         other : `~astropy.coordinates.SkyCoord` or `~astropy.coordinates.BaseCoordinateFrame`
@@ -498,6 +595,9 @@ class SkyCoord(object):
         Computes three dimensional separation between this coordinate
         and another.
 
+        For more on how to use this (and related) functionality, see the
+        examples in :doc:`/coordinates/matchsep`.
+
         Parameters
         ----------
         other : `~astropy.coordinates.SkyCoord` or `~astropy.coordinates.BaseCoordinateFrame`
@@ -542,6 +642,9 @@ class SkyCoord(object):
         Finds the nearest on-sky matches of this coordinate in a set of
         catalog coordinates.
 
+        For more on how to use this (and related) functionality, see the
+        examples in :doc:`/coordinates/matchsep`.
+
         Parameters
         ----------
         catalogcoord : `~astropy.coordinates.SkyCoord` or `~astropy.coordinates.BaseCoordinateFrame`
@@ -579,6 +682,7 @@ class SkyCoord(object):
         See Also
         --------
         astropy.coordinates.match_coordinates_sky
+        SkyCoord.match_to_catalog_3d
         """
         from .matching import match_coordinates_sky
 
@@ -603,6 +707,9 @@ class SkyCoord(object):
         from the on-sky distance if ``distance`` is set in this object or the
         ``catalogcoord`` object.
 
+        For more on how to use this (and related) functionality, see the
+        examples in :doc:`/coordinates/matchsep`.
+
         Parameters
         ----------
         catalogcoord : `~astropy.coordinates.SkyCoord` or `~astropy.coordinates.BaseCoordinateFrame`
@@ -640,6 +747,7 @@ class SkyCoord(object):
         See Also
         --------
         astropy.coordinates.match_coordinates_3d
+        SkyCoord.match_to_catalog_sky
         """
         from .matching import match_coordinates_3d
 
@@ -656,6 +764,122 @@ class SkyCoord(object):
 
         return res
 
+    def search_around_sky(self, searcharoundcoords, seplimit):
+        """
+        Searches for all coordinates in this object around a supplied set of
+        points within a given on-sky separation.
+
+        This is inteded for use on `~astropy.coordinates.SkyCoord` objects
+        with coordinate arrays, rather than a scalar coordinate.  For a scalar
+        coordinate, it is better to use
+        `~astropy.coordinates.SkyCoord.separation`.
+
+        For more on how to use this (and related) functionality, see the
+        examples in :doc:`/coordinates/matchsep`.
+
+        Parameters
+        ----------
+        searcharoundcoords : `~astropy.coordinates.SkyCoord` or `~astropy.coordinates.BaseCoordinateFrame`
+            The coordinates to search around to try to find matching points in
+            this `SkyCoord`. This should be an object with array coordinates,
+            not a scalar coordinate object.
+        seplimit : `~astropy.units.Quantity` with angle units
+            The on-sky separation to search within.
+
+        Returns
+        -------
+        idxsearcharound : integer array
+            Indices into ``self`` that matches to the corresponding element of
+            ``idxself``. Shape matches ``idxself``.
+        idxself : integer array
+            Indices into ``searcharoundcoords`` that matches to the
+            corresponding element of ``idxsearcharound``. Shape matches
+            ``idxsearcharound``.
+        sep2d : `~astropy.coordinates.Angle`
+            The on-sky separation between the coordinates. Shape matches
+            ``idxsearcharound`` and ``idxself``.
+        dist3d : `~astropy.units.Quantity`
+            The 3D distance between the coordinates. Shape matches
+            ``idxsearcharound`` and ``idxself``.
+
+        Notes
+        -----
+        This method requires `SciPy <http://www.scipy.org>`_ (>=0.12.0) to be
+        installed or it will fail.
+
+        In the current implementation, the return values are always sorted in
+        the same order as the ``searcharoundcoords`` (so ``idxsearcharound`` is
+        in ascending order).  This is considered an implementation detail,
+        though, so it could change in a future release.
+
+        See Also
+        --------
+        astropy.coordinates.search_around_sky
+        SkyCoord.search_around_3d
+        """
+        from .matching import search_around_sky
+
+        return search_around_sky(searcharoundcoords, self, seplimit,
+                                 storekdtree='_kdtree_sky')
+
+    def search_around_3d(self, searcharoundcoords, distlimit):
+        """
+        Searches for all coordinates in this object around a supplied set of
+        points within a given 3D radius.
+
+        This is inteded for use on `~astropy.coordinates.SkyCoord` objects
+        with coordinate arrays, rather than a scalar coordinate.  For a scalar
+        coordinate, it is better to use
+        `~astropy.coordinates.SkyCoord.separation_3d`.
+
+        For more on how to use this (and related) functionality, see the
+        examples in :doc:`/coordinates/matchsep`.
+
+        Parameters
+        ----------
+        searcharoundcoords : `~astropy.coordinates.SkyCoord` or `~astropy.coordinates.BaseCoordinateFrame`
+            The coordinates to search around to try to find matching points in
+            this `SkyCoord`. This should be an object with array coordinates,
+            not a scalar coordinate object.
+        distlimit : `~astropy.units.Quantity` with distance units
+            The physical radius to search within.
+
+        Returns
+        -------
+        idxsearcharound : integer array
+            Indices into ``self`` that matches to the corresponding element of
+            ``idxself``. Shape matches ``idxself``.
+        idxself : integer array
+            Indices into ``searcharoundcoords`` that matches to the
+            corresponding element of ``idxsearcharound``. Shape matches
+            ``idxsearcharound``.
+        sep2d : `~astropy.coordinates.Angle`
+            The on-sky separation between the coordinates. Shape matches
+            ``idxsearcharound`` and ``idxself``.
+        dist3d : `~astropy.units.Quantity`
+            The 3D distance between the coordinates. Shape matches
+            ``idxsearcharound`` and ``idxself``.
+
+        Notes
+        -----
+        This method requires `SciPy <http://www.scipy.org>`_ (>=0.12.0) to be
+        installed or it will fail.
+
+        In the current implementation, the return values are always sorted in
+        the same order as the ``searcharoundcoords`` (so ``idxsearcharound`` is
+        in ascending order).  This is considered an implementation detail,
+        though, so it could change in a future release.
+
+        See Also
+        --------
+        astropy.coordinates.search_around_3d
+        SkyCoord.search_around_sky
+        """
+        from .matching import search_around_3d
+
+        return search_around_3d(searcharoundcoords, self, distlimit,
+                                storekdtree='_kdtree_3d')
+
     def position_angle(self, other):
         """
         Computes the on-sky position angle (East of North) between this
@@ -700,6 +924,141 @@ class SkyCoord(object):
 
         return angle_utilities.position_angle(slon, slat, olon, olat)
 
+    # WCS pixel to/from sky conversions
+    def to_pixel(self, wcs, origin=0, mode='all'):
+        """
+        Convert this coordinate to pixel coordinates using a `~astropy.wcs.WCS`
+        object.
+
+        Parameters
+        ----------
+        wcs : `~astropy.wcs.WCS`
+            The WCS to use for convert
+        origin : int
+            Whether to return 0 or 1-based pixel coordinates.
+        mode : 'all' or 'wcs'
+            Whether to do the transformation including distortions (``'all'``) or
+            only including only the core WCS transformation (``'wcs'``).
+
+        Returns
+        -------
+        xp, yp : `numpy.ndarray`
+            The pixel coordinates
+
+        See Also
+        --------
+        from_pixel : to do the inverse operation
+        astropy.wcs.utils.skycoord_to_pixel : the implementation of this method
+        """
+        return skycoord_to_pixel(self, wcs=wcs, origin=origin, mode=mode)
+
+    @classmethod
+    def from_pixel(cls, xp, yp, wcs, origin=0, mode='all'):
+        """
+        Create a new `SkyCoord` from pixel coordinates using an
+        `~astropy.wcs.WCS` object.
+
+        Parameters
+        ----------
+        xp, yp : float or `numpy.ndarray`
+            The coordinates to convert.
+        wcs : `~astropy.wcs.WCS`
+            The WCS to use for convert
+        origin : int
+            Whether to return 0 or 1-based pixel coordinates.
+        mode : 'all' or 'wcs'
+            Whether to do the transformation including distortions (``'all'``) or
+            only including only the core WCS transformation (``'wcs'``).
+
+        Returns
+        -------
+        coord : an instance of this class
+            A new object with sky coordinates corresponding to the input ``xp``
+            and ``yp``.
+
+        See Also
+        --------
+        to_pixel : to do the inverse operation
+        astropy.wcs.utils.pixel_to_skycoord : the implementation of this method
+        """
+        return pixel_to_skycoord(xp, yp, wcs=wcs, origin=origin, mode=mode, cls=cls)
+
+    # Table interactions
+    @classmethod
+    def guess_from_table(cls, table, **coord_kwargs):
+        """
+        A convenience method to create and return a new `SkyCoord` from the data
+        in an astropy Table.
+
+        This method matches table columns that start with the case-insensitive
+        names of the the components of the requested frames, if they are also
+        followed by a non-alphanumeric character. It will also match columns
+        that *end* with the component name if a non-alphanumeric character is
+        *before* it.
+
+        For example, the first rule means columns with names like
+        ``'RA[J2000]'`` or ``'ra'`` will be interpreted as ``ra`` attributes for
+        `~astropy.coordinates.ICRS` frames, but ``'RAJ2000'`` or ``'radius'``
+        are *not*. Similarly, the second rule applied to the
+        `~astropy.coordinates.Galactic` frame means that a column named
+        ``'gal_l'`` will be used as the the ``l`` component, but ``gall`` or
+        ``'fill'`` will not.
+
+        The definition of alphanumeric here is based on Unicode's definition
+        of alphanumeric, except without ``_`` (which is normally considered
+        alphanumeric).  So for ASCII, this means the non-alphanumeric characters
+        are ``<space>_!"#$%&'()*+,-./:;<=>?@[\]^`{|}~``).
+
+        Parameters
+        ----------
+        table : astropy.Table
+            The table to load data from.
+        coord_kwargs
+            Any additional keyword arguments are passed directly to this class's
+            constructor.
+
+        Returns
+        -------
+        newsc : same as this class
+            The new `SkyCoord` (or subclass) object.
+        """
+        inital_frame = coord_kwargs.get('frame')
+        frame = _get_frame([], coord_kwargs)
+        coord_kwargs['frame'] = inital_frame
+
+        comp_kwargs = {}
+        for comp_name in frame.representation_component_names:
+            # this matches things like 'ra[...]'' but *not* 'rad'.
+            # note that the "_" must be in there explicitly, because
+            # "alphanumeric" usually includes underscores.
+            starts_with_comp = comp_name + r'(\W|\b|_)'
+            # this part matches stuff like 'center_ra', but *not*
+            # 'aura'
+            ends_with_comp = r'.*(\W|\b|_)' + comp_name + r'\b'
+            #the final regex ORs together the two patterns
+            rex = re.compile('(' +starts_with_comp + ')|(' + ends_with_comp + ')',
+                             re.IGNORECASE | re.UNICODE)
+
+            for col_name in table.colnames:
+                if rex.match(col_name):
+                    if comp_name in comp_kwargs:
+                        oldname = comp_kwargs[comp_name].name
+                        msg = ('Found at least two matches for  component "{0}"'
+                               ': "{1}" and "{2}". Cannot continue with this '
+                               'ambiguity.')
+                        raise ValueError(msg.format(comp_name, oldname, col_name))
+                    comp_kwargs[comp_name] = table[col_name]
+
+        for k, v in comp_kwargs.items():
+            if k in coord_kwargs:
+                raise ValueError('Found column "{0}" in table, but it was '
+                                 'already provided as "{1}" keyword to '
+                                 'guess_from_table function.'.format(v.name, k))
+            else:
+                coord_kwargs[k] = v
+
+        return cls(**coord_kwargs)
+
     # Name resolve
     @classmethod
     def from_name(cls, name, frame='icrs'):
@@ -733,6 +1092,7 @@ class SkyCoord(object):
         else:
             return icrs_sky_coord.transform_to(frame)
 
+
 # <----------------Private utility functions below here------------------------->
 
 
@@ -772,6 +1132,34 @@ def _get_frame(args, kwargs):
     """
     frame = kwargs.pop('frame', None)
 
+    if frame is None and len(args) > 1:
+
+        # We do not allow frames to be passed as positional arguments if data
+        # is passed separately from frame.
+
+        for arg in args:
+
+            if isinstance(arg, (SkyCoord, BaseCoordinateFrame)):
+                raise ValueError("{0} instance cannot be passed as a positional "
+                                 "argument for the frame, pass it using the "
+                                 "frame= keyword instead.".format(arg.__class__.__name__))
+
+    # If the frame is an instance or SkyCoord, we split up the attributes and
+    # make it into a class.
+
+    if isinstance(frame, SkyCoord):
+        frame = frame.frame
+
+    if isinstance(frame, BaseCoordinateFrame):
+
+        for attr in frame.get_frame_attr_names():
+            if attr in kwargs:
+                raise ValueError("cannot specify frame attribute '{0}' directly in SkyCoord since a frame instance was passed in".format(attr))
+            else:
+                kwargs[attr] = getattr(frame, attr)
+
+        frame = frame.__class__
+
     if frame is not None:
         # Frame was provided as kwarg so validate and coerce into corresponding frame.
         frame_cls = _get_frame_class(frame)
@@ -786,6 +1174,9 @@ def _get_frame(args, kwargs):
                 pass
             else:
                 args.remove(arg)
+                warnings.warn("Passing a frame as a positional argument is now "
+                              "deprecated, use the frame= keyword argument "
+                              "instead.", AstropyDeprecationWarning)
                 break
         else:
             # Not in args nor kwargs - default to icrs
@@ -849,14 +1240,19 @@ def _get_units(args, kwargs):
     return units
 
 
-def _parse_coordinate_arg(coords, frame, units):
+def _parse_coordinate_arg(coords, frame, units, init_kwargs):
     """
     Single unnamed arg supplied.  This must be:
     - Coordinate frame with data
     - Representation
+    - SkyCoord
     - List or tuple of:
       - String which splits into two values
       - Iterable with two values
+      - SkyCoord, frame, or representation objects.
+
+    Returns a dict mapping coordinate attribute names to values (or lists of
+    values)
     """
     is_scalar = False  # Differentiate between scalar and list input
     valid_kwargs = {}  # Returned dict of lon, lat, and distance (optional)
@@ -907,46 +1303,81 @@ def _parse_coordinate_arg(coords, frame, units):
         values = coords.transpose()  # Iterates over repr attrs
 
     elif isinstance(coords, (collections.Sequence, np.ndarray)):
-        # Handles generic list-like input.
+        # Handles list-like input.
 
-        # First turn into a list of lists like [[v1_0, v2_0, v3_0], ... [v1_N, v2_N, v3_N]]
         vals = []
-        for ii, coord in enumerate(coords):
-            if isinstance(coord, six.string_types):
-                coord1 = coord.split()
-                if len(coord1) == 6:
-                    coord1 = (' '.join(coord1[:3]), ' '.join(coord1[3:]))
-                coord = coord1
+        is_ra_dec_representation = ('ra' in frame.representation_component_names and
+                                    'dec' in frame.representation_component_names)
+        coord_types = (SkyCoord, BaseCoordinateFrame, BaseRepresentation)
+        if any(isinstance(coord, coord_types) for coord in coords):
+            # this parsing path is used when there are coordinate-like objects
+            # in the list - instead of creating lists of values, we create
+            # SkyCoords from the list elements and then combine them.
+            scs = [SkyCoord(coord, **init_kwargs) for coord in coords]
 
-            vals.append(coord)  # This assumes coord is a sequence at this point
+            # now check that they're all self-consistent in their frames
+            for sc in scs[1:]:
+                if not sc.is_equivalent_frame(scs[0]):
+                        raise ValueError("List of inputs don't have equivalent "
+                                         "frames: {0} != {1}".format(sc, scs[0]))
 
-        # Do some basic validation of the list elements: all have a length and all
-        # lengths the same
-        try:
-            n_coords = sorted(set(len(x) for x in vals))
-        except:
-            raise ValueError('One or more elements of input sequence does not have a length')
+            # get the frame attributes from the first one, because from above we
+            # know it matches all the others
+            for fattrnm in FRAME_ATTR_NAMES_SET():
+                valid_kwargs[fattrnm] = getattr(scs[0], fattrnm)
 
-        if len(n_coords) > 1:
-            raise ValueError('Input coordinate values must have same number of elements, found {0}'
-                             .format(n_coords))
-        n_coords = n_coords[0]
+            # Now combine the values, to be used below
+            values = []
+            for data_attr_name in frame_attr_names:
+                data_vals = []
+                for sc in scs:
+                    data_val = getattr(sc, data_attr_name)
+                    data_vals.append(data_val.reshape(1,) if sc.isscalar else data_val)
+                concat_vals = np.concatenate(data_vals)
+                # Hack because np.concatenate doesn't fully work with Quantity
+                if isinstance(concat_vals, u.Quantity):
+                    concat_vals._unit = data_val.unit
+                values.append(concat_vals)
+        else:
+            #none of the elements are "frame-like"
+            #turn into a list of lists like [[v1_0, v2_0, v3_0], ... [v1_N, v2_N, v3_N]]
+            for coord in coords:
+                if isinstance(coord, six.string_types):
+                    coord1 = coord.split()
+                    if len(coord1) == 6:
+                        coord = (' '.join(coord1[:3]), ' '.join(coord1[3:]))
+                    elif is_ra_dec_representation:
+                        coord = _parse_ra_dec(coord)
+                    else:
+                        coord = coord1
+                vals.append(coord)  # Assumes coord is a sequence at this point
 
-        # Must have no more coord inputs than representation attributes
-        if n_coords > n_attr_names:
-            raise ValueError('Input coordinates have {0} values but {1} representation '
-                             'only accepts {2}'
-                             .format(n_coords, frame.representation.get_name(), n_attr_names))
+            # Do some basic validation of the list elements: all have a length and all
+            # lengths the same
+            try:
+                n_coords = sorted(set(len(x) for x in vals))
+            except:
+                raise ValueError('One or more elements of input sequence does not have a length')
 
-        # Now transpose vals to get [(v1_0 .. v1_N), (v2_0 .. v2_N), (v3_0 .. v3_N)]
-        # (ok since we know it is exactly rectangular).  (Note: can't just use zip(*values)
-        # because Longitude et al distinguishes list from tuple so [a1, a2, ..] is needed
-        # while (a1, a2, ..) doesn't work.
-        values = [list(x) for x in zip(*vals)]
+            if len(n_coords) > 1:
+                raise ValueError('Input coordinate values must have same number of elements, found {0}'
+                                 .format(n_coords))
+            n_coords = n_coords[0]
 
-        if is_scalar:
-            values = [x[0] for x in values]
+            # Must have no more coord inputs than representation attributes
+            if n_coords > n_attr_names:
+                raise ValueError('Input coordinates have {0} values but '
+                                 'representation {1} only accepts {2}'
+                                 .format(n_coords, frame.representation.get_name(), n_attr_names))
 
+            # Now transpose vals to get [(v1_0 .. v1_N), (v2_0 .. v2_N), (v3_0 .. v3_N)]
+            # (ok since we know it is exactly rectangular).  (Note: can't just use zip(*values)
+            # because Longitude et al distinguishes list from tuple so [a1, a2, ..] is needed
+            # while (a1, a2, ..) doesn't work.
+            values = [list(x) for x in zip(*vals)]
+
+            if is_scalar:
+                values = [x[0] for x in values]
     else:
         raise ValueError('Cannot parse coordinates from first argument')
 
@@ -958,9 +1389,8 @@ def _parse_coordinate_arg(coords, frame, units):
                 frame_attr_names, repr_attr_classes, values, units):
             valid_kwargs[frame_attr_name] = repr_attr_class(value, unit=unit)
     except Exception as err:
-        raise ValueError('Cannot parse longitude and latitude from first argument: {0}'
-                         .format(err))
-
+        raise ValueError('Cannot parse first argument data "{0}" for attribute '
+                         '{1}'.format(value, frame_attr_name), err)
     return valid_kwargs
 
 
@@ -984,3 +1414,60 @@ def _get_representation_attrs(frame, units, kwargs):
             valid_kwargs[frame_attr_name] = repr_attr_class(value, unit=unit)
 
     return valid_kwargs
+
+
+def _parse_ra_dec(coord_str):
+    """
+    Parse RA and Dec values from a coordinate string. Currently the
+    following formats are supported:
+
+     * space separated 6-value format
+     * space separated <6-value format, this requires a plus or minus sign
+       separation between RA and Dec
+     * sign separated format
+     * JHHMMSS.ss+DDMMSS.ss format, with up to two optional decimal digits
+     * JDDDMMSS.ss+DDMMSS.ss format, with up to two optional decimal digits
+
+    Parameters
+    ----------
+    coord_str : str
+        Coordinate string to parse.
+
+    Returns
+    -------
+    coord : str or list of str
+        Parsed coordinate values.
+    """
+
+    if isinstance(coord_str, six.string_types):
+        coord1 = coord_str.split()
+    else:
+        # This exception should never be raised from SkyCoord
+        raise TypeError('coord_str must be a single str')
+
+    if len(coord1) == 6:
+        coord = (' '.join(coord1[:3]), ' '.join(coord1[3:]))
+    elif len(coord1) > 2:
+        coord = PLUS_MINUS_RE.split(coord_str)
+        coord = (coord[0], ' '.join(coord[1:]))
+    elif len(coord1) == 1:
+        match_j = J_PREFIXED_RA_DEC_RE.match(coord_str)
+        if match_j:
+            coord = match_j.groups()
+            if len(coord[0].split('.')[0]) == 7:
+                coord = ('{0} {1} {2}'.
+                         format(coord[0][0:3], coord[0][3:5], coord[0][5:]),
+                         '{0} {1} {2}'.
+                         format(coord[1][0:3], coord[1][3:5], coord[1][5:]))
+            else:
+                coord = ('{0} {1} {2}'.
+                         format(coord[0][0:2], coord[0][2:4], coord[0][4:]),
+                         '{0} {1} {2}'.
+                         format(coord[1][0:3], coord[1][3:5], coord[1][5:]))
+        else:
+            coord = PLUS_MINUS_RE.split(coord_str)
+            coord = (coord[0], ' '.join(coord[1:]))
+    else:
+        coord = coord1
+
+    return coord
