@@ -251,7 +251,7 @@ def test_transform_api():
     fk5_J2001_frame = FK5(equinox=J2001)
 
     # if they do not have data, the string instead is the frame specification
-    assert repr(fk5_J2001_frame) == "<FK5 Frame: equinox=J2001.000>"
+    assert repr(fk5_J2001_frame) == "<FK5 Frame (equinox=J2001.000)>"
 
     #  Note that, although a frame object is immutable and can't have data added, it
     #  can be used to create a new object that does have data by giving the
@@ -358,9 +358,7 @@ def test_highlevel_api():
     #funny ways
     #assert repr(sc.frame) == '<ICRS Coordinate: ra=120.0 deg, dec=5.0 deg>'
     rscf = repr(sc.frame)
-    assert '<ICRS Coordinate: ra=' in rscf
-    assert 'deg, dec=' in rscf
-    assert 'deg>' in rscf
+    assert rscf.startswith('<ICRS Coordinate: (ra, dec) in deg')
 
     # and  the string representation will be inherited from the low-level class.
 
@@ -368,9 +366,7 @@ def test_highlevel_api():
     # versions may round the numbers differently
     #assert repr(sc) == '<SkyCoord (ICRS): ra=120.0 deg, dec=5.0 deg>'
     rsc = repr(sc)
-    assert '<SkyCoord (ICRS): ra=' in rsc
-    assert 'deg, dec=' in rsc
-    assert 'deg>' in rsc
+    assert rsc.startswith('<SkyCoord (ICRS): (ra, dec) in deg')
 
     # Supports a variety of possible complex string formats
     sc = coords.SkyCoord('8h00m00s +5d00m00.0s', frame='icrs')
@@ -413,7 +409,7 @@ def test_highlevel_api():
     # v0.2/0.3 coordinate objects.  This will *not* be in the low-level classes
     sc = coords.SkyCoord(ra=8 * u.hour, dec=5 * u.deg, frame='icrs')
     scgal = sc.galactic
-    assert str(scgal).startswith('<SkyCoord (Galactic): l=216.317')
+    assert str(scgal).startswith('<SkyCoord (Galactic): (l, b)')
 
     # the existing `from_name` and `match_to_catalog_*` methods will be moved to the
     # high-level class as convenience functionality.
@@ -438,7 +434,7 @@ def test_highlevel_api():
 def test_highlevel_api_remote():
     m31icrs = coords.SkyCoord.from_name('M31', frame='icrs')
 
-    assert str(m31icrs) == '<SkyCoord (ICRS): ra=10.6847083 deg, dec=41.26875 deg>'
+    assert str(m31icrs) == '<SkyCoord (ICRS): (ra, dec) in deg\n    (10.6847083, 41.26875)>'
 
     m31fk4 = coords.SkyCoord.from_name('M31', frame='fk4')
 
