@@ -17,6 +17,7 @@ import itertools
 import functools
 import numpy
 import warnings
+import copy
 
 from ...extern import six
 from ...extern.six.moves import zip
@@ -24,6 +25,7 @@ from ...extern.six.moves import cStringIO as StringIO
 from ...utils.exceptions import AstropyWarning
 
 from ...table import Table
+from ...table.column import col_getattr, col_setattr, col_iter_str_vals
 from ...utils.compat import ignored
 from ...utils.data import get_readable_fileobj
 from ...utils import OrderedDict
@@ -32,6 +34,9 @@ from . import connect
 # Global dictionary mapping format arg to the corresponding Reader class
 FORMAT_CLASSES = {}
 
+# Similar dictionary for fast readers
+FAST_CLASSES = {}
+
 class MaskedConstant(numpy.ma.core.MaskedConstant):
     """A trivial extension of numpy.ma.masked
 
@@ -64,6 +69,20 @@ class OptionalTableImportError(ImportError):
     an ImportError.
     """
 
+class ParameterError(NotImplementedError):
+    """
+    Indicates that a reader cannot handle a passed parameter.
+
+    The C-based fast readers in ``io.ascii`` raise an instance of
+    this error class upon encountering a parameter that the
+    C engine cannot handle.
+    """
+
+class FastOptionsError(NotImplementedError):
+    """
+    Indicates that one of the specified options for fast
+    reading is invalid.
+    """
 
 class NoType(object):
     """
@@ -92,6 +111,12 @@ class FloatType(NumType):
     """
 
 
+class BoolType(NoType):
+    """
+    Describes boolean data.
+    """
+
+
 class IntType(NumType):
     """
     Describes integer data.
@@ -114,31 +139,43 @@ class Column(object):
 
     * **name** : column name
     * **type** : column type (NoType, StrType, NumType, FloatType, IntType)
+    * **dtype** : numpy dtype (optional, overrides **type** if set)
     * **str_vals** : list of column values as strings
     * **data** : list of converted column values
     """
     def __init__(self, name):
         self.name = name
-        self.type = NoType
+        self.type = NoType  # Generic type (Int, Float, Str etc)
+        self.dtype = None  # Numpy dtype if available
         self.str_vals = []
         self.fill_values = {}
 
 
 class BaseInputter(object):
     """
-    Get the lines from the table input and return a list of lines.  The input
-    table can be one of:
+    Get the lines from the table input and return a list of lines.
 
-    * File name
-    * String (newline separated) with all header and data lines (must have at least 2 lines)
-    * File-like object with read() method
-    * List of strings
     """
     def get_lines(self, table):
-        """Get the lines from the ``table`` input.
+        """
+        Get the lines from the ``table`` input. The input table can be one of:
 
-        :param table: table input
-        :returns: list of lines
+        * File name
+        * String (newline separated) with all header and data lines (must have at least 2 lines)
+        * File-like object with read() method
+        * List of strings
+
+        Parameters
+        ----------
+        table : str, file_like, list
+            Can be either a file name, string (newline separated) with all header and data
+            lines (must have at least 2 lines), a file-like object with a ``read()`` method,
+            or a list of strings.
+
+        Returns
+        -------
+        lines : list
+            List of lines
         """
         try:
             if (hasattr(table, 'read') or
@@ -173,7 +210,8 @@ class BaseInputter(object):
 
 
 class BaseSplitter(object):
-    """Base splitter that uses python's split method to do the work.
+    """
+    Base splitter that uses python's split method to do the work.
 
     This does not handle quoted values.  A key feature is the formulation of
     __call__ as a generator that returns a list of the split line values at
@@ -189,9 +227,10 @@ class BaseSplitter(object):
       reader.header.splitter.process_val = lambda x: x.lstrip()
       reader.data.splitter.process_val = None
 
-    :param delimiter: one-character string used to separate fields
     """
+
     delimiter = None
+    """ one-character string used to separate fields """
 
     def process_line(self, line):
         """Remove whitespace at the beginning or end of line.  This is especially useful for
@@ -232,19 +271,22 @@ class DefaultSplitter(BaseSplitter):
           for col_val in col_vals:
                ...
 
-    :param delimiter: one-character string used to separate fields.
-    :param doublequote:  control how instances of *quotechar* in a field are quoted
-    :param escapechar: character to remove special meaning from following character
-    :param quotechar: one-character stringto quote fields containing special characters
-    :param quoting: control when quotes are recognised by the reader
-    :param skipinitialspace: ignore whitespace immediately following the delimiter
     """
     delimiter = ' '
+    """ one-character string used to separate fields. """
     quotechar = '"'
+    """ control how instances of *quotechar* in a field are quoted """
     doublequote = True
+    """ character to remove special meaning from following character """
     escapechar = None
+    """ one-character stringto quote fields containing special characters """
     quoting = csv.QUOTE_MINIMAL
+    """ control when quotes are recognised by the reader """
     skipinitialspace = True
+    """ ignore whitespace immediately following the delimiter """
+    csv_writer = None
+    csv_writer_out = StringIO()
+
 
     def process_line(self, line):
         """Remove whitespace at the beginning or end of line.  This is especially useful for
@@ -254,16 +296,20 @@ class DefaultSplitter(BaseSplitter):
             line = _replace_tab_with_space(line, self.escapechar, self.quotechar)
         return line.strip()
 
-    def __init__(self):
-        self.csv_writer = None
-        self.csv_writer_out = StringIO()
 
     def __call__(self, lines):
         """Return an iterator over the table ``lines``, where each iterator output
         is a list of the split line values.
 
-        :param lines: list of table lines
-        :returns: iterator
+        Parameters
+        ----------
+        lines : list
+            List of table lines
+
+        Returns
+        -------
+        lines : iterator
+
         """
         if self.process_line:
             lines = [self.process_line(x) for x in lines]
@@ -349,23 +395,24 @@ def _get_line_index(line_or_func, lines):
 
 
 class BaseHeader(object):
-    """Base table header reader
-
-    :param auto_format: format string for auto-generating column names
-    :param start_line: None, int, or a function of ``lines`` that returns None or int
-    :param comment: regular expression for comment lines
-    :param splitter_class: Splitter class for splitting data lines into columns
-    :param names: list of names corresponding to each data column
+    """
+    Base table header reader
     """
     auto_format = 'col%d'
+    """ format string for auto-generating column names """
     start_line = None
+    """ None, int, or a function of ``lines`` that returns None or int """
     comment = None
+    """ regular expression for comment lines """
     splitter_class = DefaultSplitter
+    """ Splitter class for splitting data lines into columns """
     names = None
+    """ list of names corresponding to each data column """
+    write_comment = False
     write_spacer_lines = ['ASCII_TABLE_WRITE_SPACER_LINE']
 
     def __init__(self):
-        self.splitter = self.__class__.splitter_class()
+        self.splitter = self.splitter_class()
 
     def _set_cols_from_names(self):
         self.cols = [Column(name=x) for x in self.names]
@@ -374,8 +421,17 @@ class BaseHeader(object):
         """
         Extract any table-level metadata, e.g. keywords, comments, column metadata, from
         the table ``lines`` and update the OrderedDict ``meta`` in place.  This base
-        method does nothing.
+        method extracts comment lines and stores them in ``meta`` for output.
         """
+        if self.comment:
+            re_comment = re.compile(self.comment)
+            comment_lines = [x for x in lines if re_comment.match(x)]
+        else:
+            comment_lines = []
+        comment_lines = [re.sub('^' + self.comment, '', x).strip()
+                         for x in comment_lines]
+        if comment_lines:
+            meta.setdefault('table', {})['comments'] = comment_lines
 
     def get_cols(self, lines):
         """Initialize the header Column objects from the table ``lines``.
@@ -383,8 +439,11 @@ class BaseHeader(object):
         Based on the previously set Header attributes find or create the column names.
         Sets ``self.cols`` with the list of Columns.
 
-        :param lines: list of table lines
-        :returns: None
+        Parameters
+        ----------
+        lines : list
+            List of table lines
+
         """
 
         start_line = _get_line_index(self.start_line, self.process_lines(lines))
@@ -419,17 +478,23 @@ class BaseHeader(object):
             if line and (not self.comment or not re_comment.match(line)):
                 yield line
 
+    def write_comments(self, lines, meta):
+        if self.write_comment is not False:
+            for comment in meta.get('comments', []):
+                lines.append(self.write_comment + comment)
+
     def write(self, lines):
         if self.start_line is not None:
             for i, spacer_line in zip(range(self.start_line),
                                       itertools.cycle(self.write_spacer_lines)):
                 lines.append(spacer_line)
-            lines.append(self.splitter.join([x.name for x in self.cols]))
+            lines.append(self.splitter.join([col_getattr(x, 'name') for x in self.cols]))
 
     @property
     def colnames(self):
         """Return the column names of the table"""
-        return tuple(col.name for col in self.cols)
+        return tuple(col.name if isinstance(col, Column) else col_getattr(col, 'name')
+                     for col in self.cols)
 
     def get_type_map_key(self, col):
         return col.raw_type
@@ -442,38 +507,85 @@ class BaseHeader(object):
             raise ValueError('Unknown data type ""%s"" for column "%s"' % (
                 col.raw_type, col.name))
 
+    def check_column_names(self, names, strict_names, guessing):
+        """
+        Check column names.
+
+        This must be done before applying the names transformation
+        so that guessing will fail appropriately if ``names`` is supplied.
+        For instance if the basic reader is given a table with no column header
+        row.
+
+        Parameters
+        ----------
+        names : list
+            User-supplied list of column names
+        strict_names : bool
+            Whether to impose extra requirements on names
+        guessing : bool
+            True if this method is being called while guessing the table format
+        """
+        if strict_names:
+            # Impose strict requirements on column names (normally used in guessing)
+            bads = [" ", ",", "|", "\t", "'", '"']
+            for name in self.colnames:
+                if (_is_number(name) or
+                    len(name) == 0 or
+                    name[0] in bads or
+                    name[-1] in bads):
+                    raise ValueError('Column name {0!r} does not meet strict name requirements'
+                                     .format(name))
+        # When guessing require at least two columns
+        if guessing and len(self.colnames) <= 1:
+            raise ValueError
+
+        if names is not None and len(names) != len(self.colnames):
+            raise ValueError('Length of names argument ({0}) does not match number'
+                             ' of table columns ({1})'.format(len(names), len(self.colnames)))
+
 
 class BaseData(object):
-    """Base table data reader.
-
-    :param start_line: None, int, or a function of ``lines`` that returns None or int
-    :param end_line: None, int, or a function of ``lines`` that returns None or int
-    :param comment: Regular expression for comment lines
-    :param splitter_class: Splitter class for splitting data lines into columns
+    """
+    Base table data reader.
     """
     start_line = None
+    """ None, int, or a function of ``lines`` that returns None or int """
     end_line = None
+    """ None, int, or a function of ``lines`` that returns None or int """
     comment = None
+    """ Regular expression for comment lines """
     splitter_class = DefaultSplitter
+    """ Splitter class for splitting data lines into columns """
     write_spacer_lines = ['ASCII_TABLE_WRITE_SPACER_LINE']
     fill_include_names = None
     fill_exclude_names = None
+    # Currently, the default matches the numpy default for masked values.
+    fill_values = [(masked, '--')]
+    formats = {}
 
     def __init__(self):
         # Need to make sure fill_values list is instance attribute, not class attribute.
         # On read, this will be overwritten by the default in the ui.read (thus, in
         # the current implementation there can be no different default for different
         # Readers). On write, ui.py does not specify a default, so this line here matters.
-        # Currently, the default matches the numpy default for masked values. 
-        self.fill_values = [(masked, '--')]
-        self.formats = {}
-        self.splitter = self.__class__.splitter_class()
+        self.fill_values = copy.copy(self.fill_values)
+        self.formats = copy.copy(self.formats)
+        self.splitter = self.splitter_class()
 
     def process_lines(self, lines):
-        """Strip out comment lines and blank lines from list of ``lines``
+        """
+        Strip out comment lines and blank lines from list of ``lines``
+
+        Parameters
+        ----------
+        lines : list
+            All lines in table
+
+        Returns
+        -------
+        lines : list
+            List of lines
 
-        :param lines: all lines in table
-        :returns: list of lines
         """
         nonblank_lines = (x for x in lines if x.strip())
         if self.comment:
@@ -576,6 +688,15 @@ class BaseData(object):
                     for i in col.mask.nonzero()[0]:
                         col.str_vals[i] = mask_val
 
+    def str_vals(self):
+        '''convert all values in table to a list of lists of strings'''
+        self._set_fill_values(self.cols)
+        self._set_col_formats()
+        for col in self.cols:
+            col.str_vals = list(col_iter_str_vals(col))
+        self._replace_vals(self.cols)
+        return [col.str_vals for col in self.cols]
+
     def write(self, lines):
         if hasattr(self.start_line, '__call__'):
             raise TypeError('Start_line attribute cannot be callable for write()')
@@ -585,12 +706,7 @@ class BaseData(object):
         while len(lines) < data_start_line:
             lines.append(itertools.cycle(self.write_spacer_lines))
 
-        self._set_fill_values(self.cols)
-        self._set_col_formats()
-        for col in self.cols:
-            col.str_vals = list(col.iter_str_vals())
-        self._replace_vals(self.cols)
-        col_str_iters = [col.str_vals for col in self.cols]
+        col_str_iters = self.str_vals()
         for vals in zip(*col_str_iters):
             lines.append(self.splitter.join(vals))
 
@@ -598,8 +714,8 @@ class BaseData(object):
         """
         """
         for col in self.cols:
-            if col.name in self.formats:
-                col.format = self.formats[col.name]
+            if col_getattr(col, 'name') in self.formats:
+                col_setattr(col, 'format', self.formats[col.name])
 
 
 def convert_numpy(numpy_type):
@@ -618,13 +734,35 @@ def convert_numpy(numpy_type):
         converter_type = IntType
     elif 'float' in type_name:
         converter_type = FloatType
+    elif 'bool' in type_name:
+        converter_type = BoolType
     elif 'str' in type_name:
         converter_type = StrType
     else:
         converter_type = AllType
 
-    def converter(vals):
+    def bool_converter(vals):
+        """
+        Convert values "False" and "True" to bools.  Raise an exception
+        for any other string values.
+        """
+        # Try a smaller subset first for a long array
+        if len(vals) > 10000:
+            svals = numpy.asarray(vals[:1000])
+            if not numpy.all((svals == 'False') | (svals == 'True')):
+                raise ValueError('bool input strings must be only False or True')
+        vals = numpy.asarray(vals)
+        trues = vals == 'True'
+        falses = vals == 'False'
+        if not numpy.all(trues | falses):
+            raise ValueError('bool input strings must be only False or True')
+        return trues
+
+    def generic_converter(vals):
         return numpy.array(vals, numpy_type)
+
+    converter = bool_converter if converter_type is BoolType else generic_converter
+
     return converter, converter_type
 
 
@@ -656,8 +794,14 @@ class BaseOutputter(object):
 
     def _convert_vals(self, cols):
         for col in cols:
-            converters = self.converters.get(col.name,
-                                             self.default_converters)
+            # If a specific dtype was specified for a column, then use that
+            # to set the defaults, otherwise use the generic defaults.
+            default_converters = ([convert_numpy(col.dtype)] if col.dtype
+                                  else self.default_converters)
+
+            # If the user supplied a specific convert then that takes precedence over defaults
+            converters = self.converters.get(col.name, default_converters)
+
             col.converters = self._validate_and_copy(col, converters)
 
             while not hasattr(col, 'data'):
@@ -703,6 +847,8 @@ class TableOutputter(BaseOutputter):
             for attr in ('format', 'unit', 'description'):
                 if hasattr(col, attr):
                     setattr(out_col, attr, getattr(col, attr))
+            if hasattr(col, 'meta'):
+                out_col.meta.update(col.meta)
 
         return out
 
@@ -715,6 +861,10 @@ class MetaBaseReader(type):
         if format is None:
             return
 
+        fast = dct.get('_fast')
+        if fast is not None:
+            FAST_CLASSES[format] = cls
+
         FORMAT_CLASSES[format] = cls
 
         io_formats = ['ascii.' + format] + dct.get('_io_registry_format_aliases', [])
@@ -738,36 +888,25 @@ def _is_number(x):
         return True
     return False
 
+def _apply_include_exclude_names(table, names, include_names, exclude_names):
+    """
+    Apply names, include_names and exclude_names to a table.
 
-def _apply_include_exclude_names(table, names, include_names, exclude_names, strict_names):
-    """Apply names, include_names and exclude_names to a table.
+    Parameters
+    ----------
+    table : `~astropy.table.Table`
+        Input table
+    names : list
+        List of names to override those in table (set to None to use existing names)
+    include_names : list
+        List of names to include in output
+    exclude_names: list
+        List of names to exclude from output (applied after ``include_names``)
 
-    :param table: input table (Reader object, NumPy struct array, list of lists, etc)
-    :param names: list of names to override those in table (default=None uses existing names)
-    :param include_names: list of names to include in output (default=None selects all names)
-    :param exclude_names: list of names to exlude from output (applied after ``include_names``)
-    :param strict_names: apply strict checks on column names
     """
-    # Check column names.  This must be done before applying the names transformation
-    # so that guessing will fail appropriately if `names` is supplied.  For instance
-    # if the basic reader is given a table with no column header row.
-    if strict_names:
-        # Impose strict requirements on column names (normally used in guessing)
-        bads = [" ", ",", "|", "\t", "'", '"']
-        for name in table.colnames:
-            if (_is_number(name) or
-                    len(name) == 0 or
-                    name[0] in bads or
-                    name[-1] in bads):
-                raise ValueError('Column name {0!r} does not meet strict name requirements'
-                                 .format(name))
 
     if names is not None:
         # Rename table column names to those passed by user
-        if len(names) != len(table.colnames):
-            raise ValueError('Length of names argument ({0}) does not match number'
-                             ' of table columns ({1})'.format(len(names), len(table.colnames)))
-
         # Temporarily rename with names that are not in `names` or `table.colnames`.
         # This ensures that rename succeeds regardless of existing names.
         xxxs = 'x' * max(len(name) for name in list(names) + list(table.colnames))
@@ -806,12 +945,18 @@ class BaseReader(object):
     include_names = None
     exclude_names = None
     strict_names = False
+    guessing = False
+
+    header_class = BaseHeader
+    data_class = BaseData
+    inputter_class = BaseInputter
+    outputter_class = TableOutputter
 
     def __init__(self):
-        self.header = BaseHeader()
-        self.data = BaseData()
-        self.inputter = BaseInputter()
-        self.outputter = TableOutputter()
+        self.header = self.header_class()
+        self.data = self.data_class()
+        self.inputter = self.inputter_class()
+        self.outputter = self.outputter_class()
         # Data and Header instances benefit from a little cross-coupling.  Header may need to
         # know about number of data columns for auto-column name generation and Data may
         # need to know about header (e.g. for fixed-width tables where widths are spec'd in header.
@@ -837,8 +982,16 @@ class BaseReader(object):
         * String (newline separated) with all header and data lines (must have at least 2 lines)
         * List of strings
 
-        :param table: table input
-        :returns: output table
+        Parameters
+        ----------
+        table : str, file_like, list
+            Input table.
+
+        Returns
+        -------
+        table : `~astropy.table.Table`
+            Output table
+
         """
         # If ``table`` is a file then store the name in the ``data``
         # attribute. The ``table`` is a "file" if it is a string
@@ -848,10 +1001,6 @@ class BaseReader(object):
             if os.linesep not in table + '':
                 self.data.table_name = os.path.basename(table)
 
-        # Same from __init__.  ??? Do these need to be here?
-        self.data.header = self.header
-        self.header.data = self.data
-
         # Get a list of the lines (rows) in the table
         self.lines = self.inputter.get_lines(table)
 
@@ -864,23 +1013,27 @@ class BaseReader(object):
         # Get the table column definitions
         self.header.get_cols(self.lines)
 
-        cols = self.header.cols
+        # Make sure columns are valid
+        self.header.check_column_names(self.names, self.strict_names, self.guessing)
+
+        self.cols = cols = self.header.cols
         self.data.splitter.cols = cols
+        n_cols = len(cols)
 
         for i, str_vals in enumerate(self.data.get_str_vals()):
-            if len(str_vals) != len(cols):
-                str_vals = self.inconsistent_handler(str_vals, len(cols))
+            if len(str_vals) != n_cols:
+                str_vals = self.inconsistent_handler(str_vals, n_cols)
 
                 # if str_vals is None, we skip this row
                 if str_vals is None:
                     continue
 
                 # otherwise, we raise an error only if it is still inconsistent
-                if len(str_vals) != len(cols):
+                if len(str_vals) != n_cols:
                     errmsg = ('Number of header columns (%d) inconsistent with '
                               'data columns (%d) at data line %d\n'
                               'Header values: %s\n'
-                              'Data values: %s' % (len(cols), len(str_vals), i,
+                              'Data values: %s' % (n_cols, len(str_vals), i,
                                                    [x.name for x in cols], str_vals))
                     raise InconsistentTableError(errmsg)
 
@@ -889,15 +1042,17 @@ class BaseReader(object):
 
         self.data.masks(cols)
         table = self.outputter(cols, self.meta)
+        if hasattr(self.header, 'table_meta'):
+            table.meta.update(self.header.table_meta)
         self.cols = self.header.cols
 
-        _apply_include_exclude_names(table, self.names, self.include_names, self.exclude_names,
-                                     self.strict_names)
+        _apply_include_exclude_names(table, self.names, self.include_names, self.exclude_names)
 
         return table
 
     def inconsistent_handler(self, str_vals, ncols):
-        """Adjust or skip data entries if a row is inconsistent with the header.
+        """
+        Adjust or skip data entries if a row is inconsistent with the header.
 
         The default implementation does no adjustment, and hence will always trigger
         an exception in read() any time the number of data entries does not match
@@ -905,10 +1060,17 @@ class BaseReader(object):
 
         Note that this will *not* be called if the row already matches the header.
 
-        :param str_vals: A list of value strings from the current row of the table.
-        :param ncols: The expected number of entries from the table header.
-        :returns:
-            list of strings to be parsed into data entries in the output table. If
+        Parameters
+        ----------
+        str_vals : list
+            A list of value strings from the current row of the table.
+        ncols : int
+            The expected number of entries from the table header.
+
+        Returns
+        -------
+        str_vals : list
+            List of strings to be parsed into data entries in the output table. If
             the length of this list does not match ``ncols``, an exception will be
             raised in read().  Can also be None, in which case the row will be
             skipped.
@@ -920,7 +1082,7 @@ class BaseReader(object):
     def comment_lines(self):
         """Return lines in the table that match header.comment regexp"""
         if not hasattr(self, 'lines'):
-            raise ValueError('Table must be read prior to accessing the header_comment_lines')
+            raise ValueError('Table must be read prior to accessing the header comment lines')
         if self.header.comment:
             re_comment = re.compile(self.header.comment)
             comment_lines = [x for x in self.lines if re_comment.match(x)]
@@ -928,23 +1090,42 @@ class BaseReader(object):
             comment_lines = []
         return comment_lines
 
+    def write_header(self, lines, meta):
+        self.header.write_comments(lines, meta)
+        self.header.write(lines)
+
     def write(self, table):
-        """Write ``table`` as list of strings.
+        """
+        Write ``table`` as list of strings.
+
+        Parameters
+        ----------
+        table : `~astropy.table.Table`
+            Input table data.
+
+        Returns
+        -------
+        lines : list
+            List of strings corresponding to ASCII table
 
-        :param table: input table data (astropy.table.Table object)
-        :returns: list of strings corresponding to ASCII table
         """
 
-        _apply_include_exclude_names(table, self.names, self.include_names, self.exclude_names,
-                                     self.strict_names)
-
-        # link information about the columns to the writer object (i.e. self)
+        # Check column names before altering
         self.header.cols = list(six.itervalues(table.columns))
-        self.data.cols = list(six.itervalues(table.columns))
+        self.header.check_column_names(self.names, self.strict_names, False)
+
+        _apply_include_exclude_names(table, self.names, self.include_names, self.exclude_names)
+
+        # Now use altered columns
+        new_cols = list(six.itervalues(table.columns))
+        # link information about the columns to the writer object (i.e. self)
+        self.header.cols = new_cols
+        self.data.cols = new_cols
+        self.header.table_meta = table.meta
 
         # Write header and data to lines list
         lines = []
-        self.header.write(lines)
+        self.write_header(lines, table.meta)
         self.data.write(lines)
 
         return lines
@@ -1017,12 +1198,19 @@ def _get_reader(Reader, Inputter=None, Outputter=None, **kwargs):
     because it depends only on the "core" module.
     """
 
+    from .fastbasic import FastBasic
+    if issubclass(Reader, FastBasic): # Fast readers handle args separately
+        if Inputter is not None:
+            kwargs['Inputter'] = Inputter
+        return Reader(**kwargs)
+
+    if 'fast_reader' in kwargs:
+        del kwargs['fast_reader'] # ignore fast_reader parameter for slow readers
     reader_kwargs = dict([k, v] for k, v in kwargs.items() if k not in extra_reader_pars)
     reader = Reader(**reader_kwargs)
 
     if Inputter is not None:
         reader.inputter = Inputter()
-    reader.outputter = TableOutputter()
 
     if Outputter is not None:
         reader.outputter = Outputter()
@@ -1054,7 +1242,7 @@ def _get_reader(Reader, Inputter=None, Outputter=None, **kwargs):
             # However, position_line is given as absolute number and not relative to header_start.
             # So, ignore this Reader here.
             if (('data_start' not in kwargs) and (default_header_length is not None)
-                    and reader._format_name != 'fixed_width_two_line'):
+                    and reader._format_name not in ['fixed_width_two_line', 'commented_header']):
                 reader.data.start_line = reader.header.start_line + default_header_length
         elif kwargs['header_start'] is not None:
             # User trying to set a None header start to some value other than None
@@ -1074,7 +1262,7 @@ def _get_reader(Reader, Inputter=None, Outputter=None, **kwargs):
     # Strict names is normally set only within the guessing process to
     # indicate that column names cannot be numeric or have certain
     # characters at the beginning or end.  It gets used in
-    # core._apply_include_exclude_names().
+    # BaseHeader.check_column_names().
     if 'strict_names' in kwargs:
         reader.strict_names = kwargs['strict_names']
     if 'fill_values' in kwargs:
@@ -1093,11 +1281,20 @@ extra_writer_pars = ('delimiter', 'comment', 'quotechar', 'formats',
                      'fill_exclude_names')
 
 
-def _get_writer(Writer, **kwargs):
+def _get_writer(Writer, fast_writer, **kwargs):
     """Initialize a table writer allowing for common customizations. This
     routine is for internal (package) use only and is useful because it depends
     only on the "core" module. """
 
+    from .fastbasic import FastBasic
+
+    if issubclass(Writer, FastBasic): # Fast writers handle args separately
+        return Writer(**kwargs)
+    elif fast_writer and 'fast_{0}'.format(Writer._format_name) in FAST_CLASSES:
+        # Switch to fast writer
+        kwargs['fast_writer'] = fast_writer
+        return FAST_CLASSES['fast_{0}'.format(Writer._format_name)](**kwargs)
+
     writer_kwargs = dict([k, v] for k, v in kwargs.items() if k not in extra_writer_pars)
     writer = Writer(**writer_kwargs)
 
@@ -1117,9 +1314,7 @@ def _get_writer(Writer, **kwargs):
             # Restore the default SplitterClass process_val method which strips
             # whitespace.  This may have been changed in the Writer
             # initialization (e.g. Rdb and Tab)
-            Class = writer.data.splitter.__class__
-            obj = writer.data.splitter
-            writer.data.splitter.process_val = Class.process_val.__get__(obj, Class)
+            writer.data.splitter.process_val = lambda x: x.strip()
         else:
             writer.data.splitter.process_val = None
     if 'names' in kwargs:
