@@ -11,13 +11,16 @@ fixedwidth.py:
 from __future__ import absolute_import, division, print_function
 
 from ...extern.six.moves import zip
+from ...table.column import col_getattr
 
 from . import core
-from .core import InconsistentTableError
+from .core import InconsistentTableError, DefaultSplitter
+from . import basic
 
 
 class FixedWidthSplitter(core.BaseSplitter):
-    """Split line based on fixed start and end positions for each ``col`` in
+    """
+    Split line based on fixed start and end positions for each ``col`` in
     ``self.cols``.
 
     This class requires that the Header class will have defined ``col.start``
@@ -29,9 +32,11 @@ class FixedWidthSplitter(core.BaseSplitter):
     style so line[start:end] is the desired substring for a column.  This splitter
     class does not have a hook for ``process_lines`` since that is generally not
     useful for fixed-width input.
+
     """
     delimiter_pad = ''
     bookend = False
+    delimiter = '|'
 
     def __call__(self, lines):
         for line in lines:
@@ -55,24 +60,20 @@ class FixedWidthSplitter(core.BaseSplitter):
         return bookend_left + padded_delim.join(vals) + bookend_right
 
 
-class FixedWidthHeader(core.BaseHeader):
-    """Fixed width table header reader.
+class FixedWidthHeaderSplitter(DefaultSplitter):
+    '''Splitter class that splits on ``|``.'''
+    delimiter = '|'
 
-    The key settable class attributes are:
 
-    :param auto_format: format string for auto-generating column names
-    :param start_line: None, int, or a function of ``lines`` that returns None or int
-    :param comment: regular expression for comment lines
-    :param splitter_class: Splitter class for splitting data lines into columns
-    :param position_line: row index of line that specifies position (default = 1)
-    :param position_char: character used to write the position line (default = "-")
-    :param col_starts: list of start positions for each column (0-based counting)
-    :param col_ends: list of end positions (inclusive) for each column
-    :param delimiter_pad: padding around delimiter when writing (default = None)
-    :param bookend: put the delimiter at start and end of line when writing (default = False)
+class FixedWidthHeader(basic.BasicHeader):
     """
-
+    Fixed width table header reader.
+    """
+    splitter_class = FixedWidthHeaderSplitter
+    """ Splitter class for splitting data lines into columns """
     position_line = None   # secondary header line position
+    """ row index of line that specifies position (default = 1) """
+    set_of_position_line_characters = set(r'`~!#$%^&*-_+=\|":' + "'")
 
     def get_line(self, lines, index):
         for i, line in enumerate(self.process_lines(lines)):
@@ -83,13 +84,17 @@ class FixedWidthHeader(core.BaseHeader):
         return line
 
     def get_cols(self, lines):
-        """Initialize the header Column objects from the table ``lines``.
+        """
+        Initialize the header Column objects from the table ``lines``.
 
         Based on the previously set Header attributes find or create the column names.
-        Sets ``self.cols`` with the list of Columns. 
+        Sets ``self.cols`` with the list of Columns.
+
+        Parameters
+        ----------
+        lines : list
+            List of table lines
 
-        :param lines: list of table lines
-        :returns: None
         """
 
         # See "else" clause below for explanation of start_line and position_line
@@ -126,6 +131,16 @@ class FixedWidthHeader(core.BaseHeader):
                 # slice col_ends but expects inclusive col_ends on input (for
                 # more intuitive user interface).
                 line = self.get_line(lines, position_line)
+                if len(set(line) - set([self.splitter.delimiter, ' '])) != 1:
+                    raise InconsistentTableError('Position line should only contain delimiters and one other character, e.g. "--- ------- ---".')
+                    # The line above lies. It accepts white space as well.
+                    # We don't want to encourage using three different
+                    # characters, because that can cause ambiguities, but white
+                    # spaces are so common everywhere that practicality beats
+                    # purity here.
+                charset = self.set_of_position_line_characters.union(set([self.splitter.delimiter, ' ']))
+                if not set(line).issubset(charset):
+                    raise InconsistentTableError('Characters in position line must be part of {0}'.format(charset))
                 vals, self.col_starts, col_ends = self.get_fixedwidth_params(line)
                 self.col_ends = [x - 1 for x in col_ends]
 
@@ -137,21 +152,34 @@ class FixedWidthHeader(core.BaseHeader):
 
         self._set_cols_from_names()
 
-        # Set column start and end positions. 
+        # Set column start and end positions.
         for i, col in enumerate(self.cols):
             col.start = starts[i]
             col.end = ends[i]
 
     def get_fixedwidth_params(self, line):
-        """Split ``line`` on the delimiter and determine column values and
+        """
+        Split ``line`` on the delimiter and determine column values and
         column start and end positions.  This might include null columns with
         zero length (e.g. for ``header row = "| col1 || col2 | col3 |"`` or
         ``header2_row = "----- ------- -----"``).  The null columns are
         stripped out.  Returns the values between delimiters and the
         corresponding start and end positions.
 
-        :param line: input line
-        :returns: (vals, starts, ends)
+        Parameters
+        ----------
+        line : str
+            Input line
+
+        Returns
+        -------
+        vals : list
+            List of values.
+        starts : list
+            List of starting indices.
+        ends : list
+            List of ending indices.
+
         """
 
         # If column positions are already specified then just use those, otherwise
@@ -186,41 +214,29 @@ class FixedWidthHeader(core.BaseHeader):
         pass
 
 
-class FixedWidthData(core.BaseData):
-    """Base table data reader.
-
-    :param start_line: None, int, or a function of ``lines`` that returns None or int
-    :param end_line: None, int, or a function of ``lines`` that returns None or int
-    :param comment: Regular expression for comment lines
-    :param splitter_class: Splitter class for splitting data lines into columns
+class FixedWidthData(basic.BasicData):
+    """
+    Base table data reader.
     """
-
     splitter_class = FixedWidthSplitter
+    """ Splitter class for splitting data lines into columns """
 
     def write(self, lines):
-
-        self._set_fill_values(self.cols)
-        self._set_col_formats()
-        for col in self.cols:
-            col.str_vals = list(col.iter_str_vals())
-        self._replace_vals(self.cols)
-        col_str_iters = [col.str_vals for col in self.cols]
-
         vals_list = []
-        # Col iterator does the formatting defined above so each val is a string
-        # and vals is a tuple of strings for all columns of each row
+        col_str_iters = self.str_vals()
         for vals in zip(*col_str_iters):
             vals_list.append(vals)
 
         for i, col in enumerate(self.cols):
             col.width = max([len(vals[i]) for vals in vals_list])
             if self.header.start_line is not None:
-                col.width = max(col.width, len(col.name))
+                col.width = max(col.width, len(col_getattr(col, 'name')))
 
         widths = [col.width for col in self.cols]
 
         if self.header.start_line is not None:
-            lines.append(self.splitter.join([col.name for col in self.cols], widths))
+            lines.append(self.splitter.join([col_getattr(col, 'name') for col in self.cols],
+                                            widths))
 
         if self.header.position_line is not None:
             char = self.header.position_char
@@ -235,8 +251,9 @@ class FixedWidthData(core.BaseData):
         return lines
 
 
-class FixedWidth(core.BaseReader):
-    """Read or write a fixed width table with a single header line that defines column
+class FixedWidth(basic.Basic):
+    """
+    Read or write a fixed width table with a single header line that defines column
     names and positions.  Examples::
 
       # Bar delimiter in header and data
@@ -259,38 +276,35 @@ class FixedWidth(core.BaseReader):
 
     See the :ref:`fixed_width_gallery` for specific usage examples.
 
-    :param col_starts: list of start positions for each column (0-based counting)
-    :param col_ends: list of end positions (inclusive) for each column
-    :param delimiter_pad: padding around delimiter when writing (default = None)
-    :param bookend: put the delimiter at start and end of line when writing (default = False)
     """
     _format_name = 'fixed_width'
     _description = 'Fixed width'
 
+    header_class = FixedWidthHeader
+    data_class = FixedWidthData
+
+
     def __init__(self, col_starts=None, col_ends=None, delimiter_pad=' ', bookend=True):
-        core.BaseReader.__init__(self)
-
-        self.header = FixedWidthHeader()
-        self.data = FixedWidthData()
-        self.data.header = self.header
-        self.header.data = self.data
-
-        self.header.splitter.delimiter = '|'
-        self.data.splitter.delimiter = '|'
+        super(FixedWidth, self).__init__()
         self.data.splitter.delimiter_pad = delimiter_pad
         self.data.splitter.bookend = bookend
-        self.header.start_line = 0
-        self.data.start_line = 1
-        self.header.comment = r'\s*#'
-        self.header.write_comment = '# '
-        self.data.comment = r'\s*#'
-        self.data.write_comment = '# '
         self.header.col_starts = col_starts
         self.header.col_ends = col_ends
 
 
+class FixedWidthNoHeaderHeader(FixedWidthHeader):
+    '''Header reader for fixed with tables with no header line'''
+    start_line = None
+
+
+class FixedWidthNoHeaderData(FixedWidthData):
+    '''Data reader for fixed width tables with no header line'''
+    start_line = 0
+
+
 class FixedWidthNoHeader(FixedWidth):
-    """Read or write a fixed width table which has no header line.  Column
+    """
+    Read or write a fixed width table which has no header line.  Column
     names are either input (``names`` keyword) or auto-generated.  Column
     positions are determined either by input (``col_starts`` and ``col_stops``
     keywords) or by splitting the first data line.  In the latter case a
@@ -313,23 +327,41 @@ class FixedWidthNoHeader(FixedWidth):
 
     See the :ref:`fixed_width_gallery` for specific usage examples.
 
-    :param col_starts: list of start positions for each column (0-based counting)
-    :param col_ends: list of end positions (inclusive) for each column
-    :param delimiter_pad: padding around delimiter when writing (default = None)
-    :param bookend: put the delimiter at start and end of line when writing (default = False)
     """
     _format_name = 'fixed_width_no_header'
     _description = 'Fixed width with no header'
+    header_class = FixedWidthNoHeaderHeader
+    data_class = FixedWidthNoHeaderData
+
 
     def __init__(self, col_starts=None, col_ends=None, delimiter_pad=' ', bookend=True):
-        FixedWidth.__init__(self, col_starts, col_ends,
+        super(FixedWidthNoHeader, self).__init__(col_starts, col_ends,
                             delimiter_pad=delimiter_pad, bookend=bookend)
-        self.header.start_line = None
-        self.data.start_line = 0
+
+
+class FixedWidthTwoLineHeader(FixedWidthHeader):
+    '''Header reader for fixed width tables splitting on whitespace.
+
+    For fixed width tables with several header lines, there is typically
+    a white-space delimited format line, so splitting on white space is
+    needed.
+    '''
+    splitter_class = DefaultSplitter
+
+
+class FixedWidthTwoLineDataSplitter(FixedWidthSplitter):
+    '''Splitter for fixed width tables splitting on ``' '``.'''
+    delimiter = ' '
+
+
+class FixedWidthTwoLineData(FixedWidthData):
+    '''Data reader for fixed with tables with two header lines.'''
+    splitter_class = FixedWidthTwoLineDataSplitter
 
 
 class FixedWidthTwoLine(FixedWidth):
-    """Read or write a fixed width table which has two header lines.  The first
+    """
+    Read or write a fixed width table which has two header lines.  The first
     header line defines the column names and the second implicitly defines the
     column positions.  Examples::
 
@@ -351,18 +383,14 @@ class FixedWidthTwoLine(FixedWidth):
 
     See the :ref:`fixed_width_gallery` for specific usage examples.
 
-    :param position_line: row index of line that specifies position (default = 1)
-    :param position_char: character used to write the position line (default = "-")
-    :param delimiter_pad: padding around delimiter when writing (default = None)
-    :param bookend: put the delimiter at start and end of line when writing (default = False)
     """
     _format_name = 'fixed_width_two_line'
     _description = 'Fixed width with second header line'
+    data_class = FixedWidthTwoLineData
+    header_class = FixedWidthTwoLineHeader
 
     def __init__(self, position_line=1, position_char='-', delimiter_pad=None, bookend=False):
-        FixedWidth.__init__(self, delimiter_pad=delimiter_pad, bookend=bookend)
+        super(FixedWidthTwoLine, self).__init__(delimiter_pad=delimiter_pad, bookend=bookend)
         self.header.position_line = position_line
         self.header.position_char = position_char
         self.data.start_line = position_line + 1
-        self.header.splitter.delimiter = ' '
-        self.data.splitter.delimiter = ' '
