@@ -572,7 +572,7 @@ class TestFileFunctions(FitsTestCase):
         try:
             fits.open(self.temp('foobar.fits'))
         except IOError as e:
-            assert 'File does not exist' in str(e)
+            assert 'No such file or directory' in str(e)
         except:
             raise
 
@@ -947,6 +947,23 @@ class TestFileFunctions(FitsTestCase):
 
         self._test_write_string_bytes_io(io.BytesIO())
 
+    @pytest.mark.skipif(str('sys.platform.startswith("win32")'))
+    def test_filename_with_colon(self):
+        """
+        Test reading and writing a file with a colon in the filename.
+
+        Regression test for https://github.com/astropy/astropy/issues/3122
+        """
+
+        # Skip on Windows since colons in filenames makes NTFS sad.
+
+        filename = 'APEXHET.2014-04-01T15:18:01.000.fits'
+        hdu = fits.PrimaryHDU(data=np.arange(10))
+        hdu.writeto(self.temp(filename))
+
+        with fits.open(self.temp(filename)) as hdul:
+            assert np.all(hdul[0].data == hdu.data)
+
     def _test_write_string_bytes_io(self, fileobj):
         """
         Implemented for both test_write_stringio and test_write_bytesio.
@@ -1059,3 +1076,8 @@ class TestStreamingFunctions(FitsTestCase):
 
         with fits.open(self.data('blank.fits'), ignore_blank=True) as f:
             assert f[0].data.flat[0] == 2
+
+    def test_error_if_memmap_impossible(self):
+        pth = self.data('blank.fits')
+        with pytest.raises(ValueError):
+            fits.open(pth, memmap=True)[0].data
