@@ -118,13 +118,16 @@ class Parameter(object):
             raise TypeError('Bound parameters must have a name specified.')
 
         self._name = name
-        self.__doc__ = description.strip()
+        self.__doc__ = self._description = description.strip()
         self._default = default
 
-        self._default_fixed = fixed
-        self._default_tied = tied
-        self._default_min = min
-        self._default_max = max
+        # NOTE: These are *default* constraints--on model instances constraints
+        # are taken from the model if set, otherwise the defaults set here are
+        # used
+        self._fixed = fixed
+        self._tied = tied
+        self._min = min
+        self._max = max
 
         self._order = None
         self._shape = None
@@ -146,7 +149,7 @@ class Parameter(object):
 
         if model is not None:
             with ignored(AttributeError):
-                # This can only work if the paramter's value has been set by
+                # This can only work if the parameter's value has been set by
                 # the model
                 _, self._shape = self._validate_value(model, self.value)
         else:
@@ -154,16 +157,15 @@ class Parameter(object):
             # and ordering ID
             self._order = self._get_nextid()
 
-
     def __get__(self, obj, objtype):
         if obj is None:
             return self
 
         return self.__class__(self._name, default=self._default,
                               getter=self._getter, setter=self._setter,
-                              fixed=self._default_fixed,
-                              tied=self._default_tied, min=self._default_min,
-                              max=self._default_max, model=obj)
+                              fixed=self._fixed,
+                              tied=self._tied, min=self._min,
+                              max=self._max, model=obj)
 
     def __set__(self, obj, value):
         value, shape = self._validate_value(obj, value)
@@ -214,11 +216,13 @@ class Parameter(object):
                     "dimension {2}".format(key, self.name, n_models))
 
     def __repr__(self):
+        args = "'{0}'".format(self._name)
         if self._model is None:
-            return "Parameter('{0}')".format(self._name)
+            if self._default is not None:
+                args += ', default={0}'.format(self._default)
         else:
-            return "Parameter('{0}', value={1})".format(
-                self._name, self.value)
+            args += ', value={0}'.format(self.value)
+        return "Parameter({0})".format(args)
 
     @property
     def name(self):
@@ -299,9 +303,9 @@ class Parameter(object):
 
         if self._model is not None:
             fixed = self._model._constraints['fixed']
-            return fixed.get(self._name, self._default_fixed)
+            return fixed.get(self._name, self._fixed)
         else:
-            return self._default_fixed
+            return self._fixed
 
     @fixed.setter
     def fixed(self, value):
@@ -324,9 +328,9 @@ class Parameter(object):
 
         if self._model is not None:
             tied = self._model._constraints['tied']
-            return tied.get(self._name, self._default_tied)
+            return tied.get(self._name, self._tied)
         else:
-            return self._default_tied
+            return self._tied
 
     @tied.setter
     def tied(self, value):
@@ -346,10 +350,10 @@ class Parameter(object):
 
         if self._model is not None:
             bounds = self._model._constraints['bounds']
-            default_bounds = (self._default_min, self._default_max)
+            default_bounds = (self._min, self._max)
             return bounds.get(self._name, default_bounds)
         else:
-            return (self._default_min, self._default_max)
+            return (self._min, self._max)
 
     @bounds.setter
     def bounds(self, value):
@@ -405,6 +409,46 @@ class Parameter(object):
             raise AttributeError("can't set attribute 'max' on Parameter "
                                  "definition")
 
+    def copy(self, name=None, description=None, default=None, getter=None,
+             setter=None, fixed=False, tied=False, min=None, max=None):
+        """
+        Make a copy of this `Parameter`, overriding any of its core attributes
+        in the process (or an exact copy).
+
+        The arguments to this method are the same as those for the `Parameter`
+        initializer.  This simply returns a new `Parameter` instance with any
+        or all of the attributes overridden, and so returns the equivalent of:
+
+        .. code:: python
+
+            Parameter(self.name, self.description, ...)
+
+        """
+
+        kwargs = locals().copy()
+        del kwargs['self']
+
+        for key, value in six.iteritems(kwargs):
+            if value is None:
+                kwargs[key] = getattr(self, '_' + key)
+
+        return self.__class__(**kwargs)
+
+    @property
+    def _raw_value(self):
+        """
+        Currently for internal use only.
+
+        Like Parameter.value but does not pass the result through
+        Parameter.getter.  By design this should only be used from bound
+        parameters.
+
+        This will probably be removed are retweaked at some point in the
+        process of rethinking how parameter values are stored/updated.
+        """
+
+        return self._get_model_value(self._model)
+
     @classmethod
     def _get_nextid(cls):
         """Returns a monotonically increasing ID used to order Parameter
@@ -435,7 +479,8 @@ class Parameter(object):
 
         # Use the _param_metrics to extract the parameter value from the
         # _parameters array
-        param_slice, param_shape = model._param_metrics[self._name]
+        param_slice = model._param_metrics[self._name]['slice']
+        param_shape = model._param_metrics[self._name]['shape']
         value = model._parameters[param_slice]
         if param_shape:
             value = value.reshape(param_shape)
@@ -454,7 +499,8 @@ class Parameter(object):
         """
 
         # TODO: Maybe handle exception on invalid input shape
-        param_slice, param_shape = model._param_metrics[self._name]
+        param_slice = model._param_metrics[self._name]['slice']
+        param_shape = model._param_metrics[self._name]['shape']
         param_size = np.prod(param_shape)
 
         if np.size(value) != param_size:
@@ -514,6 +560,21 @@ class Parameter(object):
 
         return wrapper
 
+    def __array__(self, dtype=None):
+        # Make np.asarray(self) work a little more straightforwardly
+        if self._model is None:
+            return np.array([], dtype=np.float)
+        else:
+            return np.asarray(self.value, dtype=dtype)
+
+    def __nonzero__(self):
+        if self._model is None:
+            return True
+        else:
+            return bool(self.value)
+
+    __bool__ = __nonzero__
+
     def __add__(self, val):
         return self.value + val
 
