@@ -7,15 +7,12 @@ This module contains predefined polynomial models.
 from __future__ import (absolute_import, unicode_literals, division,
                         print_function)
 
-import collections
-
 import numpy as np
 
-from .core import FittableModel, Model, SerialCompositeModel, format_input
+from .core import FittableModel, Model
 from .functional_models import Shift
 from .parameters import Parameter
 from .utils import poly_map_domain, comb
-from ..logger import log
 from ..utils import lazyproperty, indent
 
 
@@ -30,7 +27,7 @@ __all__ = [
 class PolynomialBase(FittableModel):
     """
     Base class for all polynomial-like models with an arbitrary number of
-    parameters in the form of coeffecients.
+    parameters in the form of coefficients.
 
     In this case Parameter instances are returned through the class's
     ``__getattr__`` rather than through class descriptors.
@@ -38,7 +35,7 @@ class PolynomialBase(FittableModel):
 
     # Default _param_names list; this will be filled in by the implementation's
     # __init__
-    _param_names = []
+    _param_names = ()
 
     linear = True
     col_fit_deriv = False
@@ -99,19 +96,18 @@ class PolynomialModel(PolynomialBase):
     default values, names and ordering.
     """
 
-    def __init__(self, degree, n_inputs=1, n_outputs=1, n_models=None,
-                 model_set_axis=None, **params):
+    def __init__(self, degree, n_models=None, model_set_axis=None,
+                 name=None, meta=None, **params):
         self._degree = degree
-        self._order = self.get_num_coeff(n_inputs)
-        self._param_names = self._generate_coeff_names(n_inputs)
-        self._n_inputs = n_inputs
-        self._n_outputs = n_outputs
+        self._order = self.get_num_coeff(self.n_inputs)
+        self._param_names = self._generate_coeff_names(self.n_inputs)
 
         if params:
             self._validate_params(**params)
 
         super(PolynomialModel, self).__init__(
-            n_models=n_models, model_set_axis=model_set_axis, **params)
+            n_models=n_models, model_set_axis=model_set_axis, name=name,
+            meta=meta, **params)
 
     def __repr__(self):
         return self._format_repr([self.degree])
@@ -125,18 +121,6 @@ class PolynomialModel(PolynomialBase):
 
         return self._degree
 
-    @property
-    def n_inputs(self):
-        """The number of input variables to model evaluation."""
-
-        return self._n_inputs
-
-    @property
-    def n_outputs(self):
-        """The number of outputs returned when model is evaluated."""
-
-        return self._n_outputs
-
     def get_num_coeff(self, ndim):
         """
         Return the number of coefficients in one parameter set
@@ -176,7 +160,7 @@ class PolynomialModel(PolynomialBase):
                 for j in range(1, self.degree):
                     if i + j < self.degree + 1:
                         names.append('c{0}_{1}'.format(i, j))
-        return names
+        return tuple(names)
 
 
 class OrthoPolynomialBase(PolynomialBase):
@@ -206,12 +190,12 @@ class OrthoPolynomialBase(PolynomialBase):
         {keyword: value} pairs, representing {parameter_name: value}
     """
 
-    n_inputs = 2
-    n_outputs = 1
+    inputs = ('x', 'y')
+    outputs = ('z',)
 
     def __init__(self, x_degree, y_degree, x_domain=None, x_window=None,
                  y_domain=None, y_window=None, n_models=None,
-                 model_set_axis=None, **params):
+                 model_set_axis=None, name=None, meta=None, **params):
         # TODO: Perhaps some of these other parameters should be properties?
         # TODO: An awful lot of the functionality in this method is still
         # shared by PolynomialModel; perhaps some of it can be generalized in
@@ -229,7 +213,8 @@ class OrthoPolynomialBase(PolynomialBase):
             self._validate_params(**params)
 
         super(OrthoPolynomialBase, self).__init__(
-            n_models=n_models, model_set_axis=model_set_axis, **params)
+            n_models=n_models, model_set_axis=model_set_axis,
+            name=name, meta=meta, **params)
 
     def __repr__(self):
         return self._format_repr([self.x_degree, self.y_degree])
@@ -319,7 +304,7 @@ class OrthoPolynomialBase(PolynomialBase):
         for j in range(self.y_degree + 1):
             for i in range(self.x_degree + 1):
                 names.append('c{0}_{1}'.format(i, j))
-        return names
+        return tuple(names)
 
     def _fcache(self, x, y):
         # TODO: Write a docstring explaining the actual purpose of this method
@@ -327,16 +312,15 @@ class OrthoPolynomialBase(PolynomialBase):
 
         raise NotImplementedError("Subclasses should implement this")
 
-    @format_input
-    def __call__(self, x, y, model_set_axis=None):
-        """
-        Transforms data using this model.
+    def evaluate(self, x, y, *coeffs):
+        invcoeff = self.invlex_coeff()
+        return self.imhorner(x, y, invcoeff)
 
-        Parameters
-        --------------
-        x : scalar, list or array
-        y : scalar, lis or array
-        """
+    def prepare_inputs(self, x, y, **kwargs):
+        inputs, format_info = \
+                super(OrthoPolynomialBase, self).prepare_inputs(x, y, **kwargs)
+
+        x, y = inputs
 
         if x.shape != y.shape:
             raise ValueError("Expected input arrays to have the same shape")
@@ -344,9 +328,8 @@ class OrthoPolynomialBase(PolynomialBase):
             x = poly_map_domain(x, self.x_domain, self.x_window)
         if self.y_domain is not None:
             y = poly_map_domain(y, self.y_domain, self.y_window)
-        invcoeff = self.invlex_coeff()
 
-        return self.imhorner(x, y, invcoeff)
+        return (x, y), format_info
 
 
 class Chebyshev1D(PolynomialModel):
@@ -367,34 +350,16 @@ class Chebyshev1D(PolynomialModel):
         keyword : value pairs, representing parameter_name: value
     """
 
+    inputs = ('x',)
+    outputs = ('y',)
+
     def __init__(self, degree, domain=None, window=[-1, 1], n_models=None,
-                 model_set_axis=None, **params):
+                 model_set_axis=None, name=None, meta=None, **params):
         self.domain = domain
         self.window = window
         super(Chebyshev1D, self).__init__(
-            degree, n_inputs=1, n_outputs=1, n_models=n_models,
-            model_set_axis=model_set_axis, **params)
-
-    def clenshaw(self, x, coeff):
-        """Evaluates the polynomial using Clenshaw's algorithm."""
-
-        if isinstance(x, tuple) or isinstance(x, list):
-            x = np.asarray(x)
-        if len(coeff) == 1:
-            c0 = coeff[0]
-            c1 = 0
-        elif len(coeff) == 2:
-            c0 = coeff[0]
-            c1 = coeff[1]
-        else:
-            x2 = 2 * x
-            c0 = coeff[-2]
-            c1 = coeff[-1]
-            for i in range(3, len(coeff) + 1):
-                tmp = c0
-                c0 = coeff[-i] - c1
-                c1 = tmp + c1 * x2
-        return c0 + c1 * x
+            degree, n_models=n_models, model_set_axis=model_set_axis,
+            name=name, meta=meta, **params)
 
     def fit_deriv(self, x, *params):
         """
@@ -422,20 +387,40 @@ class Chebyshev1D(PolynomialModel):
             v[i] = v[i - 1] * x2 - v[i - 2]
         return np.rollaxis(v, 0, v.ndim)
 
-    @format_input
-    def __call__(self, x, model_set_axis=None):
-        """
-        Transforms data using this model.
+    def prepare_inputs(self, x, **kwargs):
+        inputs, format_info = \
+                super(PolynomialModel, self).prepare_inputs(x, **kwargs)
 
-        Parameters
-        --------------
-        x : scalar, list or array
-            input
-        """
+        x = inputs[0]
 
         if self.domain is not None:
             x = poly_map_domain(x, self.domain, self.window)
-        return self.clenshaw(x, self.param_sets)
+
+        return (x,), format_info
+
+    @classmethod
+    def evaluate(cls, x, *coeffs):
+        return cls.clenshaw(x, coeffs)
+
+    @staticmethod
+    def clenshaw(x, coeffs):
+        """Evaluates the polynomial using Clenshaw's algorithm."""
+
+        if len(coeffs) == 1:
+            c0 = coeffs[0]
+            c1 = 0
+        elif len(coeffs) == 2:
+            c0 = coeffs[0]
+            c1 = coeffs[1]
+        else:
+            x2 = 2 * x
+            c0 = coeffs[-2]
+            c1 = coeffs[-1]
+            for i in range(3, len(coeffs) + 1):
+                tmp = c0
+                c0 = coeffs[-i] - c1
+                c1 = tmp + c1 * x2
+        return c0 + c1 * x
 
 
 class Legendre1D(PolynomialModel):
@@ -456,33 +441,31 @@ class Legendre1D(PolynomialModel):
         keyword: value pairs, representing parameter_name: value
     """
 
+    inputs = ('x',)
+    outputs = ('y',)
+
     def __init__(self, degree, domain=None, window=[-1, 1], n_models=None,
-                 model_set_axis=None, **params):
+                 model_set_axis=None, name=None, meta=None, **params):
         self.domain = domain
         self.window = window
         super(Legendre1D, self).__init__(
-            degree, n_inputs=1, n_outputs=1, n_models=n_models,
-            model_set_axis=model_set_axis, **params)
+            degree, n_models=n_models, model_set_axis=model_set_axis,
+            name=name, meta=meta, **params)
 
-    def clenshaw(self, x, coeff):
-        if isinstance(x, tuple) or isinstance(x, list):
-            x = np.asarray(x)
-        if len(coeff) == 1:
-            c0 = coeff[0]
-            c1 = 0
-        elif len(coeff) == 2:
-            c0 = coeff[0]
-            c1 = coeff[1]
-        else:
-            nd = len(coeff)
-            c0 = coeff[-2]
-            c1 = coeff[-1]
-            for i in range(3, len(coeff) + 1):
-                tmp = c0
-                nd = nd - 1
-                c0 = coeff[-i] - (c1 * (nd - 1)) / nd
-                c1 = tmp + (c1 * x * (2 * nd - 1)) / nd
-        return c0 + c1 * x
+    def prepare_inputs(self, x, **kwargs):
+        inputs, format_info = \
+                super(PolynomialModel, self).prepare_inputs(x, **kwargs)
+
+        x = inputs[0]
+
+        if self.domain is not None:
+            x = poly_map_domain(x, self.domain, self.window)
+
+        return (x,), format_info
+
+    @classmethod
+    def evaluate(cls, x, *coeffs):
+        return cls.clenshaw(x, coeffs)
 
     def fit_deriv(self, x, *params):
         """
@@ -509,20 +492,24 @@ class Legendre1D(PolynomialModel):
             v[i] = (v[i - 1] * x * (2 * i - 1) - v[i - 2] * (i - 1)) / i
         return np.rollaxis(v, 0, v.ndim)
 
-    @format_input
-    def __call__(self, x, model_set_axis=None):
-        """
-        Transforms data using this model.
-
-        Parameters
-        --------------
-        x : scalar, list or array
-            input
-        """
-
-        if self.domain is not None:
-            x = poly_map_domain(x, self.domain, self.window)
-        return self.clenshaw(x, self.param_sets)
+    @staticmethod
+    def clenshaw(x, coeffs):
+        if len(coeffs) == 1:
+            c0 = coeffs[0]
+            c1 = 0
+        elif len(coeffs) == 2:
+            c0 = coeffs[0]
+            c1 = coeffs[1]
+        else:
+            nd = len(coeffs)
+            c0 = coeffs[-2]
+            c1 = coeffs[-1]
+            for i in range(3, len(coeffs) + 1):
+                tmp = c0
+                nd = nd - 1
+                c0 = coeffs[-i] - (c1 * (nd - 1)) / nd
+                c1 = tmp + (c1 * x * (2 * nd - 1)) / nd
+        return c0 + c1 * x
 
 
 class Polynomial1D(PolynomialModel):
@@ -543,13 +530,31 @@ class Polynomial1D(PolynomialModel):
         keyword: value pairs, representing parameter_name: value
     """
 
+    inputs = ('x',)
+    outputs = ('y',)
+
     def __init__(self, degree, domain=[-1, 1], window=[-1, 1], n_models=None,
-                 model_set_axis=None, **params):
+                 model_set_axis=None, name=None, meta=None, **params):
         self.domain = domain
         self.window = window
         super(Polynomial1D, self).__init__(
-            degree, n_inputs=1, n_outputs=1, n_models=n_models,
-            model_set_axis=model_set_axis, **params)
+            degree, n_models=n_models, model_set_axis=model_set_axis,
+            name=name, meta=meta, **params)
+
+    def prepare_inputs(self, x, **kwargs):
+        inputs, format_info = \
+                super(Polynomial1D, self).prepare_inputs(x, **kwargs)
+
+        x = inputs[0]
+
+        if self.domain is not None:
+            x = poly_map_domain(x, self.domain, self.window)
+
+        return (x,), format_info
+
+    @classmethod
+    def evaluate(cls, x, *coeffs):
+        return cls.horner(x, coeffs)
 
     def fit_deriv(self, x, *params):
         """
@@ -575,25 +580,13 @@ class Polynomial1D(PolynomialModel):
             v[i] = v[i - 1] * x
         return np.rollaxis(v, 0, v.ndim)
 
-    def horner(self, x, coef):
-        c0 = coef[-1] + x * 0
-        for i in range(2, len(coef) + 1):
-            c0 = coef[-i] + c0 * x
+    @staticmethod
+    def horner(x, coeffs):
+        c0 = coeffs[-1] + x * 0
+        for i in range(2, len(coeffs) + 1):
+            c0 = coeffs[-i] + c0 * x
         return c0
 
-    @format_input
-    def __call__(self, x, model_set_axis=None):
-        """
-        Transforms data using this model.
-
-        Parameters
-        --------------
-        x : scalar, list or array
-            input
-        """
-
-        return self.horner(x, self.param_sets)
-
 
 class Polynomial2D(PolynomialModel):
     """
@@ -625,39 +618,39 @@ class Polynomial2D(PolynomialModel):
         keyword: value pairs, representing parameter_name: value
     """
 
+    inputs = ('x', 'y')
+    outputs = ('z',)
+
     def __init__(self, degree, x_domain=[-1, 1], y_domain=[-1, 1],
                  x_window=[-1, 1], y_window=[-1, 1], n_models=None,
-                 model_set_axis=None, **params):
+                 model_set_axis=None, name=None, meta=None, **params):
         super(Polynomial2D, self).__init__(
-            degree, n_inputs=2, n_outputs=1, n_models=n_models,
-            model_set_axis=model_set_axis, **params)
+            degree, n_models=n_models, model_set_axis=model_set_axis,
+            name=name, meta=meta, **params)
         self.x_domain = x_domain
         self.y_domain = y_domain
         self.x_window = x_window
         self.y_window = y_window
 
-    def mhorner(self, x, y, coeff):
-        """
-        Multivariate Horner's scheme
+    def prepare_inputs(self, x, y, **kwargs):
+        inputs, format_info = \
+                super(Polynomial2D, self).prepare_inputs(x, y, **kwargs)
 
-        Parameters
-        --------------
-        x, y : array
-        coeff : array of coefficients in inverse lexical order
-        """
-        alpha = np.array(self._invlex())
-        r0 = coeff[0]
-        r1 = r0 * 0.0
-        r2 = r0 * 0.0
-        karr = np.diff(alpha, axis=0)
-        for n in range(len(karr)):
-            if karr[n, 1] != 0:
-                r2 = y * (r0 + r1 + r2)
-                r1 = coeff[0] * 0.
-            else:
-                r1 = x * (r0 + r1)
-            r0 = coeff[n + 1]
-        return r0 + r1 + r2
+        x, y = inputs
+
+        if x.shape != y.shape:
+            raise ValueError("Expected input arrays to have the same shape")
+
+        if self.x_domain is not None:
+            x = poly_map_domain(x, self.x_domain, self.x_window)
+        if self.y_domain is not None:
+            y = poly_map_domain(y, self.y_domain, self.y_window)
+
+        return (x, y), format_info
+
+    def evaluate(self, x, y, *coeffs):
+        invcoeff = self.invlex_coeff(coeffs)
+        return self.multivariate_horner(x, y, invcoeff)
 
     def fit_deriv(self, x, y, *params):
         """
@@ -700,34 +693,40 @@ class Polynomial2D(PolynomialModel):
             v = np.hstack([designx, designy])
         return v
 
-    def invlex_coeff(self):
-        coeff = []
+    def invlex_coeff(self, coeffs):
+        invlex_coeffs = []
         lencoeff = range(self.degree + 1)
         for i in lencoeff:
             for j in lencoeff:
                 if i + j <= self.degree:
                     name = 'c{0}_{1}'.format(j, i)
-                    coeff.append(getattr(self, name))
-        return np.array(coeff[::-1])
+                    coeff = coeffs[self.param_names.index(name)]
+                    invlex_coeffs.append(coeff)
+        return np.array(invlex_coeffs[::-1])
 
-    @format_input
-    def __call__(self, x, y, model_set_axis=None):
+    def multivariate_horner(self, x, y, coeffs):
         """
-        Transforms data using this model.
+        Multivariate Horner's scheme
 
         Parameters
         --------------
-        x : scalar, list or array
-            input
-        y : scalar, list or array
-            input
+        x, y : array
+        coeff : array of coefficients in inverse lexical order
         """
 
-        invcoeff = self.invlex_coeff()
-        if x.shape != y.shape:
-            raise ValueError("Expected input arrays to have the same shape")
-
-        return self.mhorner(x, y, invcoeff)
+        alpha = np.array(self._invlex())
+        r0 = coeffs[0]
+        r1 = r0 * 0.0
+        r2 = r0 * 0.0
+        karr = np.diff(alpha, axis=0)
+        for n in range(len(karr)):
+            if karr[n, 1] != 0:
+                r2 = y * (r0 + r1 + r2)
+                r1 = coeffs[0] * 0.
+            else:
+                r1 = x * (r0 + r1)
+            r0 = coeffs[n + 1]
+        return r0 + r1 + r2
 
 
 class Chebyshev2D(OrthoPolynomialBase):
@@ -762,11 +761,11 @@ class Chebyshev2D(OrthoPolynomialBase):
 
     def __init__(self, x_degree, y_degree, x_domain=None, x_window=[-1, 1],
                  y_domain=None, y_window=[-1,1], n_models=None,
-                 model_set_axis=None, **params):
+                 model_set_axis=None, name=None, meta=None, **params):
         super(Chebyshev2D, self).__init__(
             x_degree, y_degree, x_domain=x_domain, y_domain=y_domain,
             x_window=x_window, y_window=y_window, n_models=n_models,
-            model_set_axis=model_set_axis, **params)
+            model_set_axis=model_set_axis, name=name, meta=meta, **params)
 
     def _fcache(self, x, y):
         """
@@ -876,11 +875,11 @@ class Legendre2D(OrthoPolynomialBase):
 
     def __init__(self, x_degree, y_degree, x_domain=None, x_window=[-1, 1],
                  y_domain=None, y_window=[-1, 1], n_models=None,
-                 model_set_axis=None, **params):
+                 model_set_axis=None, name=None, meta=None, **params):
         super(Legendre2D, self).__init__(
             x_degree, y_degree, x_domain=x_domain, y_domain=y_domain,
             x_window=x_window, y_window=y_window, n_models=n_models,
-            model_set_axis=model_set_axis, **params)
+            model_set_axis=model_set_axis, name=name, meta=meta, **params)
 
     def _fcache(self, x, y):
         """
@@ -905,7 +904,7 @@ class Legendre2D(OrthoPolynomialBase):
 
     def fit_deriv(self, x, y, *params):
         """
-        Derivatives with repect to the coefficients.
+        Derivatives with respect to the coefficients.
         This is an array with Legendre polynomials:
 
         Lx0Ly0  Lx1Ly0...LxnLy0...LxnLym
@@ -960,10 +959,11 @@ class _SIP1D(PolynomialBase):
     and SIP should be used instead.
     """
 
-    n_inputs = 2
+    inputs = ('u', 'v')
+    outputs = ('w',)
 
     def __init__(self, order, coeff_prefix, n_models=None,
-                 model_set_axis=None, **params):
+                 model_set_axis=None, name=None, meta=None, **params):
         self.order = order
         self.coeff_prefix = coeff_prefix
         self._param_names = self._generate_coeff_names(coeff_prefix)
@@ -972,7 +972,8 @@ class _SIP1D(PolynomialBase):
             self._validate_params(**params)
 
         super(_SIP1D, self).__init__(n_models=n_models,
-                                     model_set_axis=model_set_axis, **params)
+                                     model_set_axis=model_set_axis,
+                                     name=name, meta=meta, **params)
 
     def __repr__(self):
         return self._format_repr(args=[self.order, self.coeff_prefix])
@@ -982,6 +983,12 @@ class _SIP1D(PolynomialBase):
             [('Order', self.order),
              ('Coeff. Prefix', self.coeff_prefix)])
 
+    def evaluate(self, x, y, *coeffs):
+        # TODO: Rewrite this so that it uses a simpler method of determining
+        # the matrix based on the number of given coefficients.
+        mcoef = self._coeff_matrix(self.coeff_prefix, coeffs)
+        return self._eval_sip(x, y, mcoef)
+
     def get_num_coeff(self, ndim):
         """
         Return the number of coefficients in one param set
@@ -1007,19 +1014,19 @@ class _SIP1D(PolynomialBase):
                     names.append('{0}_{1}_{2}'.format(coeff_prefix, i, j))
         return names
 
-    def _coef_matrix(self, coeff_prefix):
+    def _coeff_matrix(self, coeff_prefix, coeffs):
         mat = np.zeros((self.order + 1, self.order + 1))
         for i in range(2, self.order + 1):
             attr = '{0}_{1}_{2}'.format(coeff_prefix, i, 0)
-            mat[i, 0] = getattr(self, attr).value
+            mat[i, 0] = coeffs[self.param_names.index(attr)]
         for i in range(2, self.order + 1):
             attr = '{0}_{1}_{2}'.format(coeff_prefix, 0, i)
-            mat[0, i] = getattr(self, attr).value
+            mat[0, i] = coeffs[self.param_names.index(attr)]
         for i in range(1, self.order):
             for j in range(1, self.order):
                 if i + j < self.order + 1:
                     attr = '{0}_{1}_{2}'.format(coeff_prefix, i, j)
-                    mat[i, j] = getattr(self, attr).value
+                    mat[i, j] = coeffs[self.param_names.index(attr)]
         return mat
 
     def _eval_sip(self, x, y, coef):
@@ -1036,10 +1043,6 @@ class _SIP1D(PolynomialBase):
                     result = result + coef[i, j] * x ** i * y ** j
         return result
 
-    def __call__(self, x, y):
-        mcoef = self._coef_matrix(self.coeff_prefix)
-        return self._eval_sip(x, y, mcoef)
-
 
 class SIP(Model):
     """
@@ -1076,12 +1079,13 @@ class SIP(Model):
     .. [1] `David Shupe, et al, ADASS, ASP Conference Series, Vol. 347, 2005 <http://adsabs.harvard.edu/abs/2005ASPC..347..491S>`_
     """
 
-    n_inputs = 2
-    n_outputs = 2
+
+    inputs = ('u', 'v')
+    outputs = ('x', 'y')
 
     def __init__(self, crpix, a_order, b_order, a_coeff={}, b_coeff={},
                  ap_order=None, bp_order=None, ap_coeff={}, bp_coeff={},
-                 n_models=None, model_set_axis=None):
+                 n_models=None, model_set_axis=None, name=None, meta=None):
         self._crpix = crpix
         self._a_order = a_order
         self._b_order = b_order
@@ -1098,7 +1102,8 @@ class SIP(Model):
         self.sip1d_b = _SIP1D(b_order, coeff_prefix='B', n_models=n_models,
                               model_set_axis=model_set_axis, **b_coeff)
         super(SIP, self).__init__(n_models=n_models,
-                                  model_set_axis=model_set_axis)
+                                  model_set_axis=model_set_axis, name=name,
+                                  meta=meta)
 
     def __repr__(self):
         return '<{0}({1!r})>'.format(self.__class__.__name__,
@@ -1112,6 +1117,7 @@ class SIP(Model):
 
         return '\n'.join(parts)
 
+    @property
     def inverse(self):
         if (self._ap_order is not None and self._bp_order is not None):
             return InverseSIP(self._ap_order, self._bp_order,
@@ -1119,11 +1125,11 @@ class SIP(Model):
         else:
             raise NotImplementedError("SIP inverse coefficients are not available.")
 
-    def __call__(self, x, y):
-        u = self.shift_a(x)
-        v = self.shift_b(y)
-        f = self.sip1d_a(u, v)
-        g = self.sip1d_b(u, v)
+    def evaluate(self, x, y):
+        u = self.shift_a.evaluate(x, *self.shift_a.param_sets)
+        v = self.shift_b.evaluate(y, *self.shift_b.param_sets)
+        f = self.sip1d_a.evaluate(u, v, *self.sip1d_a.param_sets)
+        g = self.sip1d_b.evaluate(u, v, *self.sip1d_b.param_sets)
         return f, g
 
 
@@ -1146,11 +1152,11 @@ class InverseSIP(Model):
 
     """
 
-    n_inputs = 2
-    n_outputs = 2
+    inputs = ('x', 'y')
+    outputs = ('u', 'v')
 
     def __init__(self, ap_order, bp_order, ap_coeff={}, bp_coeff={},
-                 n_models=None, model_set_axis=None):
+                 n_models=None, model_set_axis=None, name=None, meta=None):
         self._ap_order = ap_order
         self._bp_order = bp_order
         self._ap_coeff = ap_coeff
@@ -1172,12 +1178,8 @@ class InverseSIP(Model):
                                      model_set_axis=model_set_axis,
                                      **bp_coeff_params)
         super(InverseSIP, self).__init__(n_models=n_models,
-                                         model_set_axis=model_set_axis)
-
-    def __call__(self, x, y):
-        x1 = self.sip1d_ap(x, y)
-        y1 = self.sip1d_bp(x, y)
-        return x1, y1
+                                         model_set_axis=model_set_axis,
+                                         name=name, meta=meta)
 
     def __repr__(self):
         return '<{0}({1!r})>'.format(self.__class__.__name__,
@@ -1190,3 +1192,8 @@ class InverseSIP(Model):
             parts.append('')
 
         return '\n'.join(parts)
+
+    def evaluate(self, x, y):
+        x1 = self.sip1d_ap.evaluate(x, y, *self.sip1d_ap.param_sets)
+        y1 = self.sip1d_bp.evaluate(x, y, *self.sip1d_bp.param_sets)
+        return x1, y1
