@@ -15,9 +15,11 @@ References
 from __future__ import (absolute_import, unicode_literals, division,
                         print_function)
 
+import abc
+
 import numpy as np
 
-from .core import (Model, format_input)
+from .core import Model
 from .parameters import Parameter, InputParameterError
 
 from ..utils.compat import ignored
@@ -27,7 +29,8 @@ projcodes = ['TAN', 'AZP', 'SZP', 'STG', 'SIN', 'ARC', 'ZPN', 'ZEA', 'AIR',
              'CYP', 'CEA', 'MER']
 
 
-__all__ = ['Pix2Sky_AZP', 'Sky2Pix_AZP', 'Pix2Sky_CAR', 'Sky2Pix_CAR',
+__all__ = ['Projection', 'Pix2SkyProjection', 'Sky2PixProjection',
+           'Pix2Sky_AZP', 'Sky2Pix_AZP', 'Pix2Sky_CAR', 'Sky2Pix_CAR',
            'Pix2Sky_CEA', 'Sky2Pix_CEA', 'Pix2Sky_CYP', 'Sky2Pix_CYP',
            'Pix2Sky_MER', 'Sky2Pix_MER',
            'Pix2Sky_SIN', 'Sky2Pix_SIN', 'Pix2Sky_STG', 'Sky2Pix_STG',
@@ -36,33 +39,37 @@ __all__ = ['Pix2Sky_AZP', 'Sky2Pix_AZP', 'Pix2Sky_CAR', 'Sky2Pix_CAR',
 
 
 class Projection(Model):
-    """
-    Base class for all sky projections.
-
-    """
-
-    n_inputs = 2
-    n_outputs = 2
+    """Base class for all sky projections."""
 
     # the radius of the projection sphere, by which x,y are scaled
     r0 = 180 / np.pi
 
+    @abc.abstractproperty
+    def inverse(self):
+        """
+        Inverse projection--all projection models must provide an inverse.
+        """
+
+
+class Pix2SkyProjection(Projection):
+    """Base class for all Pix2Sky projections."""
+
+    inputs = ('x', 'y')
+    outputs = ('phi', 'theta')
+
+
+class Sky2PixProjection(Projection):
+    """Base class for all Sky2Pix projections."""
+
+    inputs = ('phi', 'theta')
+    outputs = ('x', 'y')
+
 
 class Zenithal(Projection):
-    """
-    Base class for all Zenithal projections.
-
-    """
-
-    def _compute_rtheta(self):
-        # Subclasses must implement this method
-        raise NotImplementedError("Subclasses should implement this")
-
-    def __call__(self, x, y):
-        raise NotImplementedError("Subclasses should implement this")
+    """Base class for all Zenithal projections."""
 
 
-class Pix2Sky_AZP(Zenithal):
+class Pix2Sky_AZP(Pix2SkyProjection, Zenithal):
     """
     AZP : Zenital perspective projection - pixel to sky.
 
@@ -77,45 +84,39 @@ class Pix2Sky_AZP(Zenithal):
 
 
     def _validate_mu(mu):
-        if mu == -1:
+        if np.asarray(mu == -1).any():
             raise ValueError("AZP projection is not defined for mu=-1")
         return mu
 
     mu = Parameter(default=0.0, setter=_validate_mu)
     gamma = Parameter(default=0.0, getter=np.rad2deg, setter=np.deg2rad)
 
-    def __init__(self, mu=mu.default, gamma=gamma.default):
+    def __init__(self, mu=mu.default, gamma=gamma.default, **kwargs):
         self.check_mu(mu)
         # units : mu - in spherical radii, gamma - in deg
         # TODO: Support quantity objects here and in similar contexts
-        super(Pix2Sky_AZP, self).__init__(mu, gamma)
+        super(Pix2Sky_AZP, self).__init__(mu, gamma, **kwargs)
 
     def check_mu(self, val):
-        if val == -1:
+        if np.asarray(val == -1).any():
             raise ValueError("AZP projection is not defined for mu=-1")
 
+    @property
     def inverse(self):
         return Sky2Pix_AZP(self.mu.value, self.gamma.value)
 
-    # TODO: This contains many superfluous conversions of gamma from radians to
-    # degrees back to radians again--this is going to be reworked in a future
-    # changeset
-    def _compute_rtheta(self, x, y):
-        gamma = np.deg2rad(self.gamma)
-        return np.sqrt(x ** 2 + y ** 2 * (np.cos(gamma)) ** 2)
-
-    @format_input
-    def __call__(self, x, y, model_set_axis=None):
-        gamma = np.deg2rad(self.gamma)
-        phi = np.rad2deg(np.arctan2(x / np.cos(gamma), -y))
-        r = self._compute_rtheta(x, y)
-        pho = r / (self.r0 * (self.mu + 1) +
-                   y * np.sin(gamma))
+    @classmethod
+    def evaluate(cls, x, y, mu, gamma):
+        phi = np.arctan2(x / np.cos(gamma), -y)
+        r = cls._compute_r_theta(x, y, gamma)
+        pho = r / (cls.r0 * (mu + 1) + y * np.sin(gamma))
         psi = np.arctan2(1, pho)
-        omega = np.arcsin((pho * self.mu) / (np.sqrt(pho ** 2 + 1)))
+        omega = np.arcsin((pho * mu) / np.sqrt(pho ** 2 + 1))
+
         theta1 = np.rad2deg(psi - omega)
         theta2 = np.rad2deg(psi + omega) + 180
-        if np.abs(self.mu) < 1:
+
+        if np.abs(mu) < 1:
             if theta1 < 90 and theta1 > -90:
                 theta = theta1
             else:
@@ -127,10 +128,16 @@ class Pix2Sky_AZP(Zenithal):
                 theta = theta1
             else:
                 theta = theta2
+
+        phi = np.rad2deg(phi)
         return phi, theta
 
+    @staticmethod
+    def _compute_r_theta(x, y, gamma):
+        return np.sqrt(x ** 2 + y ** 2 * (np.cos(gamma)) ** 2)
 
-class Sky2Pix_AZP(Zenithal):
+
+class Sky2Pix_AZP(Sky2PixProjection, Zenithal):
     """
     AZP : Zenital perspective projection - sky to pixel.
 
@@ -144,173 +151,184 @@ class Sky2Pix_AZP(Zenithal):
     """
 
     def _validate_mu(mu):
-        if mu == -1:
+        if np.asarray(mu == -1).any():
             raise ValueError("AZP projection is not defined for mu=-1")
         return mu
 
     mu = Parameter(default=0.0, setter=_validate_mu)
     gamma = Parameter(default=0.0, getter=np.rad2deg, setter=np.deg2rad)
 
-    def __init__(self, mu=mu.default, gamma=gamma.default):
-        super(Sky2Pix_AZP, self).__init__(mu, gamma)
-
     def check_mu(self, val):
-        if val == -1:
+        if np.asarray(val == -1).any():
             raise ValueError("AZP projection is not defined for mu=-1")
 
+    @property
     def inverse(self):
         return Pix2Sky_AZP(self.mu.value, self.gamma.value)
 
-    def _compute_rtheta(self, phi, theta):
-        gamma = np.deg2rad(self.gamma)
-        rtheta = (self.r0 * (self.mu + 1) *
-                  np.cos(theta)) / ((self.mu + np.sin(theta)) +
-                                    np.cos(theta) * np.cos(phi) *
-                                    np.tan(gamma))
-        return rtheta
-
-    @format_input
-    def __call__(self, phi, theta, model_set_axis=None):
+    @classmethod
+    def evaluate(cls, phi, theta, mu, gamma):
         phi = np.deg2rad(phi)
         theta = np.deg2rad(theta)
-        gamma = np.deg2rad(self.gamma)
-        r = self._compute_rtheta(phi, theta)
+
+        r = cls._compute_r_theta(phi, theta, mu, gamma)
         x = r * np.sin(phi)
         y = (-r * np.cos(phi)) / np.cos(gamma)
+
         return x, y
 
+    @classmethod
+    def _compute_r_theta(cls, phi, theta, mu, gamma):
+        return ((cls.r0 * (mu + 1) * np.cos(theta)) /
+                (mu + np.sin(theta) +
+                 np.cos(theta) * np.cos(phi) * np.tan(gamma)))
 
-class Pix2Sky_TAN(Zenithal):
+
+class Pix2Sky_TAN(Pix2SkyProjection, Zenithal):
     """
     TAN : Gnomonic projection - pixel to sky.
     """
 
-    def _compute_rtheta(self, x, y):
-        return np.sqrt(x ** 2 + y ** 2)
-
+    @property
     def inverse(self):
         return Sky2Pix_TAN()
 
-    @format_input
-    def __call__(self, x, y, model_set_axis=None):
+    @classmethod
+    def evaluate(cls, x, y):
         phi = np.rad2deg(np.arctan2(x, -y))
-        rtheta = self._compute_rtheta(x, y)
-        theta = np.rad2deg(np.arctan2(self.r0, rtheta))
+        r_theta = cls._compute_r_theta(x, y)
+        theta = np.rad2deg(np.arctan2(cls.r0, r_theta))
+
         return phi, theta
 
+    @staticmethod
+    def _compute_r_theta(x, y):
+        return np.sqrt(x ** 2 + y ** 2)
 
-class Sky2Pix_TAN(Zenithal):
+
+class Sky2Pix_TAN(Sky2PixProjection, Zenithal):
     """
     TAN : Gnomonic Projection - sky to pixel.
     """
 
-    def _compute_rtheta(self, theta):
-        return 1 / np.tan(theta)
-
+    @property
     def inverse(self):
         return Pix2Sky_TAN()
 
-    @format_input
-    def __call__(self, phi, theta, model_set_axis=None):
+    @classmethod
+    def evaluate(cls, phi, theta):
         phi = np.deg2rad(phi)
         theta = np.deg2rad(theta)
-        rtheta = self._compute_rtheta(theta)
-        x = np.rad2deg(rtheta * np.sin(phi))
-        y = - np.rad2deg(rtheta * np.cos(phi))
+
+        r_theta = cls._compute_r_theta(theta)
+        x = np.rad2deg(r_theta * np.sin(phi))
+        y = -np.rad2deg(r_theta * np.cos(phi))
+
         return x, y
 
+    @staticmethod
+    def _compute_r_theta(theta):
+        return 1 / np.tan(theta)
 
-class Pix2Sky_STG(Zenithal):
+
+class Pix2Sky_STG(Pix2SkyProjection, Zenithal):
     """
     STG : Stereographic Projection - pixel to sky.
     """
 
-    def _compute_rtheta(self, x, y):
-        return np.sqrt(x ** 2 + y ** 2)
-
+    @property
     def inverse(self):
         return Sky2Pix_STG()
 
-    def __call__(self, x, y):
+    @classmethod
+    def evaluate(cls, x, y):
         phi = np.rad2deg(np.arctan2(x, -y))
-        rtheta = self._compute_rtheta(x, y)
-        theta = 90 - np.rad2deg(2 * np.arctan(rtheta / (2 * self.r0)))
+        rtheta = cls._compute_r_theta(x, y)
+        theta = 90 - np.rad2deg(2 * np.arctan(rtheta / (2 * cls.r0)))
+
         return phi, theta
 
+    @staticmethod
+    def _compute_r_theta(x, y):
+        return np.sqrt(x ** 2 + y ** 2)
 
-class Sky2Pix_STG(Zenithal):
+
+class Sky2Pix_STG(Sky2PixProjection, Zenithal):
     """
     STG : Stereographic Projection - sky to pixel.
     """
 
+    @property
     def inverse(self):
         return Pix2Sky_STG()
 
-    def _compute_rtheta(self, theta):
-        return (self.r0 * 2 * np.cos(theta)) / (1 + np.sin(theta))
-
-    @format_input
-    def __call__(self, phi, theta, model_set_axis=None):
+    @classmethod
+    def evaluate(cls, phi, theta):
         phi = np.deg2rad(phi)
         theta = np.deg2rad(theta)
-        rtheta = self._compute_rtheta(theta)
-        x = rtheta * np.sin(phi)
-        y = - rtheta * np.cos(phi)
+
+        r_theta = cls._compute_r_theta(theta)
+        x = r_theta * np.sin(phi)
+        y = -r_theta * np.cos(phi)
+
         return x, y
 
+    @classmethod
+    def _compute_r_theta(cls, theta):
+        return (cls.r0 * 2 * np.cos(theta)) / (1 + np.sin(theta))
 
-class Pix2Sky_SIN(Zenithal):
+
+class Pix2Sky_SIN(Pix2SkyProjection, Zenithal):
     """
     SIN : Slant orthographic projection - pixel to sky.
     """
 
+    @property
     def inverse(self):
         return Sky2Pix_SIN()
 
-    def _compute_rtheta(self, x, y):
-        return np.sqrt(x ** 2 + y ** 2)
-
-    @format_input
-    def __call__(self, x, y, model_set_axis=None):
-        rtheta = self._compute_rtheta(x, y)
-        theta = np.rad2deg(np.arccos(rtheta / self.r0))
+    @classmethod
+    def evaluate(cls, x, y):
+        r_theta = cls._compute_r_theta(x, y)
         phi = np.rad2deg(np.arctan2(x, -y))
+        theta = np.rad2deg(np.arccos(r_theta / cls.r0))
+
         return phi, theta
 
+    @staticmethod
+    def _compute_r_theta(x, y):
+        return np.sqrt(x ** 2 + y ** 2)
 
-class Sky2Pix_SIN(Zenithal):
+
+class Sky2Pix_SIN(Sky2PixProjection, Zenithal):
     """
-    SIN : Slant othographic projection - sky to pixel.
+    SIN : Slant orthographic projection - sky to pixel.
     """
 
+    @property
     def inverse(self):
         return Pix2Sky_SIN()
 
-    def _compute_rtheta(self, theta):
-        return self.r0 * np.cos(theta)
-
-    def __call__(self, phi, theta):
+    @classmethod
+    def evaluate(cls, phi, theta):
         phi = np.deg2rad(phi)
         theta = np.deg2rad(theta)
-        rtheta = self._compute_rtheta(theta)
-        x = rtheta * np.sin(phi)
-        y = - rtheta * np.cos(phi)
+        r_theta = cls._compute_r_theta(theta)
+        x = r_theta * np.sin(phi)
+        y = -r_theta * np.cos(phi)
+
         return x, y
 
+    @classmethod
+    def _compute_r_theta(cls, theta):
+        return cls.r0 * np.cos(theta)
+
 
 class Cylindrical(Projection):
-    """
-    Base class for Cylindrical projections.
-    """
-
-    def inverse(self):
-        raise NotImplementedError()
-
-    def __call__(self, x, y):
-        raise NotImplementedError()
+    """Base class for Cylindrical projections."""
 
 
-class Pix2Sky_CYP(Cylindrical):
+class Pix2Sky_CYP(Pix2SkyProjection, Cylindrical):
     """
     CYP : Cylindrical perspective - pixel to sky.
     """
@@ -318,7 +336,7 @@ class Pix2Sky_CYP(Cylindrical):
     def _validate_mu(mu, model):
         with ignored(AttributeError):
             # An attribute error can occur if model.lam has not been set yet
-            if mu == -model.lam:
+            if np.asarray(mu == -model.lam).any():
                 raise ValueError(
                     "CYP projection is not defined for mu=-lambda")
         return mu
@@ -326,30 +344,29 @@ class Pix2Sky_CYP(Cylindrical):
     def _validate_lam(lam, model):
         with ignored(AttributeError):
             # An attribute error can occur if model.lam has not been set yet
-            if lam == -model.mu:
+            if np.asarray(lam == -model.mu).any():
                 raise ValueError(
                     "CYP projection is not defined for mu=-lambda")
         return lam
 
-    mu = Parameter(setter=_validate_mu)
-    lam = Parameter(setter=_validate_lam)
-
-    def __init__(self, mu, lam):
-        super(Pix2Sky_CYP, self).__init__(mu, lam)
+    mu = Parameter(default=0, setter=_validate_mu)
+    lam = Parameter(default=0, setter=_validate_lam)
 
+    @property
     def inverse(self):
         return Sky2Pix_CYP(self.mu.value, self.lam.value)
 
-    @format_input
-    def __call__(self, x, y, model_set_axis=None):
-        phi = x / self.lam
-        eta = y / (self.r0 * (self.mu + self.lam))
-        theta = np.arctan2(eta, 1) + np.arcsin(eta * self.mu /
-                                               (np.sqrt(eta ** 2 + 1)))
+    @classmethod
+    def evaluate(cls, x, y, mu, lam):
+        phi = x / lam
+        eta = y / (cls.r0 * (mu + lam))
+        theta = (np.arctan2(eta, 1) +
+                 np.arcsin(eta * mu / np.sqrt(eta ** 2 + 1)))
+
         return phi, np.rad2deg(theta)
 
 
-class Sky2Pix_CYP(Cylindrical):
+class Sky2Pix_CYP(Sky2PixProjection, Cylindrical):
     """
     CYP : Cylindrical Perspective - sky to pixel.
     """
@@ -357,128 +374,141 @@ class Sky2Pix_CYP(Cylindrical):
     # TODO: Eliminate duplication on these
     def _validate_mu(mu, model):
         with ignored(AttributeError):
-            if mu == -model.lam:
+            if np.asarray(mu == -model.lam).any():
                 raise ValueError(
                     "CYP projection is not defined for mu=-lambda")
         return mu
 
     def _validate_lam(lam, model):
         with ignored(AttributeError):
-            if lam == -model.mu:
+            if np.asarray(lam == -model.mu).any():
                 raise ValueError(
                     "CYP projection is not defined for mu=-lambda")
         return lam
 
-    mu = Parameter(setter=_validate_mu)
-    lam = Parameter(setter=_validate_lam)
-
-    def __init__(self, mu, lam):
-        super(Sky2Pix_CYP, self).__init__(mu, lam)
+    mu = Parameter(default=0, setter=_validate_mu)
+    lam = Parameter(default=0, setter=_validate_lam)
 
+    @property
     def inverse(self):
         return Pix2Sky_CYP(self.mu, self.lam)
 
-    @format_input
-    def __call__(self, phi, theta, model_set_axis=None):
+    @classmethod
+    def evaluate(cls, phi, theta, mu, lam):
         theta = np.deg2rad(theta)
-        x = self.lam * phi
-        y = (self.r0 * ((self.mu + self.lam) /
-                        (self.mu + np.cos(theta))) * np.sin(theta))
+        x = lam * phi
+        y = (cls.r0 * (mu + lam) / (mu + np.cos(theta))) * np.sin(theta)
+
         return x, y
 
 
-class Pix2Sky_CEA(Cylindrical):
+class Pix2Sky_CEA(Pix2SkyProjection, Cylindrical):
     """
     CEA : Cylindrical equal area projection - pixel to sky.
     """
 
     lam = Parameter(default=1)
 
-    def __init__(self, lam=lam.default):
-        super(Pix2Sky_CEA, self).__init__(lam)
-
+    @property
     def inverse(self):
         return Sky2Pix_CEA(self.lam)
 
-    def __call__(self, x, y):
-        phi = np.asarray(x)
-        theta = np.rad2deg(np.arcsin(1 / self.r0 * self.lam * y))
+    @classmethod
+    def evaluate(cls, x, y, lam):
+        phi = x.copy()
+        theta = np.rad2deg(np.arcsin(1 / cls.r0 * lam * y))
+
         return phi, theta
 
 
-class Sky2Pix_CEA(Cylindrical):
+class Sky2Pix_CEA(Sky2PixProjection, Cylindrical):
     """
     CEA: Cylindrical equal area projection - sky to pixel.
     """
 
     lam = Parameter(default=1)
 
-    def __init__(self, lam=lam.default):
-        super(Sky2Pix_CEA, self).__init__(lam)
-
+    @property
     def inverse(self):
         return Pix2Sky_CEA(self.lam)
 
-    @format_input
-    def __call__(self, phi, theta, model_set_axis=None):
+    @classmethod
+    def evaluate(cls, phi, theta, lam):
         x = phi.copy()
         theta = np.deg2rad(theta)
-        y = self.r0 * np.sin(theta) / self.lam
+        y = cls.r0 * np.sin(theta) / lam
+
         return x, y
 
 
-class Pix2Sky_CAR(Cylindrical):
+class Pix2Sky_CAR(Pix2SkyProjection, Cylindrical):
     """
     CAR: Plate carree projection - pixel to sky.
     """
 
+    @property
     def inverse(self):
         return Sky2Pix_CAR()
 
-    @format_input
-    def __call__(self, x, y, model_set_axis=None):
-        return x.copy(), y.copy()
+    @staticmethod
+    def evaluate(x, y):
+        # The intermediate variables are only used here for clarity
+        phi = x.copy()
+        theta = y.copy()
 
-class Sky2Pix_CAR(Cylindrical):
+        return phi, theta
+
+
+class Sky2Pix_CAR(Sky2PixProjection, Cylindrical):
     """
     CAR: Plate carree projection - sky to pixel.
     """
 
+    @property
     def inverse(self):
         return Pix2Sky_CAR()
 
-    @format_input
-    def __call__(self, phi, theta, model_set_axis=None):
-        return phi.copy(), theta.copy()
+    @staticmethod
+    def evaluate(phi, theta):
+        # The intermediate variables are only used here for clarity
+        x = phi.copy()
+        y = theta.copy()
 
-class Pix2Sky_MER(Cylindrical):
+        return x, y
+
+
+class Pix2Sky_MER(Pix2SkyProjection, Cylindrical):
     """
     MER: Mercator - pixel to sky.
     """
 
+    @property
     def inverse(self):
         return Sky2Pix_MER()
 
-    @format_input
-    def __call__(self, x, y, model_set_axis=None):
+    @classmethod
+    def evaluate(cls, x, y):
         phi = x.copy()
-        theta = np.rad2deg(2 * np.arctan(np.e ** (y * np.pi / 180.))) - 90.
+        theta = np.rad2deg(2 * np.arctan(np.exp(y / cls.r0))) - 90
+
         return phi, theta
 
 
-class Sky2Pix_MER(Cylindrical):
+class Sky2Pix_MER(Sky2PixProjection, Cylindrical):
     """
     MER: Mercator - sky to pixel.
     """
 
+    @property
     def inverse(self):
         return Pix2Sky_MER()
 
-    @format_input
-    def __call__(self, phi, theta, model_set_axis=None):
+    @classmethod
+    def evaluate(cls, phi, theta):
         x = phi.copy()
         theta = np.deg2rad(theta)
-        y = self.r0 * np.log(np.tan((np.pi / 2 + theta) / 2))
+        y = cls.r0 * np.log(np.tan((np.pi / 2 + theta) / 2))
+
         return x, y
 
 
@@ -497,8 +527,9 @@ class AffineTransformation2D(Model):
         translation to apply to the inputs
     """
 
-    n_inputs = 2
-    n_outputs = 2
+    inputs = ('x', 'y')
+    outputs = ('x', 'y')
+
     standard_broadcasting = False
 
     matrix = Parameter(
@@ -508,12 +539,7 @@ class AffineTransformation2D(Model):
         setter=lambda t: AffineTransformation2D._validate_vector(t),
         default=[0.0, 0.0])
 
-    def __init__(self, matrix=matrix.default,
-                 translation=translation.default):
-        super(AffineTransformation2D, self).__init__(matrix, translation)
-        self._augmented_matrix = self._create_augmented_matrix(
-            self.matrix.value, self.translation.value)
-
+    @property
     def inverse(self):
         """
         Inverse transformation.
@@ -533,8 +559,8 @@ class AffineTransformation2D(Model):
 
         return self.__class__(matrix=matrix, translation=translation)
 
-    @format_input
-    def __call__(self, x, y, model_set_axis=None):
+    @classmethod
+    def evaluate(cls, x, y, matrix, translation):
         """
         Apply the transformation to a set of 2D Cartesian coordinates given as
         two lists--one for the x coordinates and one for a y coordinates--or a
@@ -549,19 +575,17 @@ class AffineTransformation2D(Model):
         if x.shape != y.shape:
             raise ValueError("Expected input arrays to have the same shape")
 
-        shape = x.shape
+        shape = x.shape or (1,)
         inarr = np.vstack([x.flatten(), y.flatten(), np.ones(x.size)])
 
         if inarr.shape[0] != 3 or inarr.ndim != 2:
             raise ValueError("Incompatible input shapes")
 
-        result = np.dot(self._augmented_matrix, inarr)
+        augmented_matrix = cls._create_augmented_matrix(matrix, translation)
+        result = np.dot(augmented_matrix, inarr)
 
         x, y = result[0], result[1]
-
-        if x.shape != shape:
-            x.shape = shape
-            y.shape = shape
+        x.shape = y.shape = shape
 
         return x, y
 
