@@ -1,5 +1,8 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
+from __future__ import (absolute_import, unicode_literals, division,
+                        print_function)
+
 from .. import models
 from numpy.testing import utils
 from ...tests.helper import pytest
@@ -27,8 +30,8 @@ def test_native_celestial_native():
     utils.assert_allclose(nnphi2, nnphi)
     utils.assert_allclose(ntheta2, ntheta)
 
-    assert n2c.inverse()(nnphi, ntheta) == c2n(nnphi, ntheta)
-    assert c2n.inverse()(nnphi, ntheta) == n2c(nnphi, ntheta)
+    assert n2c.inverse(nnphi, ntheta) == c2n(nnphi, ntheta)
+    assert c2n.inverse(nnphi, ntheta) == n2c(nnphi, ntheta)
 
 
 def test_native_celestial_theta90():
@@ -38,7 +41,13 @@ def test_native_celestial_theta90():
     utils.assert_allclose(alpha, 182)
 
 
-def test_MatrixRotation2D():
-    model = models.MatrixRotation2D(angle=90)
+def test_Rotation2D():
+    model = models.Rotation2D(angle=90)
     x, y = model(1, 0)
-    utils.assert_allclose([x, y], [0, -1], atol=1e-10)
+    utils.assert_allclose([x, y], [0, 1], atol=1e-10)
+
+
+def test_Rotation2D_inverse():
+    model = models.Rotation2D(angle=234.23494)
+    x, y = model.inverse(*model(1, 0))
+    utils.assert_allclose([x, y], [1, 0], atol=1e-10)
