@@ -16,147 +16,26 @@ import numpy as np
 from ..extern.six.moves import xrange
 
 
-__all__ = ['sigma_clip', 'binom_conf_interval', 'binned_binom_proportion',
+__all__ = ['binom_conf_interval', 'binned_binom_proportion',
            'median_absolute_deviation', 'biweight_location',
-           'biweight_midvariance', 'signal_to_noise_oir_ccd', 'bootstrap']
-
+           'biweight_midvariance', 'signal_to_noise_oir_ccd', 'bootstrap',
+           'mad_std', 'gaussian_fwhm_to_sigma', 'gaussian_sigma_to_fwhm']
 
 __doctest_skip__ = ['binned_binom_proportion']
 __doctest_requires__ = {'binom_conf_interval': ['scipy.special']}
 
 
-def sigma_clip(data, sig=3, iters=1, cenfunc=np.ma.median, varfunc=np.var,
-               axis=None, copy=True):
-    """Perform sigma-clipping on the provided data.
+gaussian_sigma_to_fwhm = 2.0 * np.sqrt(2.0 * np.log(2.0))
+"""
+Factor with which to multiply Gaussian 1-sigma standard deviation(s) to
+convert them to full width at half maximum(s).
+"""
 
-    This performs the sigma clipping algorithm - i.e. the data will be iterated
-    over, each time rejecting points that are more than a specified number of
-    standard deviations discrepant.
-
-    .. note::
-        `scipy.stats.sigmaclip
-        <http://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.sigmaclip.html>`_
-        provides a subset of the functionality in this function.
-
-    Parameters
-    ----------
-    data : array-like
-        The data to be sigma-clipped (any shape).
-    sig : float
-        The number of standard deviations (*not* variances) to use as the
-        clipping limit.
-    iters : int or `None`
-        The number of iterations to perform clipping for, or `None` to clip
-        until convergence is achieved (i.e. continue until the last
-        iteration clips nothing).
-    cenfunc : callable
-        The technique to compute the center for the clipping. Must be a
-        callable that takes in a masked array and outputs the central value.
-        Defaults to the median (numpy.median).
-    varfunc : callable
-        The technique to compute the standard deviation about the center. Must
-        be a callable that takes in a masked array and outputs a width
-        estimator::
-
-             deviation**2 > sig**2 * varfunc(deviation)
-
-        Defaults to the variance (numpy.var).
-
-    axis : int or `None`
-        If not `None`, clip along the given axis.  For this case, axis=int will
-        be passed on to cenfunc and varfunc, which are expected to return an
-        array with the axis dimension removed (like the numpy functions).
-        If `None`, clip over all values.  Defaults to `None`.
-    copy : bool
-        If `True`, the data array will be copied.  If `False`, the masked array
-        data will contain the same array as ``data``.  Defaults to `True`.
-
-    Returns
-    -------
-    filtered_data : `numpy.ma.MaskedArray`
-        A masked array with the same shape as ``data`` input, where the points
-        rejected by the algorithm have been masked.
-
-    Notes
-    -----
-     1. The routine works by calculating::
-
-            deviation = data - cenfunc(data [,axis=int])
-
-        and then setting a mask for points outside the range::
-
-            data.mask = deviation**2 > sig**2 * varfunc(deviation)
-
-        It will iterate a given number of times, or until no further points are
-        rejected.
-
-     2. Most numpy functions deal well with masked arrays, but if one would
-        like to have an array with just the good (or bad) values, one can use::
-
-            good_only = filtered_data.data[~filtered_data.mask]
-            bad_only = filtered_data.data[filtered_data.mask]
-
-        However, for multidimensional data, this flattens the array, which may
-        not be what one wants (especially is filtering was done along an axis).
-
-    Examples
-    --------
-
-    This will generate random variates from a Gaussian distribution and return
-    a masked array in which all points that are more than 2 *sample* standard
-    deviation from the median are masked::
-
-        >>> from astropy.stats import sigma_clip
-        >>> from numpy.random import randn
-        >>> randvar = randn(10000)
-        >>> filtered_data = sigma_clip(randvar, 2, 1)
-
-    This will clipping on a similar distribution, but for 3 sigma relative to
-    the sample *mean*, will clip until converged, and does not copy the data::
-
-        >>> from astropy.stats import sigma_clip
-        >>> from numpy.random import randn
-        >>> from numpy import mean
-        >>> randvar = randn(10000)
-        >>> filtered_data = sigma_clip(randvar, 3, None, mean, copy=False)
-
-    This will clip along one axis on a similar distribution with bad points
-    inserted::
-
-        >>> from astropy.stats import sigma_clip
-        >>> from numpy.random import normal
-        >>> from numpy import arange, diag, ones
-        >>> data = arange(5)+normal(0.,0.05,(5,5))+diag(ones(5))
-        >>> filtered_data = sigma_clip(data, axis=0, sig=2.3)
-
-    Note that along the other axis, no points would be masked, as the variance
-    is higher.
-
-    """
-
-    if axis is not None:
-        cenfunc_in = cenfunc
-        varfunc_in = varfunc
-        cenfunc = lambda d: np.expand_dims(cenfunc_in(d, axis=axis), axis=axis)
-        varfunc = lambda d: np.expand_dims(varfunc_in(d, axis=axis), axis=axis)
-
-    filtered_data = np.ma.array(data, copy=copy)
-
-    if iters is None:
-        i = -1
-        lastrej = filtered_data.count() + 1
-        while(filtered_data.count() != lastrej):
-            i += 1
-            lastrej = filtered_data.count()
-            do = filtered_data - cenfunc(filtered_data)
-            filtered_data.mask |= do * do > varfunc(filtered_data) * sig ** 2
-        iters = i + 1
-    else:
-        for i in range(iters):
-            do = filtered_data - cenfunc(filtered_data)
-            filtered_data.mask |= do * do > varfunc(filtered_data) * sig ** 2
-
-    return filtered_data
+gaussian_fwhm_to_sigma = 1. / gaussian_sigma_to_fwhm
+"""
+Factor with which to multiply Gaussian full width at half maximum(s) to
+convert them to 1-sigma standard deviation(s).
+"""
 
 
 # TODO Note scipy dependency
@@ -573,7 +452,7 @@ def median_absolute_deviation(a, axis=None):
 
     Parameters
     ----------
-    a : array_like
+    a : array-like
         Input array or object that can be converted to an array.
     axis : int, optional
         Axis along which the medians are computed. The default (axis=None)
@@ -642,16 +521,16 @@ def biweight_location(a, c=6.0, M=None):
 
     Parameters
     ----------
-    a : array_like
+    a : array-like
         Input array or object that can be converted to an array.
     c : float
         Tuning constant for the biweight estimator.  Default value is 6.0.
     M : float, optional
-        Initial gues for the biweight location.
+        Initial guess for the biweight location.
 
     Returns
     -------
-    biweight_location: float
+    biweight_location : float
         Returns the biweight location for the array elements.
 
     Examples
@@ -685,8 +564,8 @@ def biweight_location(a, c=6.0, M=None):
     # now remove the outlier points
     mask = np.abs(u) < 1
 
-    u = (1 - u**2)**2
-    return M+(d[mask]*u[mask]).sum()/u[mask].sum()
+    u = (1 - u ** 2) ** 2
+    return M + (d[mask] * u[mask]).sum() / u[mask].sum()
 
 
 def biweight_midvariance(a, c=9.0, M=None):
@@ -700,7 +579,7 @@ def biweight_midvariance(a, c=9.0, M=None):
 
     .. math::
 
-      C_{bl}= n^{1/2} \\frac{[\Sigma_{|u_i|<1} (x_i-M)**2(1-u_i^2)^4]^{0.5}}
+      C_{bl}= (n')^{1/2} \\frac{[\Sigma_{|u_i|<1} (x_i-M)^2(1-u_i^2)^4]^{0.5}}
       {|\Sigma_{|u_i|<1} (1-u_i^2)(1-5u_i^2)|}
 
     where :math:`u_i` is given by
@@ -709,19 +588,30 @@ def biweight_midvariance(a, c=9.0, M=None):
 
       u_{i} = \\frac{(x_i-M)}{cMAD}
 
-    where MAD is the median absolute deviation.  For the midvariance
-    parameter, c is typically uses a value of 9.0.
+    where MAD is the median absolute deviation.
+
+    :math:`n'` is the number of data for which :math:`|u_i| < 1` holds, while the
+    summations are over all i up to n:
+
+    .. math::
+
+        n' = \Sigma_{|u_i|<1}^n 1
+
+    This is slightly different than given in the reference below, but
+    results in a value closer to the true midvariance.
+
+    The midvariance parameter c is typically 9.0.
 
     For more details, see Beers, Flynn, and Gebhardt, 1990, AJ, 100, 32B
 
     Parameters
     ----------
-    a : array_like
+    a : array-like
         Input array or object that can be converted to an array.
     c : float
         Tuning constant for the biweight estimator.  Default value is 9.0.
     M : float, optional
-        Initial gues for the biweight location.
+        Initial guess for the biweight location.
 
     Returns
     -------
@@ -758,9 +648,9 @@ def biweight_midvariance(a, c=9.0, M=None):
     # now remove the outlier points
     mask = np.abs(u) < 1
 
-    u = u**2
+    u = u ** 2
     n = mask.sum()
-    return n**0.5 * (d[mask] * d[mask] * (1 - u[mask])**4).sum()**0.5\
+    return n ** 0.5 * (d[mask] * d[mask] * (1 - u[mask]) ** 4).sum() ** 0.5\
         / np.abs(((1 - u[mask]) * (1 - 5 * u[mask])).sum())
 
 
@@ -787,10 +677,10 @@ def signal_to_noise_oir_ccd(t, source_eps, sky_eps, dark_eps, rd, npix,
         make sense.
     dark_eps : float
         Number of thermal electrons per second per pixel. If this is given in
-        DN or ADU, then multipy by the gain to get the value in electrons.
+        DN or ADU, then multiply by the gain to get the value in electrons.
     rd : float
         Read noise of the CCD in electrons. If this is given in
-        DN or ADU, then multipy by the gain to get the value in electrons.
+        DN or ADU, then multiply by the gain to get the value in electrons.
     npix : float
         Size of the aperture in pixels
     gain : float
@@ -801,9 +691,9 @@ def signal_to_noise_oir_ccd(t, source_eps, sky_eps, dark_eps, rd, npix,
     SNR : float or numpy.ndarray
         Signal to noise ratio calculated from the inputs
     """
-    signal = t*source_eps*gain
+    signal = t * source_eps * gain
     noise = np.sqrt(t * (source_eps * gain + npix *
-                         (sky_eps * gain + dark_eps)) + npix * rd**2)
+                         (sky_eps * gain + dark_eps)) + npix * rd ** 2)
     return signal / noise
 
 
@@ -819,7 +709,7 @@ def bootstrap(data, bootnum=100, samples=None, bootfunc=None):
     Parameters
     ----------
     data : numpy.ndarray
-        N-D array. The boostrap resampling will be performed on the first
+        N-D array. The bootstrap resampling will be performed on the first
         index, so the first index should access the relevant information
         to be bootstrapped.
     bootnum : int
@@ -860,3 +750,43 @@ def bootstrap(data, bootnum=100, samples=None, bootfunc=None):
             boot[i] = bootfunc(data[bootarr])
 
     return boot
+
+
+def mad_std(data):
+    """
+    Calculate a robust standard deviation using the `median absolute
+    deviation (MAD)
+    <http://en.wikipedia.org/wiki/Median_absolute_deviation>`_.
+
+    The standard deviation estimator is given by:
+
+    .. math::
+
+        \\sigma \\approx \\frac{\\textrm{MAD}}{\Phi^{-1}(3/4)} \\approx 1.4826 \ \\textrm{MAD}
+
+    where :math:`\Phi^{-1}(P)` is the normal inverse cumulative
+    distribution function evaluated at probability :math:`P = 3/4`.
+
+    Parameters
+    ----------
+    data : array-like
+        Data array or object that can be converted to an array.
+
+    Returns
+    -------
+    result : float
+        The robust standard deviation of the data.
+
+    Examples
+    --------
+    >>> from astropy.stats import mad_std
+    >>> from astropy.utils import NumpyRNGContext
+    >>> from numpy.random import normal
+    >>> with NumpyRNGContext(12345):
+    ...     data = normal(5, 2, size=(100, 100))
+    ...     mad_std(data)    # doctest: +FLOAT_CMP
+    2.02327646594
+    """
+
+    # NOTE: 1. / scipy.stats.norm.ppf(0.75) = 1.482602218505602
+    return median_absolute_deviation(data) * 1.482602218505602
