@@ -20,12 +20,12 @@ conf = Conf()
 
 from .column import Column, MaskedColumn
 from .groups import TableGroups, ColumnGroups
-from .table import Table, TableColumns, Row, TableFormatter
-from .np_utils import TableMergeError
-from .operations import join, hstack, vstack
+from .table import Table, QTable, TableColumns, Row, TableFormatter
+from .operations import join, hstack, vstack, unique, TableMergeError
 
 # Import routines that connect readers/writers to astropy.table
 from ..io.ascii import connect
 from ..io.fits import connect
 from ..io.misc import connect
 from ..io.votable import connect
+from . import jsviewer
