@@ -3,18 +3,15 @@ from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 from ..extern import six
 
-import operator
-
 import weakref
 
 from copy import deepcopy
-from distutils import version
 
 import numpy as np
 from numpy import ma
 
 from ..units import Unit, Quantity
-from ..utils import deprecated
+from ..utils.compat import NUMPY_LT_1_8
 from ..utils.console import color_print
 from ..utils.metadata import MetaData
 from . import groups
@@ -23,7 +20,6 @@ from .np_utils import fix_column_name
 
 from ..config import ConfigAlias
 
-NUMPY_VERSION = version.LooseVersion(np.__version__)
 
 AUTO_COLNAME = ConfigAlias(
     '0.4', 'AUTO_COLNAME', 'auto_colname',
@@ -32,7 +28,7 @@ AUTO_COLNAME = ConfigAlias(
 # Create a generic TableFormatter object for use by bare columns with no
 # parent table.
 FORMATTER = pprint.TableFormatter()
-
+INTEGER_TYPES = (int, long, np.integer) if six.PY2 else (int, np.integer)
 
 def _auto_names(n_cols):
     from . import conf
@@ -48,13 +44,137 @@ _comparison_functions = set(
      np.isfinite, np.isinf, np.isnan, np.sign, np.signbit])
 
 
+COLUMN_ATTRS = set(['name', 'unit', 'dtype', 'format', 'description', 'meta', 'parent_table'])
+
+def col_setattr(col, attr, value):
+    """
+    Set one of the column attributes.
+
+    Warning: this function is subject to change or removal.
+    """
+    if attr not in COLUMN_ATTRS:
+        raise AttributeError("attribute must be one of {0}".format(COLUMN_ATTRS))
+
+    # The unit and dtype attributes are considered universal and do NOT get
+    # stored in _astropy_column_attrs.  For BaseColumn instances use the usual setattr.
+    if isinstance(col, BaseColumn):
+        setattr(col, attr, value)
+    else:
+        # If no _astropy_column_attrs or it is None then convert to dict
+        if getattr(col, '_astropy_column_attrs', None) is None:
+            col._astropy_column_attrs = {}
+        if attr == 'parent_table':
+            value = None if value is None else weakref.ref(value)
+        col._astropy_column_attrs[attr] = value
+
+def col_getattr(col, attr, default=None):
+    """
+    Get one of the column attributes
+
+    Warning: this function is subject to change or removal.
+    """
+    if attr not in COLUMN_ATTRS:
+        raise AttributeError("attribute must be one of {0}".format(COLUMN_ATTRS))
+
+    # The unit and dtype attributes are considered universal and do NOT get
+    # stored in _astropy_column_attrs.  For BaseColumn instances use the usual setattr.
+    if (isinstance(col, BaseColumn) or
+            (isinstance(col, Quantity) and attr in ('dtype', 'unit'))):
+        value = getattr(col, attr, default)
+    else:
+        # If col does not have _astropy_column_attrs or it is None (meaning
+        # nothing has been set yet) then return default, otherwise look for
+        # the attribute in the astropy_column_attrs dict.
+        if getattr(col, '_astropy_column_attrs', None) is None:
+            value = default
+        else:
+            value = col._astropy_column_attrs.get(attr, default)
+
+        # Weak ref for parent table
+        if attr == 'parent_table' and callable(value):
+            value = value()
+
+        # Mixins have a default dtype of Object if nothing else was set
+        if attr == 'dtype' and value is None:
+            value = np.dtype('O')
+
+    return value
+
+def _col_update_attrs_from(newcol, col, exclude_attrs=['name', 'parent_table']):
+    """
+    Update _astropy_column_attrs from mixin `col` to `newcol`.  Does nothing
+    for BaseColumn cols
+
+    Warning: this function is subject to change or removal.
+    """
+    if isinstance(newcol, BaseColumn):
+        return
+
+    attrs = COLUMN_ATTRS - set(exclude_attrs)
+    for attr in attrs:
+        val = col_getattr(col, attr)
+        if val is not None:
+            col_setattr(newcol, attr, deepcopy(val))
+
+def col_iter_str_vals(col):
+    """
+    This is a mixin-safe version of Column.iter_str_vals.
+
+    Warning: this function is subject to change or removal.
+    """
+    parent_table = col_getattr(col, 'parent_table')
+    formatter = FORMATTER if parent_table is None else parent_table.formatter
+    _pformat_col_iter = formatter._pformat_col_iter
+    for str_val in _pformat_col_iter(col, -1, False, False, {}):
+        yield str_val
+
+def col_copy(col):
+    """
+    This is a mixin-safe version of Column.copy() (with copy_data=True).
+
+    Warning: this function is subject to change or removal.
+    """
+    if isinstance(col, BaseColumn):
+        return col.copy()
+
+    if hasattr(col, '_astropy_column_attrs'):
+        col_setattr(col, 'parent_table', None)  # Don't copy weakref to parent table
+    newcol = col.copy() if hasattr(col, 'copy') else deepcopy(col)
+
+    # Copy old attributes.  Even deepcopy above may not get this (e.g. pandas).
+    if (not hasattr(newcol, '_astropy_column_attrs') or
+            newcol._astropy_column_attrs is None):
+        _column_attrs = deepcopy(getattr(col, '_astropy_column_attrs', {}))
+        newcol._astropy_column_attrs = _column_attrs
+
+    return newcol
+
+
+class FalseArray(np.ndarray):
+    def __new__(cls, shape):
+        obj = np.zeros(shape, dtype=np.bool).view(cls)
+        return obj
+
+    def __setitem__(self, item, val):
+        val = np.asarray(val)
+        if np.any(val):
+            raise ValueError('Cannot set any element of {0} class to True'
+                             .format(self.__class__.__name__))
+
+    def __setslice__(self, start, stop, val):
+        val = np.asarray(val)
+        if np.any(val):
+            raise ValueError('Cannot set any element of {0} class to True'
+                             .format(self.__class__.__name__))
+
+
 class BaseColumn(np.ndarray):
 
     meta = MetaData()
 
     def __new__(cls, data=None, name=None,
                 dtype=None, shape=(), length=0,
-                description=None, unit=None, format=None, meta=None):
+                description=None, unit=None, format=None, meta=None, copy=False):
 
         if data is None:
             dtype = (np.dtype(dtype).str, shape)
@@ -64,7 +184,7 @@ class BaseColumn(np.ndarray):
             # BaseColumn with none of the expected attributes.  In this case
             # do NOT execute this block which initializes from ``data``
             # attributes.
-            self_data = np.asarray(data.data, dtype=dtype)
+            self_data = np.array(data.data, dtype=dtype, copy=copy)
             if description is None:
                 description = data.description
             if unit is None:
@@ -77,12 +197,19 @@ class BaseColumn(np.ndarray):
                 name = data.name
         elif isinstance(data, Quantity):
             if unit is None:
-                self_data = np.asarray(data, dtype=dtype)
+                self_data = np.array(data, dtype=dtype, copy=copy)
                 unit = data.unit
             else:
-                self_data = np.asarray(data.to(unit), dtype=dtype)
+                self_data = np.array(data.to(unit), dtype=dtype, copy=copy)
+            if description is None:
+                description = col_getattr(data, 'description')
+            if format is None:
+                format = col_getattr(data, 'format')
+            if meta is None:
+                meta = deepcopy(col_getattr(data, 'meta'))
+
         else:
-            self_data = np.asarray(data, dtype=dtype)
+            self_data = np.array(data, dtype=dtype, copy=copy)
 
         self = self_data.view(cls)
         self._name = fix_column_name(name)
@@ -138,7 +265,7 @@ class BaseColumn(np.ndarray):
 
         Returns
         -------
-        col: Column or MaskedColumn
+        col : Column or MaskedColumn
             Copy of the current column (same type as original)
         """
         if data is None:
@@ -196,6 +323,11 @@ class BaseColumn(np.ndarray):
 
         return reconstruct_func, reconstruct_func_args, state
 
+    def __getitem__(self, item):
+        if isinstance(item, INTEGER_TYPES):
+            return self.data[item]  # Return as plain ndarray or ma.MaskedArray
+        else:
+            return super(BaseColumn, self).__getitem__(item)
 
     # avoid == and != to be done based on type of subclass
     # (helped solve #1446; see also __array_wrap__)
@@ -217,10 +349,7 @@ class BaseColumn(np.ndarray):
         # or viewcast e.g. obj.view(Column).  In either case we want to
         # init Column attributes for self from obj if possible.
         self.parent_table = None
-        for attr in ('name', 'unit', 'format', 'description'):
-            val = getattr(obj, attr, None)
-            setattr(self, attr, val)
-        self.meta = deepcopy(getattr(obj, 'meta', {}))
+        self._copy_attrs(obj)
 
     def __array_wrap__(self, out_arr, context=None):
         """
@@ -252,6 +381,9 @@ class BaseColumn(np.ndarray):
 
     @property
     def name(self):
+        """
+        The name of this column.
+        """
         return self._name
 
     @name.setter
@@ -261,9 +393,6 @@ class BaseColumn(np.ndarray):
         if self.parent_table is not None:
             table = self.parent_table
             table.columns._rename_column(self.name, val)
-            table._data.dtype.names = list(table.columns)
-            if table.masked:
-                table._data.mask.dtype.names = list(table.columns)
 
         self._name = val
 
@@ -289,7 +418,8 @@ class BaseColumn(np.ndarray):
         # Iterate over formatted values with no max number of lines, no column
         # name, no unit, and ignoring the returned header info in outs.
         _pformat_col_iter = self._formatter._pformat_col_iter
-        for str_val in _pformat_col_iter(self, -1, False, False, {}):
+        for str_val in _pformat_col_iter(self, -1, show_name=False, show_unit=False,
+                                         show_dtype=False, outs={}):
             yield str_val
 
     def attrs_equal(self, col):
@@ -305,7 +435,7 @@ class BaseColumn(np.ndarray):
 
         Returns
         -------
-        equal: boolean
+        equal : boolean
             True if all attributes are equal
         """
         if not isinstance(col, BaseColumn):
@@ -321,7 +451,8 @@ class BaseColumn(np.ndarray):
     def _formatter(self):
         return FORMATTER if (self.parent_table is None) else self.parent_table.formatter
 
-    def pformat(self, max_lines=None, show_name=True, show_unit=False):
+    def pformat(self, max_lines=None, show_name=True, show_unit=False, show_dtype=False,
+                html=False):
         """Return a list of formatted string representation of column values.
 
         If no value of ``max_lines`` is supplied then the height of the
@@ -342,6 +473,12 @@ class BaseColumn(np.ndarray):
         show_unit : bool
             Include a header row for unit (default=False)
 
+        show_dtype : bool
+            Include column dtype (default=False)
+
+        html : bool
+            Format the output as an HTML table (default=False)
+
         Returns
         -------
         lines : list
@@ -349,10 +486,12 @@ class BaseColumn(np.ndarray):
 
         """
         _pformat_col = self._formatter._pformat_col
-        lines, n_header = _pformat_col(self, max_lines, show_name, show_unit)
+        lines, outs = _pformat_col(self, max_lines, show_name=show_name,
+                                   show_unit=show_unit, show_dtype=show_dtype,
+                                   html=html)
         return lines
 
-    def pprint(self, max_lines=None, show_name=True, show_unit=False):
+    def pprint(self, max_lines=None, show_name=True, show_unit=False, show_dtype=False):
         """Print a formatted string representation of column values.
 
         If no value of ``max_lines`` is supplied then the height of the
@@ -372,9 +511,15 @@ class BaseColumn(np.ndarray):
 
         show_unit : bool
             Include a header row for unit (default=False)
+
+        show_dtype : bool
+            Include column dtype (default=True)
         """
         _pformat_col = self._formatter._pformat_col
-        lines, n_header = _pformat_col(self, max_lines, show_name, show_unit)
+        lines, outs = _pformat_col(self, max_lines, show_name=show_name, show_unit=show_unit,
+                                   show_dtype=show_dtype)
+
+        n_header = outs['n_header']
         for i, line in enumerate(lines):
             if i < n_header:
                 color_print(line, 'red')
@@ -434,21 +579,6 @@ class BaseColumn(np.ndarray):
     def unit(self):
         self._unit = None
 
-    @property
-    @deprecated('0.3', alternative=':attr:`Column.unit`')
-    def units(self):
-        return self.unit
-
-    @units.setter
-    @deprecated('0.3', alternative=':attr:`Column.unit`')
-    def units(self, unit):
-        self.unit = unit
-
-    @units.deleter
-    @deprecated('0.3', alternative=':attr:`Column.unit`')
-    def units(self):
-        del self.unit
-
     def convert_unit_to(self, new_unit, equivalencies=[]):
         """
         Converts the values of the column in-place from the current
@@ -524,6 +654,48 @@ class BaseColumn(np.ndarray):
     def __repr__(self):
         return np.asarray(self).__repr__()
 
+    @property
+    def quantity(self):
+        """
+        A view of this table column as a `~astropy.units.Quantity` object with
+        units given by the Column's `unit` parameter.
+        """
+        # the Quantity initializer is used here because it correctly fails
+        # if the column's values are non-numeric (like strings), while .view
+        # will happily return a quantity with gibberish for numerical values
+        return Quantity(self, copy=False, dtype=self.dtype, order='A')
+
+    def to(self, unit, equivalencies=[], **kwargs):
+        """
+        Converts this table column to a `~astropy.units.Quantity` object with
+        the requested units.
+
+        Parameters
+        ----------
+        unit : `~astropy.units.Unit` or str
+            The unit to convert to (i.e., a valid argument to the
+            :meth:`astropy.units.Quantity.to` method).
+        equivalencies : list of equivalence pairs, optional
+            Equivalencies to use for this conversion.  See
+            :meth:`astropy.units.Quantity.to` for more details.
+
+        Returns
+        -------
+        quantity : `~astropy.units.Quantity`
+            A quantity object with the contents of this column in the units
+            ``unit``.
+        """
+        return self.quantity.to(unit, equivalencies)
+
+    def _copy_attrs(self, obj):
+        """
+        Copy key column attributes from ``obj`` to self
+        """
+        for attr in ('name', 'unit', 'format', 'description'):
+            val = getattr(obj, attr, None)
+            setattr(self, attr, val)
+        self.meta = deepcopy(getattr(obj, 'meta', {}))
+
 
 class Column(BaseColumn):
     """Define a data column for use in a Table object.
@@ -592,29 +764,54 @@ class Column(BaseColumn):
 
     def __new__(cls, data=None, name=None,
                 dtype=None, shape=(), length=0,
-                description=None, unit=None, format=None, meta=None):
+                description=None, unit=None, format=None, meta=None, copy=False):
 
         if isinstance(data, MaskedColumn) and np.any(data.mask):
             raise TypeError("Cannot convert a MaskedColumn with masked value to a Column")
 
         self = super(Column, cls).__new__(cls, data=data, name=name, dtype=dtype,
                                           shape=shape, length=length, description=description,
-                                          unit=unit, format=format, meta=meta)
+                                          unit=unit, format=format, meta=meta, copy=copy)
         return self
 
-    def __repr__(self):
-        unit = None if self.unit is None else six.text_type(self.unit)
-        out = "<{0} name={1} unit={2} format={3} " \
-            "description={4}>\n{5}".format(
-            self.__class__.__name__,
-            repr(self.name), repr(unit),
-            repr(self.format), repr(self.description), repr(self.data))
+    def _base_repr_(self, html=False):
+        descr_vals = [self.__class__.__name__]
+        unit = None if self.unit is None else str(self.unit)
+        shape = None if self.ndim <= 1 else self.shape[1:]
+        for attr, val in (('name', self.name),
+                          ('dtype', self.dtype.name),
+                          ('shape', shape),
+                          ('unit', unit),
+                          ('format', self.format),
+                          ('description', self.description),
+                          ('length', len(self))):
+
+            if val is not None:
+                descr_vals.append('{0}={1}'.format(attr, repr(val)))
+
+        descr = '<' + ' '.join(descr_vals) + '>\n'
+
+        if html:
+            from ..utils.xml.writer import xml_escape
+            descr = xml_escape(descr)
+
+        data_lines, outs = self._formatter._pformat_col(
+            self, show_name=False, show_unit=False, show_length=False, html=html)
+
+        out = descr + '\n'.join(data_lines)
+        if six.PY2 and isinstance(out, six.text_type):
+            out = out.encode('utf-8')
 
         return out
 
+    def _repr_html_(self):
+        return self._base_repr_(html=True)
+
+    def __repr__(self):
+        return self._base_repr_(html=False)
+
     def __unicode__(self):
-        _pformat_col = self._formatter._pformat_col
-        lines, n_header = _pformat_col(self)
+        lines, outs = self._formatter._pformat_col(self)
         return '\n'.join(lines)
     if six.PY3:
         __str__ = __unicode__
@@ -624,13 +821,62 @@ class Column(BaseColumn):
     if six.PY2:
         __str__ = __bytes__
 
+    # Set items using a view of the underlying data, as it gives an
+    # order-of-magnitude speed-up. [#2994]
+    def __setitem__(self, index, value):
+        self.data[index] = value
+
+    # # Set slices using a view of the underlying data, as it gives an
+    # # order-of-magnitude speed-up.  Only gets called in Python 2.  [#3020]
+    def __setslice__(self, start, stop, value):
+        self.data.__setslice__(start, stop, value)
+
+    def insert(self, obj, values):
+        """
+        Insert values before the given indices in the column and return
+        a new `~astropy.table.Column` object.
+
+        Parameters
+        ----------
+        obj : int, slice or sequence of ints
+            Object that defines the index or indices before which ``values`` is
+            inserted.
+        values : array_like
+            Value(s) to insert.  If the type of ``values`` is different
+            from that of quantity, ``values`` is converted to the matching type.
+            ``values`` should be shaped so that it can be broadcast appropriately
+
+        Returns
+        -------
+        out : `~astropy.table.Column`
+            A copy of column with ``values`` and ``mask`` inserted.  Note that the
+            insertion does not occur in-place: a new column is returned.
+        """
+        if self.dtype.kind == 'O':
+            # Even if values is array-like (e.g. [1,2,3]), insert as a single
+            # object.  Numpy.insert instead inserts each element in an array-like
+            # input individually.
+            data = np.insert(self, obj, None, axis=0)
+            data[obj] = values
+        else:
+            # Explicitly convert to dtype of this column.  Needed because numpy 1.7
+            # enforces safe casting by default, so .  This isn't the case for 1.6 or 1.8+.
+            values = np.asarray(values, dtype=self.dtype)
+            data = np.insert(self, obj, values, axis=0)
+        out = data.view(self.__class__)
+        out.__array_finalize__(self)
+        return out
+
     # We do this to make the methods show up in the API docs
     name = BaseColumn.name
+    unit = BaseColumn.unit
     copy = BaseColumn.copy
     more = BaseColumn.more
     pprint = BaseColumn.pprint
     pformat = BaseColumn.pformat
     convert_unit_to = BaseColumn.convert_unit_to
+    quantity = BaseColumn.quantity
+    to = BaseColumn.to
 
 
 class MaskedColumn(Column, ma.MaskedArray):
@@ -707,7 +953,7 @@ class MaskedColumn(Column, ma.MaskedArray):
 
     def __new__(cls, data=None, name=None, mask=None, fill_value=None,
                 dtype=None, shape=(), length=0,
-                description=None, unit=None, format=None, meta=None):
+                description=None, unit=None, format=None, meta=None, copy=False):
 
         if mask is None and hasattr(data, 'mask'):
             mask = data.mask
@@ -723,7 +969,7 @@ class MaskedColumn(Column, ma.MaskedArray):
         # First just pass through all args and kwargs to BaseColumn, then wrap that object
         # with MaskedArray.
         self_data = BaseColumn(data, dtype=dtype, shape=shape, length=length, name=name,
-                               unit=unit, format=format, description=description, meta=meta)
+                               unit=unit, format=format, description=description, meta=meta, copy=copy)
         self = ma.MaskedArray.__new__(cls, data=self_data, mask=mask)
 
         # Note: do not set fill_value in the MaskedArray constructor because this does not
@@ -745,8 +991,9 @@ class MaskedColumn(Column, ma.MaskedArray):
         3).  Here we change the string to a byte string so that in Python 3 the
         isinstance(val, basestring) part fails.
         """
-        if (NUMPY_VERSION < version.LooseVersion('1.8.0') and
-                isinstance(val, six.string_types) and (self.dtype.char not in 'SV')):
+
+        if (NUMPY_LT_1_8 and isinstance(val, six.string_types) and
+                (self.dtype.char not in 'SV')):
             val = val.encode()
         return val
 
@@ -760,9 +1007,6 @@ class MaskedColumn(Column, ma.MaskedArray):
         if it exists.  Setting one or the other alone doesn't work."""
         val = self._fix_fill_value(val)
 
-        if self.parent_table:
-            self.parent_table._data[self._name].fill_value = val
-
         # Yet another ma bug workaround: If the value of fill_value for a string array is
         # requested but not yet set then it gets created as 'N/A'.  From this point onward
         # any new fill_values are truncated to 3 characters.  Note that this does not
@@ -817,6 +1061,77 @@ class MaskedColumn(Column, ma.MaskedArray):
                          meta=deepcopy(self.meta))
         return out
 
+    def insert(self, obj, values, mask=None):
+        """
+        Insert values along the given axis before the given indices and return
+        a new `~astropy.table.MaskedColumn` object.
+
+        Parameters
+        ----------
+        obj : int, slice or sequence of ints
+            Object that defines the index or indices before which ``values`` is
+            inserted.
+        values : array_like
+            Value(s) to insert.  If the type of ``values`` is different
+            from that of quantity, ``values`` is converted to the matching type.
+            ``values`` should be shaped so that it can be broadcast appropriately
+        mask : boolean array_like
+            Mask value(s) to insert.  If not supplied then False is used.
+
+        Returns
+        -------
+        out : `~astropy.table.MaskedColumn`
+            A copy of column with ``values`` and ``mask`` inserted.  Note that the
+            insertion does not occur in-place: a new masked column is returned.
+        """
+        self_ma = self.data  # self viewed as MaskedArray
+
+        if self.dtype.kind == 'O':
+            # Even if values is array-like (e.g. [1,2,3]), insert as a single
+            # object.  Numpy.insert instead inserts each element in an array-like
+            # input individually.
+            new_data = np.insert(self_ma.data, obj, None, axis=0)
+            new_data[obj] = values
+        else:
+            # Explicitly convert to dtype of this column.  Needed because numpy 1.7
+            # enforces safe casting by default, so .  This isn't the case for 1.6 or 1.8+.
+            values = np.asarray(values, dtype=self.dtype)
+            new_data = np.insert(self_ma.data, obj, values, axis=0)
+
+        if mask is None:
+            if self.dtype.kind == 'O':
+                mask = False
+            else:
+                mask = np.zeros(values.shape, dtype=np.bool)
+        new_mask = np.insert(self_ma.mask, obj, mask, axis=0)
+        new_ma = np.ma.array(new_data, mask=new_mask, copy=False)
+
+        out = new_ma.view(self.__class__)
+        out.parent_table = None
+        out._copy_attrs(self)
+
+        return out
+
+    def __getitem__(self, item):
+        out = super(MaskedColumn, self).__getitem__(item)
+
+        # Fixes issue #3023: when calling getitem with a MaskedArray subclass
+        # the original object attributes are not copied.
+        if out.__class__ is self.__class__:
+            out.parent_table = None
+            out._copy_attrs(self)
+
+        return out
+
+    # Set items and slices using MaskedArray method, instead of falling through
+    # to the (faster) Column version which uses an ndarray view.  This doesn't
+    # copy the mask properly. See test_setting_from_masked_column test.
+    def __setitem__(self, index, value):
+        ma.MaskedArray.__setitem__(self, index, value)
+
+    def __setslice__(self, start, stop, value):
+        ma.MaskedArray.__setslice__(self, start, stop, value)
+
     # We do this to make the methods show up in the API docs
     name = BaseColumn.name
     copy = BaseColumn.copy
