@@ -107,7 +107,7 @@ def column_group_by(column, keys):
     from .table import Table
 
     if isinstance(keys, Table):
-        keys = keys._data
+        keys = keys.as_array()
 
     if not isinstance(keys, np.ndarray):
         raise TypeError('Keys input must be numpy array, but got {0}'
@@ -216,8 +216,8 @@ class ColumnGroups(BaseGroups):
             return self._keys
 
     def aggregate(self, func):
-        from .column import MaskedColumn
-        
+        from .column import MaskedColumn, col_getattr
+
         i0s, i1s = self.indices[:-1], self.indices[1:]
         par_col = self.parent_column
         masked = isinstance(par_col, MaskedColumn)
@@ -235,11 +235,16 @@ class ColumnGroups(BaseGroups):
             else:
                 vals = np.array([func(par_col[i0: i1]) for i0, i1 in izip(i0s, i1s)])
         except Exception:
-            raise TypeError("Cannot aggregate column '{0}'"
-                            .format(par_col.name))
+            raise TypeError("Cannot aggregate column '{0}' with type '{1}'"
+                            .format(col_getattr(par_col, 'name'),
+                                    col_getattr(par_col, 'dtype')))
 
-        out = par_col.__class__(data=vals, name=par_col.name, description=par_col.description,
-                                unit=par_col.unit, format=par_col.format, meta=par_col.meta)
+        out = par_col.__class__(data=vals,
+                                name=col_getattr(par_col, 'name'),
+                                description=col_getattr(par_col, 'description'),
+                                unit=col_getattr(par_col, 'unit'),
+                                format=col_getattr(par_col, 'format'),
+                                meta=col_getattr(par_col, 'meta'))
         return out
 
     def filter(self, func):
@@ -316,13 +321,15 @@ class TableGroups(BaseGroups):
         out : Table
             New table with the aggregated rows.
         """
+        from .column import col_getattr
+
         i0s, i1s = self.indices[:-1], self.indices[1:]
         out_cols = []
         parent_table = self.parent_table
 
         for col in six.itervalues(parent_table.columns):
             # For key columns just pick off first in each group since they are identical
-            if col.name in self.key_colnames:
+            if col_getattr(col, 'name') in self.key_colnames:
                 new_col = col.take(i0s)
             else:
                 try:
