@@ -6,11 +6,12 @@ import copy
 import functools
 import sys
 
-from datetime import datetime
+from datetime import datetime, tzinfo, timedelta
 
 import numpy as np
 
-from ...tests.helper import pytest
+from ...tests.helper import pytest, catch_warnings
+from ...extern import six
 from .. import Time, ScaleValueError, erfa_time, TIME_SCALES
 from ...coordinates import EarthLocation
 
@@ -55,6 +56,26 @@ class TestBasic():
 
         assert allclose_sec(t.cxcsec, np.array([31536064.307456788, 378691266.18400002]))
 
+    def test_different_dimensions(self):
+        """Test scalars, vector, and higher-dimensions"""
+        # scalar
+        val, val1 = 2450000.0, 0.125
+        t1 = Time(val, val1, format='jd')
+        assert t1.isscalar is True and t1.shape == ()
+        # vector
+        val = np.arange(2450000., 2450010.)
+        t2 = Time(val, format='jd')
+        assert t2.isscalar is False and t2.shape == val.shape
+        # explicitly check broadcasting for mixed vector, scalar.
+        val2 = 0.
+        t3 = Time(val, val2, format='jd')
+        assert t3.isscalar is False and t3.shape == val.shape
+        val2 = (np.arange(5.)/10.).reshape(5, 1)
+        # now see if broadcasting to two-dimensional works
+        t4 = Time(val, val2, format='jd')
+        assert t4.isscalar is False
+        assert t4.shape == np.broadcast(val, val2).shape
+
     def test_copy_time(self):
         """Test copying the values of a Time object by passing it into the
         Time initializer.
@@ -87,7 +108,7 @@ class TestBasic():
         t = Time(mjd, format='mjd', scale='utc', location=('45d', '50d'))
         t1 = t[3]
         assert t1.isscalar is True
-        assert np.all(t1._time.jd1 == np.array([t._time.jd1[3]]))
+        assert t1._time.jd1 == t._time.jd1[3]
         assert t1.location is t.location
         t1a = Time(mjd[3], format='mjd', scale='utc')
         assert t1a.isscalar is True
@@ -124,6 +145,19 @@ class TestBasic():
         assert t4.location.view(np.ndarray)[5] != allzeros
         t6.location.view(np.ndarray)[-1] = allzeros
         assert t4.location.view(np.ndarray)[5] == allzeros
+        # Test subscription also works for two-dimensional arrays.
+        frac = np.arange(0., 0.999, 0.2)
+        t7 = Time(mjd[:, np.newaxis] + frac, format='mjd', scale='utc',
+                  location=('45d', '50d'))
+        assert t7[0, 0]._time.jd1 == t7._time.jd1[0, 0]
+        assert t7[0, 0].isscalar is True
+        assert np.all(t7[5]._time.jd1 == t7._time.jd1[5])
+        assert np.all(t7[5]._time.jd2 == t7._time.jd2[5])
+        assert np.all(t7[:, 2]._time.jd1 == t7._time.jd1[:, 2])
+        assert np.all(t7[:, 2]._time.jd2 == t7._time.jd2[:, 2])
+        assert np.all(t7[:, 0]._time.jd1 == t._time.jd1)
+        assert np.all(t7[:, 0]._time.jd2 == t._time.jd2)
+
 
     def test_properties(self):
         """Use properties to convert scales and formats.  Note that the UT1 to
@@ -211,7 +245,7 @@ class TestBasic():
     def test_location_array(self):
         """Check that location arrays are checked for size and used
         for the corresponding times.  Also checks that erfa_time.d_tdb_tt
-        can handle array-valued locations.
+        can handle array-valued locations, and can broadcast these if needed.
         """
 
         lat = 19.48125
@@ -234,6 +268,23 @@ class TestBasic():
             Time(['2006-01-15 21:24:37.5']*3, format='iso', scale='utc',
                  precision=6, location=(np.array([lon, 0]),
                                         np.array([lat, 0])))
+        # multidimensional
+        mjd = np.arange(50000., 50008.).reshape(4, 2)
+        t3 = Time(mjd, format='mjd', scale='utc', location=(lon, lat))
+        assert t3.shape == (4, 2)
+        assert t3.location.shape == ()
+        assert t3.tdb.shape == (4, 2)
+        t4 = Time(mjd, format='mjd', scale='utc',
+                  location=(np.array([lon, 0]), np.array([lat, 0])))
+        assert t4.shape == (4, 2)
+        assert t4.location.shape == (2,)
+        assert t4.tdb.shape == (4, 2)
+        t5 = Time(mjd, format='mjd', scale='utc',
+                  location=(np.array([[lon], [0], [0], [0]]),
+                            np.array([[lat], [0], [0], [0]])))
+        assert t5.shape == (4, 2)
+        assert t5.location.shape == (4, 1)
+        assert t5.tdb.shape == (4, 2)
 
     def test_all_transforms(self):
         """Test that all transforms work.  Does not test correctness,
@@ -242,7 +293,7 @@ class TestBasic():
         lon = -155.933222
         for scale1 in TIME_SCALES:
             t1 = Time('2006-01-15 21:24:37.5', format='iso', scale=scale1,
-                      lat=lat, lon=lon)
+                      location=(lon, lat))
             for scale2 in TIME_SCALES:
                 t2 = getattr(t1, scale2)
                 t21 = getattr(t2, scale1)
@@ -250,6 +301,7 @@ class TestBasic():
 
     def test_creating_all_formats(self):
         """Create a time object using each defined format"""
+        Time(2000.5, format='decimalyear')
         Time(100.0, format='cxcsec')
         Time(100.0, format='unix')
         Time(100.0, format='gps')
@@ -289,6 +341,17 @@ class TestBasic():
         t = Time('2000-01-01 01:01:01.123456789', scale='tai')
         assert t.datetime == datetime(2000, 1, 1, 1, 1, 1, 123457)
 
+        # broadcasting
+        dt3 = (dt + (dt2-dt)*np.arange(12)).reshape(4, 3)
+        t3 = Time(dt3, scale='utc')
+        assert t3.shape == (4, 3)
+        assert t3[2, 1].value == dt3[2, 1]
+        assert t3[2, 1] == Time(dt3[2, 1])
+        assert np.all(t3.value == dt3)
+        assert np.all(t3[1].value == dt3[1])
+        assert np.all(t3[:, 2] == Time(dt3[:, 2]))
+        assert Time(t3[2, 0]) == t3[2, 0]
+
     def test_epoch_transform(self):
         """Besselian and julian epoch transforms"""
         jd = 2457073.05631
@@ -319,6 +382,9 @@ class TestBasic():
             Time(50000.0, 'bad', format='mjd', scale='tai')
         with pytest.raises(ValueError):
             Time('2005-08-04T00:01:02.000Z', scale='tai')
+        # regression test against #3396
+        with pytest.raises(ValueError):
+            Time(np.nan, format='jd', scale='utc')
 
     def test_utc_leap_sec(self):
         """Time behaves properly near or in UTC leap second.  This
@@ -377,6 +443,13 @@ class TestBasic():
         assert t3.format == t1.format  # yday
         assert np.all(t3.value == np.concatenate([[t1.tt.yday], t2.tt.yday]))
 
+        # OK, how likely is this... but might as well test.
+        mjd = np.arange(50000., 50006.)
+        frac = np.arange(0., 0.999, 0.2)
+        t4 = Time(mjd[:, np.newaxis] + frac, format='mjd', scale='utc')
+        t5 = Time([t4[:2], t4[4:5]])
+        assert t5.shape == (3, 5)
+
 
 class TestVal2():
     """Tests related to val2"""
@@ -392,11 +465,13 @@ class TestVal2():
         assert t.mjd[0] == t.mjd[1]
         assert t.jd[0] == t.jd[1]
 
-    def test_val_matches_val2(self):
+    def test_val_broadcasts_against_val2(self):
+        mjd = np.arange(50000., 50007.)
+        frac = np.arange(0., 0.999, 0.2)
+        t = Time(mjd[:, np.newaxis], frac, format='mjd', scale='utc')
+        assert t.shape == (7, 5)
         with pytest.raises(ValueError):
-            Time([0.0, 50000.0], [0.0], format='mjd', scale='tai')
-        with pytest.raises(ValueError):
-            Time([0.0], 0.0, format='mjd', scale='tai')
+            Time([0.0, 50000.0], [0.0, 1.0, 2.0], format='mjd', scale='tai')
 
 
 class TestSubFormat():
@@ -533,26 +608,29 @@ class TestSubFormat():
 
 
 class TestSofaErrors():
-    """Test that erfa_time.pyx handles erfa status return values correctly"""
+    """Test that erfa status return values are handled correctly"""
 
     def test_bad_time(self):
         iy = np.array([2000], dtype=np.intc)
         im = np.array([2000], dtype=np.intc)  # bad month
         id = np.array([2000], dtype=np.intc)  # bad day
-        djm0 = np.array([0], dtype=np.double)
-        djm = np.array([0], dtype=np.double)
         with pytest.raises(ValueError):  # bad month, fatal error
-            erfa_time.cal2jd(iy, im, id, djm0, djm)
+            djm0, djm = erfa_time.cal2jd(iy, im, id)
 
-        # Set month to a good value so now the bad day just gives a warning
+        iy[0] = -5000
         im[0] = 2
-        erfa_time.cal2jd(iy, im, id, djm0, djm)
+        with pytest.raises(ValueError):  # bad year, fatal error
+            djm0, djm = erfa_time.cal2jd(iy, im, id)
+
+        iy[0] = 2000
+        with catch_warnings() as w:
+            djm0, djm = erfa_time.cal2jd(iy, im, id)
+        assert len(w) == 1
+        assert 'bad day    (JD computed)' in six.text_type(w[0].message)
+
         assert allclose_jd(djm0, [2400000.5])
         assert allclose_jd(djm, [53574.])
 
-        # How do you test for warnings in pytest?  Test that dubious year for
-        # UTC works.
-
 
 class TestCopyReplicate():
     """Test issues related to copying and replicating data"""
@@ -640,11 +718,8 @@ def test_now():
     # times are more like microseconds.  But it seems safer in case some
     # platforms have slow clock calls or something.
 
-    version_info = sys.version_info
-    # py < 2.7 and py3 < 3.2 doesn't have `total_seconds`
-    if ((version_info[0] == 2 and version_info[1] < 7) or
-        (version_info[0] == 3 and version_info[1] < 2) or
-            version_info[0] < 2):
+    # py < 2.7 doesn't have `total_seconds`
+    if sys.version_info[:2] < (2, 7):
         total_secs = lambda td: (td.microseconds + (
             td.seconds + td.days * 24 * 3600) * 10 ** 6) / 10 ** 6.
     else:
@@ -652,6 +727,20 @@ def test_now():
     assert total_secs(dt) < 0.1
 
 
+def test_decimalyear():
+    t = Time('2001:001', format='yday')
+    assert t.decimalyear == 2001.0
+
+    t = Time(2000.0, [0.5, 0.75], format='decimalyear')
+    assert np.all(t.value == [2000.5, 2000.75])
+
+    jd0 = Time('2000:001').jd
+    jd1 = Time('2001:001').jd
+    d_jd = jd1 - jd0
+    assert np.all(t.jd == [jd0 + 0.5 * d_jd,
+                           jd0 + 0.75 * d_jd])
+
+
 def test_dir():
     t = Time('2000:001', format='yday', scale='tai')
     assert 'utc' in dir(t)
@@ -680,3 +769,16 @@ def test_byteorder():
     time_little = Time(little_endian, format='mjd')
     assert np.all(time_big == time_mjd)
     assert np.all(time_little == time_mjd)
+
+
+def test_datetime_tzinfo():
+    """
+    Test #3160 that time zone info in datetime objects is respected.
+    """
+    class TZm6(tzinfo):
+        def utcoffset(self, dt):
+            return timedelta(hours=-6)
+
+    d = datetime(2002, 1, 2, 10, 3, 4, tzinfo=TZm6())
+    t = Time(d)
+    assert t.value == datetime(2002, 1, 2, 16, 3, 4)
