@@ -17,9 +17,11 @@ import textwrap
 import warnings
 import numpy as np
 
+from ..utils.decorators import lazyproperty, deprecated
 from ..utils.exceptions import AstropyWarning
-from ..utils.misc import isiterable, InheritDocstrings, lazyproperty
-from .utils import is_effectively_unity, sanitize_scale, validate_power
+from ..utils.misc import isiterable, InheritDocstrings
+from .utils import (is_effectively_unity, sanitize_scale, validate_power,
+                    add_powers)
 from . import format as unit_format
 
 # TODO: Support functional units, e.g. log(x), ln(x)
@@ -306,14 +308,14 @@ def set_enabled_units(units):
     >>> u.m.find_equivalent_units()
       Primary name | Unit definition | Aliases
     [
-      AU           | 1.49598e+11 m   | au           ,
-      Angstrom     | 1e-10 m         | AA, angstrom ,
-      cm           | 0.01 m          | centimeter   ,
-      lyr          | 9.46073e+15 m   | lightyear    ,
-      m            | irreducible     | meter        ,
-      micron       | 1e-06 m         |              ,
-      pc           | 3.08568e+16 m   | parsec       ,
-      solRad       | 6.95508e+08 m   | R_sun, Rsun  ,
+      AU           | 1.49598e+11 m   | au, astronomical_unit ,
+      Angstrom     | 1e-10 m         | AA, angstrom          ,
+      cm           | 0.01 m          | centimeter            ,
+      lyr          | 9.46073e+15 m   | lightyear             ,
+      m            | irreducible     | meter                 ,
+      micron       | 1e-06 m         |                       ,
+      pc           | 3.08568e+16 m   | parsec                ,
+      solRad       | 6.95508e+08 m   | R_sun, Rsun           ,
     ]
     """
     # get a context with a new registry, using equivalencies of the current one
@@ -353,19 +355,19 @@ def add_enabled_units(units):
     ...
       Primary name | Unit definition | Aliases
     [
-      AU           | 1.49598e+11 m   | au               ,
-      Angstrom     | 1e-10 m         | AA, angstrom     ,
-      cm           | 0.01 m          | centimeter       ,
-      ft           | 0.3048 m        | foot             ,
-      inch         | 0.0254 m        |                  ,
-      lyr          | 9.46073e+15 m   | lightyear        ,
-      m            | irreducible     | meter            ,
-      mi           | 1609.34 m       | mile             ,
-      micron       | 1e-06 m         |                  ,
-      nmi          | 1852 m          | nauticalmile, NM ,
-      pc           | 3.08568e+16 m   | parsec           ,
-      solRad       | 6.95508e+08 m   | R_sun, Rsun      ,
-      yd           | 0.9144 m        | yard             ,
+      AU           | 1.49598e+11 m   | au, astronomical_unit ,
+      Angstrom     | 1e-10 m         | AA, angstrom          ,
+      cm           | 0.01 m          | centimeter            ,
+      ft           | 0.3048 m        | foot                  ,
+      inch         | 0.0254 m        |                       ,
+      lyr          | 9.46073e+15 m   | lightyear             ,
+      m            | irreducible     | meter                 ,
+      mi           | 1609.34 m       | mile                  ,
+      micron       | 1e-06 m         |                       ,
+      nmi          | 1852 m          | nauticalmile, NM      ,
+      pc           | 3.08568e+16 m   | parsec                ,
+      solRad       | 6.95508e+08 m   | R_sun, Rsun           ,
+      yd           | 0.9144 m        | yard                  ,
     ]
     """
     # get a context with a new registry, which is a copy of the current one
@@ -439,6 +441,19 @@ class UnitsError(Exception):
     """
 
 
+class UnitScaleError(UnitsError, ValueError):
+    """
+    Used to catch the errors involving scaled units,
+    which are not recognized by FITS format.
+    """
+    pass
+
+
+# Maintain error in old location for backward compatibility
+from .format import fits as _fits
+_fits.UnitScaleError = UnitScaleError
+
+
 class UnitsWarning(AstropyWarning):
     """
     The base class for unit-specific exceptions.
@@ -558,7 +573,7 @@ class UnitBase(object):
         """
         return [1]
 
-    def to_string(self, format='generic'):
+    def to_string(self, format=unit_format.Generic):
         """
         Output the unit in the given format as a string.
 
@@ -625,9 +640,16 @@ class UnitBase(object):
         if isinstance(m, (bytes, six.text_type)):
             return Unit(m) / self
 
-        # Cannot handle this as Unit, re-try as Quantity
+        # Cannot handle this as Unit.  Here, m cannot be a Quantity,
+        # so we make it into one, fasttracking when it does not have a unit,
+        # for the common case of <array> / <unit>.
         from .quantity import Quantity
-        return m / Quantity(1, self)
+        if hasattr(m, 'unit'):
+            result = Quantity(m)
+            result /= self
+            return result
+        else:
+            return Quantity(m, self**(-1))
 
     __truediv__ = __div__
 
@@ -644,7 +666,7 @@ class UnitBase(object):
                 return m
             return CompositeUnit(1, [self, m], [1, 1], _error_check=False)
 
-        # Cannot handle this as Unit, re-try as Quantity
+        # Cannot handle this as Unit, re-try as Quantity.
         from .quantity import Quantity
         return Quantity(1, self) * m
 
@@ -652,9 +674,16 @@ class UnitBase(object):
         if isinstance(m, (bytes, six.text_type)):
             return Unit(m) * self
 
-        # Cannot handle this as Unit, re-try as Quantity
+        # Cannot handle this as Unit.  Here, m cannot be a Quantity,
+        # so we make it into one, fasttracking when it does not have a unit
+        # for the common case of <array> * <unit>.
         from .quantity import Quantity
-        return m * Quantity(1, self)
+        if hasattr(m, 'unit'):
+            result = Quantity(m)
+            result *= self
+            return result
+        else:
+            return Quantity(m, self)
 
     def __hash__(self):
         # This must match the hash used in CompositeUnit for a unit
@@ -667,7 +696,7 @@ class UnitBase(object):
 
         try:
             other = Unit(other, parse_strict='silent')
-        except (ValueError, UnitsError):
+        except (ValueError, UnitsError, TypeError):
             return False
         try:
             return is_effectively_unity(self._to(other))
@@ -758,7 +787,7 @@ class UnitBase(object):
 
     def _apply_equivalences(self, unit, other, equivalencies):
         """
-        Internal function (used from `get_converter`) to apply
+        Internal function (used from `_get_converter`) to apply
         equivalence pairs.
         """
         def make_converter(scale1, func, scale2):
@@ -810,6 +839,17 @@ class UnitBase(object):
             "{0} and {1} are not convertible".format(
                 unit_str, other_str))
 
+    def _get_converter(self, other, equivalencies=[]):
+        other = Unit(other)
+
+        try:
+            scale = self._to(other)
+        except UnitsError:
+            return self._apply_equivalences(
+                self, other, self._normalize_equivalencies(equivalencies))
+        return lambda val: scale * _condition_arg(val)
+
+    @deprecated('1.0')
     def get_converter(self, other, equivalencies=[]):
         """
         Return the conversion function to convert values from ``self``
@@ -839,14 +879,7 @@ class UnitBase(object):
         UnitsError
             If units are inconsistent
         """
-        other = Unit(other)
-
-        try:
-            scale = self._to(other)
-        except UnitsError:
-            return self._apply_equivalences(
-                self, other, self._normalize_equivalencies(equivalencies))
-        return lambda val: scale * _condition_arg(val)
+        return self._get_converter(other, equivalencies=equivalencies)
 
     def _to(self, other):
         """
@@ -909,7 +942,7 @@ class UnitBase(object):
         UnitsError
             If units are inconsistent
         """
-        return self.get_converter(other, equivalencies=equivalencies)(value)
+        return self._get_converter(other, equivalencies=equivalencies)(value)
 
     def in_units(self, other, value=1.0, equivalencies=[]):
         """
@@ -968,9 +1001,14 @@ class UnitBase(object):
         for funit, tunit, a, b in equivalencies:
             if tunit is not None:
                 if self._is_equivalent(funit):
-                    units.append(tunit.decompose())
+                    scale = funit.decompose().scale / unit.scale
+                    units.append(Unit(a(1.0 / scale) * tunit).decompose())
                 elif self._is_equivalent(tunit):
-                    units.append(funit.decompose())
+                    scale = tunit.decompose().scale / unit.scale
+                    units.append(Unit(b(1.0 / scale) * funit).decompose())
+            else:
+                if self._is_equivalent(funit):
+                    units.append(Unit(unit.scale))
 
         # Store partial results
         partial_results = []
@@ -1008,8 +1046,9 @@ class UnitBase(object):
         # Do we have any minimal results?
         for final_result in final_results:
             if len(final_result):
-                cached_results[key] = final_result
-                return final_result
+                results = final_results[0].union(final_results[1])
+                cached_results[key] = results
+                return results
 
         partial_results.sort(key=lambda x: x[0])
 
@@ -1120,6 +1159,10 @@ class UnitBase(object):
                     elif unit._is_equivalent(tunit):
                         if has_bases_in_common(funit.decompose(), other):
                             return True
+                else:
+                    if unit._is_equivalent(funit):
+                        if has_bases_in_common(dimensionless_unscaled, other):
+                            return True
             return False
 
         def filter_units(units):
@@ -1275,12 +1318,15 @@ class UnitBase(object):
         units = set(unit_registry.get_units_with_physical_type(self))
         for funit, tunit, a, b in equivalencies:
             if tunit is not None:
-                if funit not in units:
-                    units.update(
-                        unit_registry.get_units_with_physical_type(funit))
-                if tunit not in units:
+                if self.is_equivalent(funit) and tunit not in units:
                     units.update(
                         unit_registry.get_units_with_physical_type(tunit))
+                if self._is_equivalent(tunit) and funit not in units:
+                    units.update(
+                        unit_registry.get_units_with_physical_type(funit))
+            else:
+                if self.is_equivalent(funit):
+                    units.add(dimensionless_unscaled)
         return units
 
     class EquivalentUnitsList(list):
@@ -1346,8 +1392,8 @@ class UnitBase(object):
         results = self.compose(
             equivalencies=equivalencies, units=units, max_depth=1,
             include_prefix_units=include_prefix_units)
-        results = [
-            x.bases[0] for x in results if len(x.bases) == 1]
+        results = set(
+            x.bases[0] for x in results if len(x.bases) == 1)
         return self.EquivalentUnitsList(results)
 
     def is_unity(self):
@@ -1624,7 +1670,7 @@ class UnrecognizedUnit(IrreducibleUnit):
     if six.PY3:
         __str__ = __unicode__
 
-    def to_string(self, format='generic'):
+    def to_string(self, format=unit_format.Generic):
         return self.name
 
     def _unrecognized_operator(self, *args, **kwargs):
@@ -1647,7 +1693,7 @@ class UnrecognizedUnit(IrreducibleUnit):
         self._normalize_equivalencies(equivalencies)
         return self == other
 
-    def get_converter(self, other, equivalencies=None):
+    def _get_converter(self, other, equivalencies=None):
         self._normalize_equivalencies(equivalencies)
         raise ValueError(
             "The unit {0!r} is unrecognized.  It can not be converted "
@@ -1713,7 +1759,7 @@ class _UnitMetaClass(InheritDocstrings):
                 return dimensionless_unscaled
 
             if format is None:
-                format = 'generic'
+                format = unit_format.Generic
 
             f = unit_format.get_format(format)
             if six.PY3 and isinstance(s, bytes):
@@ -1725,8 +1771,10 @@ class _UnitMetaClass(InheritDocstrings):
                 if parse_strict == 'silent':
                     pass
                 else:
-                    if format != 'generic':
-                        format_clause = format + ' '
+                    # Deliberately not isinstance here. Subclasses
+                    # should use their name.
+                    if type(f) is not unit_format.Generic:
+                        format_clause = f.name + ' '
                     else:
                         format_clause = ''
                     msg = ("'{0}' did not parse as {1}unit: {2}"
@@ -1975,7 +2023,7 @@ class CompositeUnit(UnitBase):
                         break
 
             if unit in new_parts:
-                new_parts[unit] += power
+                new_parts[unit] = add_powers(new_parts[unit], power)
             else:
                 new_parts[unit] = power
             return scale
@@ -2062,6 +2110,16 @@ si_prefixes = [
 ]
 
 
+binary_prefixes = [
+    (['Ki'], ['kibi'], 2. ** 10),
+    (['Mi'], ['mebi'], 2. ** 20),
+    (['Gi'], ['gibi'], 2. ** 30),
+    (['Ti'], ['tebi'], 2. ** 40),
+    (['Pi'], ['pebi'], 2. ** 50),
+    (['Ei'], ['exbi'], 2. ** 60)
+]
+
+
 def _add_prefixes(u, excludes=[], namespace=None, prefixes=False):
     """
     Set up all of the standard metric prefixes for a unit.  This
@@ -2180,7 +2238,7 @@ def def_unit(s, represents=None, register=None, doc=None,
     """
     if register is not None:
         warnings.warn(
-            "The registry kwarg was removed in astropy 0.3. "
+            "The register kwarg was removed in astropy 0.3. "
             "Use the namespace kwarg to inject the unit into "
             "a namespace and add_enabled_units() to enable it "
             "in the global unit registry.",
