@@ -13,11 +13,11 @@ import decimal
 import numpy as np
 from numpy.testing import (assert_allclose, assert_array_equal,
                            assert_array_almost_equal)
-from distutils import version
-NUMPY_VERSION = version.LooseVersion(np.__version__)
+
 
 from ...tests.helper import raises, pytest
-from ...utils import isiterable
+from ...utils import isiterable, minversion
+from ...utils.compat import NUMPY_LT_1_7
 from ... import units as u
 from ...units.quantity import _UNIT_NOT_INITIALISED
 from ...extern.six.moves import xrange
@@ -182,6 +182,40 @@ class TestQuantityCreation(object):
         q3 = u.Quantity(q1, u.m, ndmin=3)
         assert q3.ndim == 3 and q3.shape == (1, 1, 10)
 
+    def test_non_quantity_with_unit(self):
+        """Test that unit attributes in objects get recognized."""
+        class MyQuantityLookalike(np.ndarray):
+            pass
+
+        a = np.arange(3.)
+        mylookalike = a.copy().view(MyQuantityLookalike)
+        mylookalike.unit = 'm'
+        q1 = u.Quantity(mylookalike)
+        assert isinstance(q1, u.Quantity)
+        assert q1.unit is u.m
+        assert np.all(q1.value == a)
+
+        q2 = u.Quantity(mylookalike, u.mm)
+        assert q2.unit is u.mm
+        assert np.all(q2.value == 1000.*a)
+
+        q3 = u.Quantity(mylookalike, copy=False)
+        assert np.all(q3.value == mylookalike)
+        q3[2] = 0
+        assert q3[2] == 0.
+        assert mylookalike[2] == 0.
+
+        mylookalike = a.copy().view(MyQuantityLookalike)
+        mylookalike.unit = u.m
+        q4 = u.Quantity(mylookalike, u.mm, copy=False)
+        q4[2] = 0
+        assert q4[2] == 0.
+        assert mylookalike[2] == 2.
+
+        mylookalike.unit = 'nonsense'
+        with pytest.raises(TypeError):
+            u.Quantity(mylookalike)
+
 
 class TestQuantityOperations(object):
     q1 = u.Quantity(11.42, u.meter)
@@ -320,6 +354,17 @@ class TestQuantityOperations(object):
         with pytest.raises(u.UnitsError):
             new_q = q1 + q2
 
+    def test_non_number_type(self):
+        q1 = u.Quantity(11.412, unit=u.meter)
+        type_err_msg = ("Unsupported operand type(s) for ufunc add: "
+                        "'Quantity' and 'dict'")
+        with pytest.raises(TypeError) as exc:
+            q1 + {'a': 1}
+        assert exc.value.args[0] == type_err_msg
+
+        with pytest.raises(TypeError):
+            q1 + u.meter
+
     def test_dimensionless_operations(self):
         # test conversion to dimensionless
         dq = 3. * u.m / u.km
@@ -627,6 +672,10 @@ class TestQuantityDisplay(object):
         assert str(self.scalarintq * q3) == "1"
         assert str(self.arrq * q2) == "[ 1.   2.3  8.9]"
 
+    def test_dimensionless_quantity_format(self):
+        q1 = u.Quantity(3.14)
+        assert format(q1, '.2f') == '3.14'
+
     def test_scalar_quantity_str(self):
         assert str(self.scalarintq) == "1 m"
         assert str(self.scalarfloatq) == "1.3 m"
@@ -652,13 +701,62 @@ class TestQuantityDisplay(object):
         assert repr(bad_quantity).endswith(_UNIT_NOT_INITIALISED + '>')
 
     def test_repr_latex(self):
-        q2 = u.Quantity(1.5e14, 'm/s')
+        from ...units.quantity import conf
+
+        q2scalar = u.Quantity(1.5e14, 'm/s')
         assert self.scalarintq._repr_latex_() == '$1 \\; \\mathrm{m}$'
         assert self.scalarfloatq._repr_latex_() == '$1.3 \\; \\mathrm{m}$'
-        assert (q2._repr_latex_() ==
-                '$1.5\\times 10^{+14} \\; \\mathrm{\\frac{m}{s}}$')
-        with pytest.raises(NotImplementedError):
-            self.arrq._repr_latex_()
+        assert (q2scalar._repr_latex_() ==
+                '$1.5 \\times 10^{14} \\; \\mathrm{\\frac{m}{s}}$')
+
+        if NUMPY_LT_1_7:
+            with pytest.raises(NotImplementedError):
+                self.arrq._repr_latex_()
+            return  # all arrays should fail
+
+        assert self.arrq._repr_latex_() == '$[1,~2.3,~8.9] \; \mathrm{m}$'
+
+        qmed = np.arange(100)*u.m
+        qbig = np.arange(1000)*u.m
+        qvbig = np.arange(10000)*1e9*u.m
+
+        pops = np.get_printoptions()
+        oldlat = conf.latex_array_threshold
+        try:
+            #check thresholding behavior
+            conf.latex_array_threshold = 100  # should be default
+            lsmed = qmed._repr_latex_()
+            assert r'\dots' not in lsmed
+            lsbig = qbig._repr_latex_()
+            assert r'\dots' in lsbig
+            lsvbig = qvbig._repr_latex_()
+            assert r'\dots' in lsvbig
+
+            conf.latex_array_threshold = 1001
+            lsmed = qmed._repr_latex_()
+            assert r'\dots' not in lsmed
+            lsbig = qbig._repr_latex_()
+            assert r'\dots' not in lsbig
+            lsvbig = qvbig._repr_latex_()
+            assert r'\dots' in lsvbig
+
+
+            conf.latex_array_threshold = -1  # means use the numpy threshold
+            np.set_printoptions(threshold=99)
+            lsmed = qmed._repr_latex_()
+            assert r'\dots' in lsmed
+            lsbig = qbig._repr_latex_()
+            assert r'\dots' in lsbig
+            lsvbig = qvbig._repr_latex_()
+            assert r'\dots' in lsvbig
+        finally:
+            # prevent side-effects from influencing other tests
+            np.set_printoptions(**pops)
+            conf.latex_array_threshold = oldlat
+
+        qinfnan = [np.inf, -np.inf, np.nan] * u.m
+        assert qinfnan._repr_latex_() == r'$[\infty,~-\infty,~{\rm NaN}] \; \mathrm{m}$'
+
 
 
 def test_decompose():
@@ -897,9 +995,6 @@ def test_copy():
     assert q1.dtype == q2.dtype
     assert q1.value is not q2.value
 
-    if NUMPY_VERSION < version.LooseVersion('1.6.0'):
-        return  # numpy 1.5 doesn't allow arguments to `copy`
-
     q3 = q1.copy(order='F')
     assert q3.flags['F_CONTIGUOUS']
     assert np.all(q1.value == q3.value)
@@ -1022,3 +1117,41 @@ def test_quantity_from_table():
     qbp = u.Quantity(t['b'], u.pc)
     assert qbp.unit == u.pc
     assert_array_equal(qbp.value, t['b'])
+
+def test_insert():
+    """
+    Test Quantity.insert method.  This does not test the full capabilities
+    of the underlying np.insert, but hits the key functionality for
+    Quantity.
+    """
+    q = [1, 2] * u.m
+
+    # Insert a compatible float with different units
+    q2 = q.insert(0, 1 * u.km)
+    assert np.all(q2.value == [ 1000,  1,  2])
+    assert q2.unit is u.m
+    assert q2.dtype.kind == 'f'
+
+    if minversion(np, '1.8.0'):
+        q2 = q.insert(1, [1, 2] * u.km)
+        assert np.all(q2.value == [1, 1000, 2000, 2])
+        assert q2.unit is u.m
+
+    # Cannot convert 1.5 * u.s to m
+    with pytest.raises(u.UnitsError):
+        q.insert(1, 1.5 * u.s)
+
+    # Tests with multi-dim quantity
+    q = [[1, 2], [3, 4]] * u.m
+    q2 = q.insert(1, [10, 20] * u.m, axis=0)
+    assert np.all(q2.value == [[  1,  2],
+                               [ 10, 20],
+                               [  3,  4]])
+
+    q2 = q.insert(1, [10, 20] * u.m, axis=1)
+    assert np.all(q2.value == [[  1, 10,  2],
+                               [  3, 20,  4]])
+
+    q2 = q.insert(1, 10 * u.m, axis=1)
+    assert np.all(q2.value == [[  1,  10, 2],
+                               [  3,  10, 4]])
