@@ -62,7 +62,8 @@ from ..extern import six
 from ..extern.six.moves import range
 from .. import conf
 
-from .misc import deprecated, isiterable
+from .decorators import deprecated
+from .misc import isiterable
 
 
 __all__ = [
@@ -81,7 +82,7 @@ _DEFAULT_ENCODING = 'utf-8'
 
 def _get_stdout(stderr=False):
     """
-    This utility fucntion contains the logic to determine what streams to use
+    This utility function contains the logic to determine what streams to use
     by default for standard out/err.
 
     Typically this will just return `sys.stdout`, but it contains additional
@@ -430,7 +431,7 @@ def human_file_size(size):
     size : str
         A human-friendly representation of the size of the file
     """
-    suffixes = ' kMGTPEH'
+    suffixes = ' kMGTPEZY'
     if size == 0:
         num_scale = 0
     else:
@@ -442,7 +443,9 @@ def human_file_size(size):
     num_scale = int(math.pow(1000, num_scale))
     value = size / num_scale
     str_value = str(value)
-    if str_value[2] == '.':
+    if suffix == ' ':
+        str_value = str_value[:str_value.index('.')]
+    elif str_value[2] == '.':
         str_value = str_value[:2]
     else:
         str_value = str_value[:3]
@@ -464,7 +467,7 @@ class ProgressBar(six.Iterator):
         for item in ProgressBar(items):
             item.process()
     """
-    def __init__(self, total_or_items, file=None):
+    def __init__(self, total_or_items, ipython_widget=False, file=None):
         """
         Parameters
         ----------
@@ -472,6 +475,10 @@ class ProgressBar(six.Iterator):
             If an int, the number of increments in the process being
             tracked.  If a sequence, the items to iterate over.
 
+        ipython_widget : bool, optional
+            If `True`, the progress bar will display as an IPython
+            notebook widget.
+
         file : writable file-like object, optional
             The file to write the progress bar to.  Defaults to
             `sys.stdout`.  If `file` is not a tty (as determined by
@@ -480,10 +487,16 @@ class ProgressBar(six.Iterator):
             completely silent.
         """
 
+        if ipython_widget:
+            # Import only if ipython_widget, i.e., widget in IPython
+            # notebook
+            from IPython.html import widgets
+            from IPython.display import display
+
         if file is None:
             file = _get_stdout()
 
-        if not isatty(file):
+        if not isatty(file) and not ipython_widget:
             self.update = self._silent_update
             self._silent = True
         else:
@@ -502,17 +515,19 @@ class ProgressBar(six.Iterator):
 
         self._file = file
         self._start_time = time.time()
-
-        self._should_handle_resize = (
-            _CAN_RESIZE_TERMINAL and self._file.isatty())
-        self._handle_resize()
-        if self._should_handle_resize:
-            signal.signal(signal.SIGWINCH, self._handle_resize)
-            self._signal_set = True
-        else:
-            self._signal_set = False
-
         self._human_total = human_file_size(self._total)
+        self._ipython_widget = ipython_widget
+
+
+        self._signal_set = False
+        if not ipython_widget:
+            self._should_handle_resize = (
+                _CAN_RESIZE_TERMINAL and self._file.isatty())
+            self._handle_resize()
+            if self._should_handle_resize:
+                signal.signal(signal.SIGWINCH, self._handle_resize)
+                self._signal_set = True
+
         self.update(0)
 
     def _handle_resize(self, signum=None, frame=None):
@@ -546,14 +561,27 @@ class ProgressBar(six.Iterator):
 
 
     def update(self, value=None):
+        """
+        Update progress bar via the console or notebook accordingly.
+        """
+
+        # Update self.value
+        if value is None:
+            value = self._current_value + 1
+        self._current_value = value
+
+        # Choose the appropriate environment
+        if self._ipython_widget:
+            self._update_ipython_widget(value)
+        else:
+            self._update_console(value)
+
+    def _update_console(self, value=None):
         """
         Update the progress bar to the given value (out of the total
         given to the constructor).
         """
-        if value is None:
-            value = self._current_value = self._current_value + 1
-        else:
-            self._current_value = value
+
         if self._total == 0:
             frac = 1.0
         else:
@@ -591,6 +619,31 @@ class ProgressBar(six.Iterator):
             write(human_time(t))
         self._file.flush()
 
+    def _update_ipython_widget(self, value=None):
+        """
+        Update the progress bar to the given value (out of a total
+        given to the constructor).
+
+        This method is for use in the IPython notebook 2+.
+        """
+
+        # Create and display an empty progress bar widget,
+        # if none exists.
+        if not hasattr(self, '_widget'):
+            # Import only if an IPython widget, i.e., widget in iPython NB
+            from IPython.html import widgets
+            from IPython.display import display
+
+            self._widget = widgets.FloatProgressWidget()
+            display(self._widget)
+            self._widget.value = 0
+
+        # Calculate percent completion, and update progress bar
+        percent = (value/self._total) * 100
+        self._widget.value = percent
+        self._widget.description =' ({0:>6s}%)'.format('{0:.2f}'.format(percent))
+
+
     def _silent_update(self, value=None):
         pass
 
@@ -651,40 +704,6 @@ class ProgressBar(six.Iterator):
 
         return results
 
-    @deprecated('0.3', alternative='ProgressBar')
-    @classmethod
-    def iterate(cls, items, file=None):
-        """
-        Iterate over a sequence while indicating progress with a progress
-        bar in the terminal.
-
-        ::
-
-            for item in ProgressBar.iterate(items):
-                pass
-
-        Parameters
-        ----------
-        items : sequence
-            A sequence of items to iterate over
-
-        file : writeable file-like object, optional
-            The file to write the progress bar to.  Defaults to
-            `sys.stdout`.  If `file` is not a tty (as determined by
-            calling its `isatty` member, if any), the scrollbar will
-            be completely silent.
-
-        Returns
-        -------
-        generator :
-            A generator over ``items``.
-        """
-
-        if file is None:
-            file = _get_stdout()
-
-        return cls(items, file=file)
-
 
 class Spinner(object):
     """
@@ -984,8 +1003,10 @@ class _GetchUnix(object):
     def __init__(self):
         import tty
         import sys
-        import termios  # import termios now or else you'll get the Unix
-                        # version on the Mac
+
+        # import termios now or else you'll get the Unix
+        # version on the Mac
+        import termios
 
     def __call__(self):
         import sys
