@@ -85,13 +85,13 @@ class SAMPClient(object):
         passed from the Hub end of the connection.
 
     ssl_version : int, optional
-        Which version of the SSL protocol to use. Typically, the server
-        chooses a particular protocol version, and the client must adapt to
-        the server's choice. Most of the versions are not interoperable with
-        the other versions. If not specified the default SSL version is
-        `ssl.PROTOCOL_SSLv23`. This version provides the most compatibility
-        with other versions Hub side. Other SSL protocol versions are:
-        `ssl.PROTOCOL_SSLv2`, `ssl.PROTOCOL_SSLv3` and `ssl.PROTOCOL_TLSv1`.
+        Which version of the SSL protocol to use. Typically, the
+        server chooses a particular protocol version, and the client
+        must adapt to the server's choice. Most of the versions are
+        not interoperable with the other versions. If not specified,
+        the default SSL version is taken from the default in the
+        installed version of the Python standard `ssl` library.  See
+        the `ssl` documentation for more information.
 
     callable : bool, optional
         Whether the client can receive calls and notifications. If set to
@@ -118,9 +118,6 @@ class SAMPClient(object):
         if description is not None:
             metadata["samp.description.text"] = description
 
-        if SSL_SUPPORT and ssl_version is None:
-            ssl_version = ssl.PROTOCOL_SSLv23
-
         self._metadata = metadata
 
         self._addr = addr
@@ -365,7 +362,7 @@ class SAMPClient(object):
 
         Returns
         -------
-        confimation : str
+        confirmation : str
             Any confirmation string.
         """
         return self._handle_call(private_key, sender_id, msg_id, message)
@@ -474,7 +471,7 @@ class SAMPClient(object):
         Parameters
         ----------
         mtype : str
-            MType to be catched.
+            MType to be caught.
 
         function : callable
             Application function to be used when ``mtype`` is received.
@@ -517,7 +514,7 @@ class SAMPClient(object):
         Parameters
         ----------
         mtype : str
-            MType to be catched.
+            MType to be caught.
 
         function : callable
             Application function to be used when ``mtype`` is received.
@@ -557,7 +554,7 @@ class SAMPClient(object):
         Parameters
         ----------
         msg_tag : str
-            Message-tag to be catched.
+            Message-tag to be caught.
 
         function : callable
             Application function to be used when ``msg_tag`` is received.
@@ -601,7 +598,7 @@ class SAMPClient(object):
 
         declare : bool
             Specify whether the client must be automatically declared as
-            unsubscribed from the MType (see alse
+            unsubscribed from the MType (see also
             :meth:`~astropy.vo.samp.client.SAMPClient.declare_subscriptions`).
         """
         if self._callable:
@@ -661,11 +658,11 @@ class SAMPClient(object):
             result = self.hub.register(self.hub.lockfile["samp.secret"])
 
             if result["samp.self-id"] == "":
-                raise SAMPClientError("Registation failed - "
+                raise SAMPClientError("Registration failed - "
                                       "samp.self-id was not set by the hub.")
 
             if result["samp.private-key"] == "":
-                raise SAMPClientError("Registation failed - "
+                raise SAMPClientError("Registration failed - "
                                       "samp.private-key was not set by the hub.")
 
             self._public_id = result["samp.self-id"]
@@ -734,7 +731,7 @@ class SAMPClient(object):
         Parameters
         ----------
         metadata : dict, optional
-            Dictionary containig the client application metadata as defined in
+            Dictionary containing the client application metadata as defined in
             the SAMP definition document. If omitted, then no metadata are
             declared.
         """
