@@ -375,9 +375,10 @@ PyWcsprm_init(
       return -1;
     }
 
-    if (convert_rejections_to_warnings(wcsprintf_buf())) {
+    wcsvfree(&nwcs, &wcs);
+
+    if (convert_rejections_to_warnings()) {
       free(colsel_ints);
-      wcsvfree(&nwcs, &wcs);
       return -1;
     }
 
@@ -413,6 +414,7 @@ PyWcsprm_init(
     }
 
     if (nwcs == 0) {
+      wcsvfree(&nwcs, &wcs);
       PyErr_SetString(
           WcsExc_NoWcsKeywordsFound,
           "No WCS keywords found in the given header");
@@ -450,6 +452,37 @@ PyWcsprm_init(
   }
 }
 
+/*@null@*/ static PyObject*
+PyWcsprm_bounds_check(
+    PyWcsprm* self,
+    PyObject* args,
+    PyObject* kwds) {
+
+  unsigned char pix2sky    = 1;
+  unsigned char sky2pix    = 1;
+  int           bounds     = 0;
+  const char*   keywords[] = {"pix2world", "world2pix", NULL};
+
+  if (!PyArg_ParseTupleAndKeywords(
+          args, kwds, "|bb:bounds_check", (char **)keywords,
+          &pix2sky, &sky2pix)) {
+    return NULL;
+  }
+
+  if (pix2sky) {
+      bounds |= 2|4;
+  }
+
+  if (sky2pix) {
+      bounds |= 1;
+  }
+
+  wcsbchk(&self->x, bounds);
+
+  Py_RETURN_NONE;
+}
+
+
 /*@null@*/ static PyObject*
 PyWcsprm_copy(
     PyWcsprm* self) {
@@ -462,6 +495,8 @@ PyWcsprm_copy(
     return NULL;
   }
 
+  wcsini(0, self->x.naxis, &copy->x);
+
   wcsprm_python2c(&self->x);
   status = wcscopy(1, &self->x, &copy->x);
   wcsprm_c2python(&self->x);
@@ -581,8 +616,9 @@ PyWcsprm_find_all_wcs(
     return NULL;
   }
 
-  if (convert_rejections_to_warnings(wcsprintf_buf())) {
-    wcsvfree(&nwcs, &wcs);
+  wcsvfree(&nwcs, &wcs);
+
+  if (convert_rejections_to_warnings()) {
     return NULL;
   }
 
@@ -693,6 +729,45 @@ PyWcsprm_celfix(
   }
 }
 
+static PyObject *
+PyWcsprm_compare(
+    PyWcsprm* self,
+    PyObject* args,
+    PyObject* kwds) {
+
+  int cmp = 0;
+  PyWcsprm *other;
+  double tolerance = 0.0;
+  int equal;
+  int status;
+
+  const char* keywords[] = {"other", "cmp", "tolerance", NULL};
+
+  if (!PyArg_ParseTupleAndKeywords(
+          args, kwds, "O!|id:compare", (char **)keywords,
+          &PyWcsprmType, &other, &cmp, &tolerance)) {
+    return NULL;
+  }
+
+
+  wcsprm_python2c(&self->x);
+  wcsprm_python2c(&other->x);
+  status = wcscompare(cmp, tolerance, &self->x, &other->x, &equal);
+  wcsprm_c2python(&self->x);
+  wcsprm_c2python(&other->x);
+
+  if (status) {
+    wcserr_fix_to_python_exc(self->x.err);
+    return NULL;
+  } else {
+    if (equal) {
+      Py_RETURN_TRUE;
+    } else {
+      Py_RETURN_FALSE;
+    }
+  }
+}
+
 /*@null@*/ static PyObject*
 PyWcsprm_cylfix(
     PyWcsprm* self,
@@ -711,7 +786,7 @@ PyWcsprm_cylfix(
     return NULL;
   }
 
-  if (naxis_obj != NULL) {
+  if (naxis_obj != NULL && naxis_obj != Py_None) {
     naxis_array = (PyArrayObject*)PyArray_ContiguousFromAny(
         naxis_obj, 1, 1, PyArray_INT);
     if (naxis_array == NULL) {
@@ -814,7 +889,7 @@ PyWcsprm_fix(
     }
   }
 
-  if (naxis_obj != NULL) {
+  if (naxis_obj != NULL && naxis_obj != Py_None) {
     naxis_array = (PyArrayObject*)PyArray_ContiguousFromAny(
         naxis_obj, 1, 1, PyArray_INT);
     if (naxis_array == NULL) {
@@ -1382,7 +1457,7 @@ PyWcsprm_s2p(
   /* unoffset_array(world, origin); */
   unoffset_array(pixcrd, origin);
   unoffset_array(imgcrd, origin);
-  if (status == 8) {
+  if (status == 9) {
     set_invalid_to_nan(
         ncoord, 1, (double*)PyArray_DATA(phi), (int*)PyArray_DATA(stat));
     set_invalid_to_nan(
@@ -1608,6 +1683,45 @@ PyWcsprm___str__(
   #endif
 }
 
+PyObject *PyWcsprm_richcompare(PyObject *a, PyObject *b, int op) {
+  int equal;
+  int status;
+
+  struct wcsprm *ax;
+  struct wcsprm *bx;
+
+  if ((op == Py_EQ || op == Py_NE) &&
+      PyObject_TypeCheck(b, &PyWcsprmType)) {
+    ax = &((PyWcsprm *)a)->x;
+    bx = &((PyWcsprm *)b)->x;
+
+    wcsprm_python2c(ax);
+    wcsprm_python2c(bx);
+    status = wcscompare(
+        WCSCOMPARE_ANCILLARY, 0.0,
+        ax, bx, &equal);
+    wcsprm_c2python(ax);
+    wcsprm_c2python(bx);
+
+    if (status == 0) {
+      if (op == Py_NE) {
+        equal = !equal;
+      }
+      if (equal) {
+        Py_RETURN_TRUE;
+      } else {
+        Py_RETURN_FALSE;
+      }
+    } else {
+      wcs_to_python_exc(&(((PyWcsprm *)a)->x));
+      return NULL;
+    }
+  }
+
+  Py_INCREF(Py_NotImplemented);
+  return Py_NotImplemented;
+}
+
 /*@null@*/ static PyObject*
 PyWcsprm_sub(
     PyWcsprm* self,
@@ -1619,13 +1733,10 @@ PyWcsprm_sub(
   PyObject*  py_axes        = NULL;
   PyWcsprm*  py_dest_wcs    = NULL;
   PyObject*  element        = NULL;
-  #if PY3K
   PyObject*  element_utf8   = NULL;
-  #endif
   char*      element_str    = NULL;
   int        element_val    = 0;
   int        nsub           = 0;
-  int        alloc_size     = 0;
   int*       axes           = NULL;
   int        status         = -1;
   const char*    keywords[] = {"axes", NULL};
@@ -1638,14 +1749,14 @@ PyWcsprm_sub(
 
   if (py_axes == NULL || py_axes == Py_None) {
     /* leave all variables as is */
-  } else if (PySequence_Check(py_axes)) {
+  } else if (PyList_Check(py_axes) || PyTuple_Check(py_axes)) {
     tmp = PySequence_Size(py_axes);
     if (tmp == -1) {
       goto exit;
     }
     nsub = (int)tmp;
 
-    axes = malloc(nsub * sizeof(int));
+    axes = malloc(nsub * sizeof(int) * 2);
     if (axes == NULL) {
       PyErr_SetString(PyExc_MemoryError, "Out of memory");
       goto exit;
@@ -1657,19 +1768,19 @@ PyWcsprm_sub(
         goto exit;
       }
 
-      #if PY3K
-      if (PyUnicode_Check(element)) {
-        element_utf8 = PyUnicode_AsUTF8String(element);
-        if (element_utf8 == NULL) {
-          goto exit;
+      if (PyUnicode_Check(element) || PyBytes_Check(element)) {
+        if (PyUnicode_Check(element)) {
+          element_utf8 = PyUnicode_AsUTF8String(element);
+          if (element_utf8 == NULL) {
+            goto exit;
+          }
+
+          element_str = PyBytes_AsString(element_utf8);
+          Py_DECREF(element_utf8); element_utf8 = NULL;
+        } else if (PyBytes_Check(element)) {
+          element_str = PyBytes_AsString(element);
         }
-        element_str = PyBytes_AsString(element_utf8);
-        Py_DECREF(element_utf8); element_utf8 = NULL;
-      #else
-      if (PyString_Check(element)) {
-        /* Doesn't return NULL, because we already known it's a string */
-        element_str = PyString_AsString(element);
-      #endif
+
         if (strncmp(element_str, "longitude", 10) == 0) {
           element_val = WCSSUB_LONGITUDE;
         } else if (strncmp(element_str, "latitude", 9) == 0) {
@@ -1737,21 +1848,15 @@ PyWcsprm_sub(
     goto exit;
   }
 
-  if (nsub == 0) {
-    alloc_size = self->x.naxis;
-  } else {
-    alloc_size = nsub;
-  }
-
   py_dest_wcs = (PyWcsprm*)PyWcsprm_cnew();
   py_dest_wcs->x.flag = -1;
-  status = wcsini(1, alloc_size, &py_dest_wcs->x);
+  status = wcsini(0, nsub, &py_dest_wcs->x);
   if (status != 0) {
     goto exit;
   }
 
   wcsprm_python2c(&self->x);
-  status = wcssub(0, &self->x, &nsub, axes, &py_dest_wcs->x);
+  status = wcssub(1, &self->x, &nsub, axes, &py_dest_wcs->x);
   wcsprm_c2python(&self->x);
   if (PyWcsprm_cset(py_dest_wcs, 0)) {
     goto exit;
@@ -1825,9 +1930,7 @@ PyWcsprm_to_header(
   wcsprm_c2python(&self->x);
 
   if (status != 0) {
-    PyErr_SetString(
-        PyExc_RuntimeError,
-        "Unknown error occurred.  Something is seriously wrong.");
+    wcs_to_python_exc(&(self->x));
     goto exit;
   }
 
@@ -3248,8 +3351,10 @@ static PyGetSetDef PyWcsprm_getset[] = {
 };
 
 static PyMethodDef PyWcsprm_methods[] = {
+  {"bounds_check", (PyCFunction)PyWcsprm_bounds_check, METH_VARARGS|METH_KEYWORDS, doc_bounds_check},
   {"cdfix", (PyCFunction)PyWcsprm_cdfix, METH_NOARGS, doc_cdfix},
   {"celfix", (PyCFunction)PyWcsprm_celfix, METH_NOARGS, doc_celfix},
+  {"compare", (PyCFunction)PyWcsprm_compare, METH_VARARGS|METH_KEYWORDS, doc_compare},
   {"__copy__", (PyCFunction)PyWcsprm_copy, METH_NOARGS, doc_copy},
   {"cylfix", (PyCFunction)PyWcsprm_cylfix, METH_VARARGS|METH_KEYWORDS, doc_cylfix},
   {"datfix", (PyCFunction)PyWcsprm_datfix, METH_NOARGS, doc_datfix},
@@ -3310,7 +3415,7 @@ PyTypeObject PyWcsprmType = {
   doc_Wcsprm,                   /* tp_doc */
   0,                            /* tp_traverse */
   0,                            /* tp_clear */
-  0,                            /* tp_richcompare */
+  PyWcsprm_richcompare,         /* tp_richcompare */
   0,                            /* tp_weaklistoffset */
   0,                            /* tp_iter */
   0,                            /* tp_iternext */
@@ -3329,6 +3434,9 @@ PyTypeObject PyWcsprmType = {
 
 #define CONSTANT(a) PyModule_AddIntConstant(m, #a, a)
 
+#define XSTRINGIFY(s) STRINGIFY(s)
+#define STRINGIFY(s) #s
+
 int
 _setup_wcsprm_type(
     PyObject* m) {
@@ -3376,5 +3484,8 @@ _setup_wcsprm_type(
     CONSTANT(WCSHDO_PVn_ma)    ||
     CONSTANT(WCSHDO_CRPXna)    ||
     CONSTANT(WCSHDO_CNAMna)    ||
-    CONSTANT(WCSHDO_WCSNna));
+    CONSTANT(WCSHDO_WCSNna)    ||
+    CONSTANT(WCSCOMPARE_ANCILLARY) ||
+    CONSTANT(WCSCOMPARE_TILING) ||
+    CONSTANT(WCSCOMPARE_CRPIX));
 }
