@@ -1,7 +1,7 @@
 /*============================================================================
 
-  WCSLIB 4.19 - an implementation of the FITS WCS standard.
-  Copyright (C) 1995-2013, Mark Calabretta
+  WCSLIB 4.25 - an implementation of the FITS WCS standard.
+  Copyright (C) 1995-2014, Mark Calabretta
 
   This file is part of WCSLIB.
 
@@ -22,7 +22,7 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: wcsutil.h,v 4.19 2013/09/29 14:17:51 mcalabre Exp $
+  $Id: wcsutil.h,v 4.25 2014/12/14 14:29:36 mcalabre Exp $
 *=============================================================================
 *
 * Summary of the wcsutil routines
@@ -105,6 +105,74 @@
 *                         1: All equal.
 *
 *
+* wcsutil_Eq() - Test for equality of two double arrays
+* -----------------------------------------------------
+* INTERNAL USE ONLY.
+*
+* wcsutil_Eq() tests for equality of two double-precision arrays.
+*
+* Given:
+*   nelem     int       The number of elements in each array.
+*
+*   tol       double    Tolerance for comparison of the floating-point values.
+*                       For example, for tol == 1e-6, all floating-point
+*                       values in the arrays must be equal to the first 6
+*                       decimal places.  A value of 0 implies exact equality.
+*
+*   arr1      const double*
+*                       The first array.
+*
+*   arr2      const double*
+*                       The second array
+*
+* Function return value:
+*             int       Status return value:
+*                         0: Not equal.
+*                         1: Equal.
+*
+*
+* wcsutil_intEq() - Test for equality of two int arrays
+* -----------------------------------------------------
+* INTERNAL USE ONLY.
+*
+* wcsutil_intEq() tests for equality of two int arrays.
+*
+* Given:
+*   nelem     int       The number of elements in each array.
+*
+*   arr1      const int*
+*                       The first array.
+*
+*   arr2      const int*
+*                       The second array
+*
+* Function return value:
+*             int       Status return value:
+*                         0: Not equal.
+*                         1: Equal.
+*
+*
+* wcsutil_strEq() - Test for equality of two string arrays
+* --------------------------------------------------------
+* INTERNAL USE ONLY.
+*
+* wcsutil_strEq() tests for equality of two string arrays.
+*
+* Given:
+*   nelem     int       The number of elements in each array.
+*
+*   arr1      const char**
+*                       The first array.
+*
+*   arr2      const char**
+*                       The second array
+*
+* Function return value:
+*             int       Status return value:
+*                         0: Not equal.
+*                         1: Equal.
+*
+*
 * wcsutil_setAll() - Set a particular vector element
 * --------------------------------------------------
 * INTERNAL USE ONLY.
@@ -211,7 +279,9 @@
 * INTERNAL USE ONLY.
 *
 * wcsutil_double2str() converts a double to a string, but unlike sprintf() it
-* ignores the locale and always uses a '.' as the decimal separator.
+* ignores the locale and always uses a '.' as the decimal separator.  Also,
+* unless it includes an exponent, the formatted value will always have a
+* fractional part, ".0" being appended if necessary.
 *
 * Returned:
 *   buf       char *    The buffer to write the string into.
@@ -220,7 +290,10 @@
 *   format    char *    The formatting directive, such as "%f".  This
 *                       may be any of the forms accepted by sprintf(), but
 *                       should only include a formatting directive and
-*                       nothing else.
+*                       nothing else.  For "%g" and "%G" formats, unless it
+*                       includes an exponent, the formatted value will always
+*                       have a fractional part, ".0" being appended if
+*                       necessary.
 *
 *   value     double    The value to convert to a string.
 *
@@ -255,6 +328,10 @@ void wcsutil_blank_fill(int n, char c[]);
 void wcsutil_null_fill (int n, char c[]);
 
 int  wcsutil_allEq (int nvec, int nelem, const double *first);
+int  wcsutil_Eq(int nelem, double tol, const double *arr1,
+                const double *arr2);
+int  wcsutil_intEq(int nelem, const int *arr1, const int *arr2);
+int  wcsutil_strEq(int nelem, char (*arr1)[72], char (*arr2)[72]);
 void wcsutil_setAll(int nvec, int nelem, double *first);
 void wcsutil_setAli(int nvec, int nelem, int *first);
 void wcsutil_setBit(int nelem, const int *sel, int bits, int *array);
